package com.asda.android.app.account.view

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.method.PasswordTransformationMethod
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.annotation.StringRes
import androidx.lifecycle.ViewModelProvider
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia.PV_CHANGE_PASS_WORD
import com.asda.android.analytics.constants.Anivia.SECTION_ACCOUNT
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.FloatLabel
import com.asda.android.base.core.view.fragments.BaseEditFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.restapi.singleprofile.BasicInformation
import com.asda.android.restapi.singleprofile.ProfileResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.singleprofile.base.SingleProfile.authentication
import com.asda.android.singleprofile.viewmodel.PersonalDetailViewModel
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import com.jakewharton.rxbinding2.widget.RxTextView
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.functions.BiFunction
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class ChangePasswordFragment : BaseEditFragment() {
    private var mNewPasswordText: FloatLabel? = null
    private var mNewPassword: EditText? = null
    private var mCurrentPassword: EditText? = null
    private var mCurrentPasswordText: FloatLabel? = null
    private var mSaveButton: Button? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val pageViewEvent = PageViewEvent(PV_CHANGE_PASS_WORD, SECTION_ACCOUNT, SECTION_ACCOUNT)
        Tracker.get().trackPageView(pageViewEvent)
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.account_change_password, fragmentContainer, false)
        setupView(view)
        return view
    }

    private fun setupView(view: View) {
        val description = view.findViewById<View>(R.id.description) as TextView
        description.text = getString(R.string.min_password_warn, SharedPreferencesUtil.getPasswordMinLength(activity))
        mNewPassword = view.findViewById<View>(R.id.new_password) as EditText
        mNewPasswordText = view.findViewById<View>(R.id.new_password_text) as FloatLabel
        mNewPasswordText?.attach(mNewPassword)
        mCurrentPassword = view.findViewById<View>(R.id.current_password) as EditText
        mCurrentPasswordText = view.findViewById<View>(R.id.current_password_text) as FloatLabel
        mCurrentPasswordText?.attach(mCurrentPassword)
        mSaveButton = ViewUtil.findViewById(view, R.id.save_button)
        setupHidePasswordCheckbox(view.findViewById<View>(R.id.show_hide_current_password) as CheckBox, mCurrentPassword)
        setupHidePasswordCheckbox(view.findViewById<View>(R.id.show_hide_new_password) as CheckBox, mNewPassword)
        setupSaveButton(view, R.id.save_button)
        mCurrentPassword?.let {
            safeAdd(
                RxTextView.textChanges(it)
                    .doOnNext {
                        mCurrentPasswordText?.clearError()
                    }
                    .debounce(ERROR_DELAY.toLong(), TimeUnit.MILLISECONDS)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribeWith(object : ResourceObserver<CharSequence?>() {
                        override fun onNext(charSequence: CharSequence) {
                            if (charSequence.isEmpty()) {
                                displayErrorMessage(mCurrentPassword, mCurrentPasswordText, R.string.error_current_password, false)
                            } else {
                                mCurrentPasswordText?.clearError()
                            }
                        }

                        override fun onError(e: Throwable) {
                            dispose()
                        }

                        override fun onComplete() {
                            dispose()
                        }
                    })
            )
        }

        mNewPassword?.let {
            safeAdd(
                RxTextView.textChanges(it)
                    .doOnNext { charSequence: CharSequence? -> mNewPasswordText!!.clearError() }
                    .debounce(ERROR_DELAY.toLong(), TimeUnit.MILLISECONDS)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribeWith(object : ResourceObserver<CharSequence?>() {
                        override fun onNext(charSequence: CharSequence) {
                            if (!Utils.isValidPassword(charSequence) && charSequence.isNotEmpty()) {
                                displayErrorMessage(mNewPassword, mNewPasswordText, R.string.error_new_password, false)
                            } else {
                                mNewPasswordText?.clearError()
                            }
                        }

                        override fun onError(e: Throwable) {
                            dispose()
                        }

                        override fun onComplete() {
                            dispose()
                        }
                    })
            )
        }

        if (mCurrentPassword != null && mNewPassword != null) {
            safeAdd(
                Observable
                    .combineLatest(
                        RxTextView.textChanges(mCurrentPassword!!),
                        RxTextView.textChanges(mNewPassword!!),
                        BiFunction { currentPwd: CharSequence?, newPwd: CharSequence? ->
                            currentPwd.isNullOrEmpty().not() && Utils.isValidPassword(newPwd)
                        }
                    )
                    .startWith(false)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribeWith(object : ResourceObserver<Boolean?>() {
                        override fun onNext(isValid: Boolean) {
                            enableButton(activity, mSaveButton, isValid)
                        }

                        override fun onError(e: Throwable) {
                            dispose()
                        }

                        override fun onComplete() {
                            dispose()
                        }
                    })
            )
        }
    }

    override fun onSaveButtonClick() {
        if (!wereFieldsChanged()) {
            pop()
        } else {
            changePassword(mCurrentPassword?.text.toString(), mNewPassword?.text.toString())
        }
    }

    override fun wereFieldsChanged(): Boolean {
        return mNewPassword?.text.toString().isNotEmpty() || mCurrentPassword?.text.toString().isNotEmpty()
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    private fun displayErrorMessage(toRequestFocus: View?, errorLabel: FloatLabel?, @StringRes errorMsgId: Int, requestFocus: Boolean) {
        errorLabel?.setError(errorMsgId)
        if (requestFocus && toRequestFocus != null) {
            toRequestFocus.requestFocus()
            view?.scrollTo(toRequestFocus.left, toRequestFocus.top - 80)
        }
    }

    private fun changePassword(oldPassword: String, newPassword: String) {
        val context: Activity? = activity
        val pd = CustomProgressDialog.create(context)
        pd.setCancelable(false)
        pd.setMessage(resources.getString(R.string.saving_in_progress))
        pd.isIndeterminate = true
        pd.show()
        mCurrentPasswordText?.clearError()
        mNewPasswordText?.clearError()
        val profileResponse = ProfileResponse()
        profileResponse.basic = BasicInformation()
        if (authentication.profileResponse?.basic != null && profileResponse.basic != null) {
            profileResponse.basic?.newPassword = newPassword
            profileResponse.basic?.password = oldPassword
            profileResponse.basic?.email = null // Set as null, otherwise it will try to update email.
            profileResponse.profileId = authentication.profileResponse?.profileId
            val model = ViewModelProvider(this).get(PersonalDetailViewModel::class.java)
            val d: Disposable = model.updatePersonalDetail(profileResponse)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribeWith(object : ResourceObserver<ProfileResponse?>() {
                    override fun onNext(response: ProfileResponse) {
                        if (isAdded) {
                            try {
                                pd.dismiss()
                            } catch (ignored: Exception) {
                                // No op
                            }
                            if (response.isSuccess) {
                                Toast.makeText(context, R.string.password_saved, Toast.LENGTH_SHORT).show()
                                pop()
                            } else {
                                Toast.makeText(context, R.string.invalid_current_password, Toast.LENGTH_SHORT).show()
                            }
                        }
                    }

                    override fun onError(e: Throwable) {
                        if (isAdded) {
                            try {
                                pd.dismiss()
                            } catch (ignored: Exception) {
                                // No op
                            }
                            if (context != null) AsdaDialogHelper.displayErrorDialog(null, -1, context, "Change Password")
                        }
                        dispose()
                    }

                    override fun onComplete() {
                        dispose()
                    }
                })
            safeAdd(d)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        mCurrentPassword?.let {
            ViewUtil.hideKeyboard(it)
        }
        return super.onOptionsItemSelected(item)
    }

    override fun getActionBarTitle(): String? {
        return getString(R.string.account_setting_change_password)
    }

    private fun setupHidePasswordCheckbox(showHidePassword: CheckBox, passwordView: EditText?) {
        showHidePassword.setOnCheckedChangeListener { _: CompoundButton?, isChecked: Boolean ->
            if (isChecked) {
                showHidePassword.setText(R.string.hide)
                passwordView?.transformationMethod = null
            } else {
                showHidePassword.setText(R.string.show)
                passwordView?.transformationMethod = PasswordTransformationMethod.getInstance()
            }
        }
        showHidePassword.isChecked = false
    }

    private fun enableButton(context: Context?, button: Button?, enable: Boolean) {
        button?.isClickable = enable
        context?.let {
            button?.isEnabled = enable
        }
    }

    companion object {
        const val TAG = "ChangePasswordFragment"
        private const val ERROR_DELAY = 2000
        fun newInstance(): ChangePasswordFragment {
            return ChangePasswordFragment()
        }
    }
}
