package com.asda.android.app.bookslot.helper

import android.content.Context
import android.util.Log
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.ErrorCodes.ERROR_CODE_INVALID_POSTCODE
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.isBefore
import com.asda.android.base.core.utils.orFalse
import com.asda.android.base.interfaces.IBookSlotHelper
import com.asda.android.home.R
import com.asda.android.orders.wismo.constants.WismoCardConstants.FULFILLMENT_TYPE_CNC
import com.asda.android.orders.wismo.constants.WismoCardConstants.FULFILLMENT_TYPE_HD
import com.asda.android.orders.wismo.constants.WismoCardConstants.FULFILLMENT_TYPE_RCNC
import com.asda.android.restapi.service.constants.SlotConstants
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.restapi.service.data.bookslotv3.Error
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.restapi.service.data.cart.Cart
import com.asda.android.restapi.utils.SubscriptionUtil.getSubscriptionExpiryDate
import com.asda.android.service.constants.CartManagerConstants
import com.asda.android.singleprofile.validator.PostCodeValidator
import com.asda.android.slots.bookslot.bookslotv3.service.BookSlotRepository
import com.asda.android.slots.utils.SlotUtils
import io.reactivex.Single
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*

class BookSlotHelper : IBookSlotHelper {
    private val tag = "BookSlotHelper"
    private val inputDateFormatter: DateFormat = SimpleDateFormat("EEEE, d MMMM yyyy", Locale.UK)
    private val timeFormatter: DateFormat = SimpleDateFormat("h:mma", Locale.UK)
    private var mSlotWasBookedRecently = CartManagerConstants.SLOT_TYPE_NONE
    private val bookSlotRepository = BookSlotRepository()
    override fun getBookedUntilMessage(context: Context, cart: Cart?): String {
        var message = ""
        cart?.let {
            it.cartMetadata?.recurringSlot?.let {
                message = if (!it.slotDate.isNullOrEmpty()) {
                    val reservedUntilTime = getSubscriptionExpiryDate(
                        it.slotDate,
                        DateFormats.EEE_DD_MMM_YYYY,
                        DateFormats.HH_MM_AA_EEE_DD_MMM,
                        TimeZoneFormat.UK,
                        TimeZoneFormat.UK
                    )
                    context.getString(R.string.slot_reserved_until, reservedUntilTime)
                } else {
                    context.getString(R.string.message_checkout_by, it.expiryTime)
                }
            }
        }
        return message
    }

    override fun getSlotDate(context: Context, cart: Cart?): String? {
        var formattedDate: String? = null
        try {
            inputDateFormatter.parse(cart?.fulfillmentDetails?.slotInfo?.endDateTime.orEmpty())?.let {
                formattedDate = inputDateFormatter.format(it)
            }
        } catch (e: Exception) {
            Log.w(tag, e)
        }
        var timeString = ""
        try {
            if (!cart?.fulfillmentDetails?.slotInfo?.startDateTime.isNullOrEmpty()) {
                inputDateFormatter.parse(cart?.fulfillmentDetails?.slotInfo?.startDateTime.orEmpty())?.let {
                    timeString = timeFormatter.format(it)
                }
            }
            if (!cart?.fulfillmentDetails?.slotInfo?.endDateTime.isNullOrEmpty()) {
                inputDateFormatter.parse(cart?.fulfillmentDetails?.slotInfo?.endDateTime.orEmpty())?.let {
                    timeString += " - " + timeFormatter.format(it)
                }
            }
        } catch (e: Exception) {
            timeString = cart?.fulfillmentDetails?.slotInfo?.startDateTime.orEmpty()
            if (!cart?.fulfillmentDetails?.slotInfo?.endDateTime.isNullOrEmpty()) {
                timeString += " - " + cart?.fulfillmentDetails?.slotInfo?.endDateTime
            }
        }
        return context.getString(R.string.home_card_slot_date, formattedDate, timeString)
    }

    override fun getSlotTypeByDeliveryOption(payload: WismoOrderResponse.Payload?): Int {
        return if (payload != null) {
            if (FULFILLMENT_TYPE_HD.equals(payload.fulfillmentType, ignoreCase = true)) {
                CartManagerConstants.SLOT_TYPE_DELIVERY
            } else if (FULFILLMENT_TYPE_CNC.equals(payload.fulfillmentType, ignoreCase = true) ||
                FULFILLMENT_TYPE_RCNC.equals(payload.fulfillmentType, ignoreCase = true)
            ) {
                CartManagerConstants.SLOT_TYPE_CC
            } else {
                CartManagerConstants.SLOT_TYPE_NONE
            }
        } else {
            CartManagerConstants.SLOT_TYPE_NONE
        }
    }

    override fun getSlotTypeByDeliveryOption(deliveryOption: String?): Int {
        return if (SlotConstants.DELIVERY_OPTION_HOME_DELIVERY == deliveryOption) {
            CartManagerConstants.SLOT_TYPE_DELIVERY
        } else if (SlotConstants.DELIVERY_OPTION_CLICK_AND_COLLECT == deliveryOption || SlotConstants.DELIVERY_OPTION_REMOTE_CLICK_AND_COLLECT == deliveryOption) {
            CartManagerConstants.SLOT_TYPE_CC
        } else {
            CartManagerConstants.SLOT_TYPE_NONE
        }
    }

    override fun getSlotTypeByDeliveryOptionforODS(fulfillmentType: String?): Int {
        var slotType = CartManagerConstants.SLOT_TYPE_NONE
        if (SlotConstants.ODS_FULFILLMENT_OPTION_HOME_DELIVERY == fulfillmentType) {
            slotType = CartManagerConstants.SLOT_TYPE_DELIVERY
        } else if (SlotConstants.ODS_FULFILLMENT_OPTION_IN_STORE_PICKUP == fulfillmentType || SlotConstants.ODS_FULFILLMENT_OPTION_REMOTE_CLICK_AND_COLLECT == fulfillmentType) {
            slotType = CartManagerConstants.SLOT_TYPE_CC
        }
        return slotType
    }

    override fun getSlotStatus(cart: Cart?): Int {
        if (cart == null) {
            return CartManagerConstants.SLOT_TYPE_NONE
        }
        return if (hasSlotExpired(
                cart.fulfillmentDetails?.slotInfo?.slotExpired,
                cart.fulfillmentDetails?.slotInfo?.bookedUntilTime
            )
        ) {
            CartManagerConstants.SLOT_EXPIRED
        } else hasSlot()
    }

    override fun hasSlotExpired(slotExpired: Boolean?, bookedUntilTime: String?): Boolean {
        return slotExpired.orFalse() || (
            bookedUntilTime != null &&
                bookedUntilTime.isBefore(DateFormats.YYYY_MM_DD_T_HH_MM_SS_ZZ)
            )
    }

    override fun makeServiceEligibilityCall(postcode: String?, apiKey: String?): Single<ServiceEligibilityData> {
        var serviceEligibilityDataSingle: Single<ServiceEligibilityData>? = null
        serviceEligibilityDataSingle = if (isValidPostCode(postcode)) {
            bookSlotRepository.makeServiceEligibilityCall(postcode!!, apiKey = apiKey)
        } else {
            Single.just(ServiceEligibilityData(errors = listOf(Error(code = ERROR_CODE_INVALID_POSTCODE))))
        }
        return serviceEligibilityDataSingle ?: Single.just(ServiceEligibilityData())
    }

    override fun makeServiceEligibilityCall(postcode: String, includeAccessPointList: Boolean?): Single<ServiceEligibilityData>? {
        return bookSlotRepository.makeServiceEligibilityCall(postcode, includeAccessPointList)
    }

    override fun hasSlot(): Int {
        return if (mSlotWasBookedRecently != CartManagerConstants.SLOT_TYPE_NONE) {
            mSlotWasBookedRecently
        } else
            CartManagerConstants.SLOT_TYPE_NONE
    }

    override fun setSlot(slotType: Int) {
        mSlotWasBookedRecently = slotType
    }

    override fun isValidPostCode(postcode: String?): Boolean {
        return PostCodeValidator().isValid(postcode)
    }

    override fun setServiceEligibilityFlags(serviceEligibilityData: ServiceEligibilityData?) {
        SlotUtils.setServiceEligibilityFlags(serviceEligibilityData)
    }
}
