package com.asda.android.app.chatbot.view

import android.app.Activity
import android.util.Log
import android.view.animation.AnimationUtils
import androidx.annotation.UiThread
import androidx.annotation.VisibleForTesting
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.R
import com.asda.android.app.chatbot.utils.ChatBotUtils
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.base.core.model.DeepLinkUrlParserInputModel
import com.asda.android.base.core.parser.DeepLinkUrlIdParser
import com.asda.android.base.core.parser.DeepLinkUrlParser
import com.asda.android.base.core.utils.orFalse
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.base.core.web.model.WebViewModel
import com.asda.android.base.interfaces.IChatBotManager
import com.asda.android.restapi.service.base.AsdaServiceSettings.groceriesBaseUrl
import com.asda.android.search.constants.Constants.EMPTY_STRING
import java.lang.ref.WeakReference

/**
 * Manager class which handles all the work related to chat-bot starting from initialization, handling
 * web navigation and till closing the chat-bot session
 */
object ChatBotManager : IChatBotManager {
    private val chatbotURL = groceriesBaseUrl + "chatbot"
    const val ORDER_ID_KEY = "order_id_key"
    private var isChatBotFragmentVisible = false
    private val deepLinkUrlParser = DeepLinkUrlParser()
    private val deepLinkUrlIdParser = DeepLinkUrlIdParser()

    private val isChatBotActive = MutableLiveData<Boolean>(false)
    override val isChatBotActiveLiveData: LiveData<Boolean>
        get() = isChatBotActive
    private val chatBotUtils = ChatBotUtils()

    /**
     * Function initializes the chat-bot fragment
     *
     * @param activity Activity instance
     */
    override fun initializeChatBotFragment(activity: WeakReference<Activity>, source: String) {
        activity.get()?.tryCast<AsdaActivity> {
            if (isChatBotActive.value.orFalse().not()) {
                val chatBotFragment =
                    ChatBotWebFragment.newInstance(
                        WebViewModel(
                            ChatBotWebFragment.TAG,
                            chatbotURL,
                            getString(R.string.txt_ask_question),
                            EMPTY_STRING
                        ),
                        source
                    )
                supportFragmentManager
                    .beginTransaction()
                    .replace(
                        chatBotContainer.id, chatBotFragment,
                        ChatBotWebFragment.TAG
                    )
                    .commit()
                setChatBotSession(true)
            }
            showChatBot(activity)
            chatBotUtils.trackPageView(source)
        }
    }

    /**
     * Function handles redirection form chat-bot web-view. Chat-bot web-view provides javascript
     * callback on click of hyperlink to native code
     *
     * @param activity AsdaActivity instance
     * @param url redirection url [it can be absolute as well as relative]
     */
    @VisibleForTesting
    @UiThread
    override fun handleRedirectionFromChatBot(
        activity: WeakReference<Activity>,
        url: String
    ) {
        activity.get()?.tryCast<AsdaActivity> {
            if (url.isNotBlank()) {
                // Identifies the url type
                val type = deepLinkUrlParser.parse(DeepLinkUrlParserInputModel(url, null))
                // Checks for url details like order id
                val urlDetails = deepLinkUrlIdParser.parse(
                    DeepLinkUrlParserInputModel(url, null)
                )
                // Function handles all the deep-link redirection in the app
                MerchandisingHandler.handleCmsMerchandisingEvent(
                    this,
                    null,
                    null,
                    type,
                    urlDetails.first,
                    null,
                    urlDetails.second,
                    url
                )
                // If it is one of the bottom tab redirection, hide chat-bot fragment and show main fragment container
                if (chatBotUtils.isFragmentRedirection(type)) {
                    hideChatBot(WeakReference(this))
                }
            }
        }
    }

    /**
     * Function hides chat-bot container and shows main app container
     *
     * @param activity AsdaActivity instance
     */
    override fun hideChatBot(
        activity: WeakReference<Activity>
    ) {
        activity.get()?.tryCast<AsdaActivity> {
            handleChatBotContainerVisibility(
                this,
                false
            )
        }
    }

    /**
     * Function hides main container and shows chat-bot container
     *
     * @param activity AsdaActivity instance
     */
    override fun showChatBot(
        activity: WeakReference<Activity>
    ) {
        activity.get()?.tryCast<AsdaActivity> {
            handleChatBotContainerVisibility(
                this,
                true
            )
        }
    }

    private fun handleChatBotContainerVisibility(activity: AsdaActivity, showChatBot: Boolean) {
        activity.apply {
            executeIfSafe {
                isChatBotFragmentVisible = if (showChatBot) {
                    // Hide main app fragment container and show chat-bot fragment container
                    headerController.hideActionBar()
                    bottomTabView.gone()
                    mainContainer.gone()
                    chatBotContainer.visible()
                    // Slide-in animation
                    val slideInAnimation = AnimationUtils.loadAnimation(this, R.anim.slide_in_left)
                    chatBotContainer.startAnimation(slideInAnimation)
                    true
                } else {
                    // Hide chat-bot fragment container and show main app fragment container
                    bottomTabView.visible()
                    mainContainer.visible()
                    chatBotContainer.gone()
                    // Slide-out animation
                    val slidOutAnimation = AnimationUtils.loadAnimation(this, R.anim.slide_out_right)
                    chatBotContainer.startAnimation(slidOutAnimation)
                    headerController.showActionBar()
                    false
                }
            }
        }
    }

    /**
     * Function closes chat-bot session and view
     *
     * @return activity AsdaActivity instance
     */
    override fun closeChatBotSession(activity: WeakReference<Activity>) {
        activity.get()?.tryCast<AsdaActivity> {
            if (isChatBotActive.value.orFalse()) {
                try {
                    hideChatBot(activity)
                    // Remove chat-bot fragment from fragment manager to save memory
                    val chatBotFragment =
                        supportFragmentManager.findFragmentByTag(ChatBotWebFragment.TAG)
                    chatBotFragment?.let {
                        if (isFinishing.not() && it.isAdded) {
                            supportFragmentManager
                                .beginTransaction()
                                .remove(it)
                                .commit()
                        }
                    }
                    setChatBotSession(false)
                } catch (ex: Exception) {
                    Log.w(ChatBotWebFragment.TAG, "exception in removing fragment $ex")
                }
            }
        }
    }

    override fun isChatBotVisible() = isChatBotFragmentVisible

    @VisibleForTesting
    override fun setChatBotSession(isActive: Boolean) {
        isChatBotActive.value = isActive
    }

    private fun AppCompatActivity.executeIfSafe(callback: () -> Unit) {
        if (isFinishing.not() && lifecycle.currentState.isAtLeast(Lifecycle.State.RESUMED)) {
            callback()
        }
    }
}
