package com.asda.android.app.checkout

import android.app.Dialog
import android.content.Context
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.StrikethroughSpan
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import android.widget.RadioGroup
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.OnCXOCardAddedListener
import com.asda.android.app.model.CardCheckoutModel
import com.asda.android.app.view.AsdaFullScreenActivity
import com.asda.android.app.view.AsdaPresenter
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.cxo.CartManager
import com.asda.android.cxo.repository.CheckoutRepository
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.restapi.service.data.checkout.CardDetails
import com.asda.android.restapi.service.data.checkout.ChangeCreditCardRequest
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.singleprofile.features.account.datasource.UserAccountDetails
import com.asda.android.utils.view.ViewUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers

class SelectCardPresenter(
    context: Context,
    selectedCardNickName: String?,
    isColleagueDiscountAccepted: Boolean
) : AsdaPresenter() {

    interface OnCXOCardChangedListener {
        fun onCardChanged(
            checkoutResponse: CheckoutResponse?
        )
    }

    private val mContext: Context
    private val mView: View
    private var mSelectedCardNickName: String?
    private var mOnCXOCardChangedListener: OnCXOCardChangedListener? = null
    private val compositeDisposable = CompositeDisposable()
    private var mUserClickedSave = false
    private var mIsColleagueDiscountAccepted = true

    fun setOnCXOCardChangedListener(listener: OnCXOCardChangedListener?) {
        mOnCXOCardChangedListener = listener
    }

    override fun onPop() {
        if (mView.findViewById<View?>(R.id.save_button) != null && !mUserClickedSave) {
            mView.findViewById<View>(R.id.save_button).callOnClick()
        }
    }

    private fun wireListeners() {
        mView
            .findViewById<View>(R.id.save_button)
            .setOnClickListener { view: View? ->
                mUserClickedSave = true
                val radioGroup = ViewUtil.findViewById<RadioGroup>(mView, R.id.group)
                val checkedId = radioGroup.checkedRadioButtonId
                if (checkedId != -1) {
                    val checkedView = radioGroup.findViewById<View>(checkedId)
                    val paymentCard = checkedView.tag as PaymentCards
                    if (paymentCard != null) {
                        if (paymentCard.expired) {
                            DialogHelper.displayErrorDialog(
                                mContext.getString(R.string.selected_card_expired), mContext, SCREEN_NAME
                            )
                            setSingleClickFlag(false)
                        } else if (mSelectedCardNickName == paymentCard.nickName || mSelectedCardNickName == paymentCard.orderCardNickName || mSelectedCardNickName == paymentCard.card) {
                            pop()
                            if (mOnCXOCardChangedListener != null) {
                                mOnCXOCardChangedListener?.onCardChanged(null)
                            }
                        } else {
                            var cardId = paymentCard.cardId
                            if (cardId == null) {
                                cardId = paymentCard.orderCardNickName
                            }
                            changeCreditCardSp(cardId, false, true)
                        }
                    }
                } else {
                    pop()
                }
            }
        mView
            .findViewById<View>(R.id.add_label)
            .setOnClickListener { view: View? ->
                pushPresenter(
                    AddCardWrapperPresenter(
                        (mContext as AppCompatActivity),
                        object : OnCXOCardAddedListener {
                            override fun onCXOCardAdded(checkoutResponse: CheckoutResponse?, cardNickName: String?) {
                                if (checkoutResponse == null && cardNickName != null) {
                                    changeCreditCardSp(cardNickName, false, false)
                                } else {
                                    pop()
                                    if (mOnCXOCardChangedListener != null) {
                                        mOnCXOCardChangedListener?.onCardChanged(checkoutResponse)
                                    }
                                }
                            }
                        }
                    )
                )
            }
    }

    // for single profile users, we use same api end point but need to replace card nickname with
    // credit cardId, that's it
    private fun changeCreditCardSp(
        creditCardId: String?,
        refreshOnFail: Boolean,
        popPresenter: Boolean
    ) {
        changeCreditCard(creditCardId, refreshOnFail, popPresenter)
    }

    private fun changeCreditCard(
        cardNickName: String?,
        refreshOnFail: Boolean,
        popPresenter: Boolean
    ) {
        if (popPresenter) {
            showDialog(DIALOG_SAVING)
        }
        CheckoutRepository()
            .changeCreditCard(
                CartManager.instance.cartId,
                ChangeCreditCardRequest(CardDetails(cardNickName))
            )
            ?.subscribeOn(Schedulers.io())
            ?.observeOn(AndroidSchedulers.mainThread())
            ?.subscribe(
                object : ResourceSingleObserver<CheckoutResponse?>() {
                    override fun onSuccess(
                        checkoutResponse: CheckoutResponse
                    ) {
                        dismissDialog(DIALOG_SAVING)
                        if (isPopped) {
                            return
                        }
                        if (popPresenter) {
                            pop()
                            if (mOnCXOCardChangedListener != null) {
                                mOnCXOCardChangedListener?.onCardChanged(checkoutResponse)
                            }
                        } else {
                            mSelectedCardNickName = cardNickName
                            getCards()
                        }
                    }

                    override fun onError(e: Throwable) {
                        if (isPopped) {
                            return
                        }
                        setSingleClickFlag(false)
                        dismissDialog(DIALOG_SAVING)
                        if (refreshOnFail) {
                            getCards()
                        }
                    }
                })
    }

    override fun getView(): View {
        return mView
    }

    override fun onCreateView(parent: ViewGroup) {
        // No op
    }

    private fun populateCreditCards(paymentCards: List<PaymentCards>) {
        val radioGroup = ViewUtil.findViewById<RadioGroup>(mView, R.id.group)
        radioGroup.clearCheck()
        radioGroup.removeAllViews()
        var id = 0
        var selected = 0
        for (card in paymentCards) {
            initRadioGroupForCreditCards(radioGroup, card, id, true)
            if (mSelectedCardNickName == card.nickName || mSelectedCardNickName == card.orderCardNickName || mSelectedCardNickName == card.cardId || mSelectedCardNickName == card.card) {
                selected = id
            }
            ++id
        }
        if (id == 0) {
            radioGroup.visibility = View.GONE
        } else {
            radioGroup.visibility = View.VISIBLE
            radioGroup.check(selected)
        }
    }

    private fun initRadioGroupForCreditCards(
        radioGroup: RadioGroup,
        card: PaymentCards,
        id: Int,
        isSingleProfile: Boolean
    ) {
        val radioButton = ViewUtil.inflate<RadioButton>(
            mView.context, R.layout.card_billing_address, mView as ViewGroup, false
        )
        var cardText: CharSequence?
        cardText = if (isSingleProfile) {
            if (!TextUtils.isEmpty(card.orderCardNickName) &&
                card.orderCardNickName?.toLowerCase()?.contains("temp") == true
            ) {
                card.orderCardNickName
            } else {
                card.cardBrand + " " + card.card
            }
        } else {
            if (card.card != null) card.card else card.orderCardNickName
        }
        radioButton.id = id
        radioButton.tag = card
        radioButton.isEnabled = !card.expired
        if (card.expired) {
            radioButton.isEnabled = false
            val builder = SpannableStringBuilder(cardText.toString() + " (expired)")
            builder.setSpan(
                StrikethroughSpan(),
                0,
                builder.length,
                SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE
            )
            cardText = builder
        }
        radioButton.text = cardText
        radioGroup.addView(
            radioButton, RadioGroup.LayoutParams.MATCH_PARENT, RadioGroup.LayoutParams.WRAP_CONTENT
        )
    }

    override fun onBeforePoppedTo() {
        if (mContext is AsdaFullScreenActivity) {
            mContext.showHeaderWithBackCarat()
        }
        super.onBeforePoppedTo()
    }

    override fun onBeforePush() {
        if (mContext is AsdaFullScreenActivity) {
            mContext.showHeaderWithBackCarat()
        }
        getCards()
    }

    private fun getCards() {
        mView.findViewById<View>(R.id.loading).visibility = View.VISIBLE
        var d: Disposable? = null
        val observable = UserAccountDetails.instance.userCreditCards
        if (observable != null) {
            d = observable
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    { model: PaymentDetailsResponse? ->
                        if (model != null) {
                            if (isPopped) {
                                return@subscribe
                            }
                            mView.findViewById<View>(R.id.loading).visibility = View.GONE
                            populateCreditCards(
                                CardCheckoutModel(model, null, null).paymentCardsList
                            )
                            wireListeners()
                        }
                    }
                ) { throwable: Throwable? ->
                    if (isPopped) {
                        return@subscribe
                    }
                    pop()
                }
        }
        if (d != null) {
            compositeDisposable.add(d)
        } else {
            Log.e(TAG, "Disposable is null")
            mView.findViewById<View>(R.id.loading).visibility = View.GONE
        }
    }

    override fun onCreateDialog(dialog: Int): Dialog? {
        val d: Dialog?
        when (dialog) {
            DIALOG_SAVING -> {
                val pd = CustomProgressDialog.create(mContext)
                pd.setCancelable(false)
                pd.setMessage(mContext.getString(R.string.saving_in_progress))
                pd.isIndeterminate = true
                d = pd
            }
            else -> d = super.onCreateDialog(dialog)
        }
        return d
    }

    override fun onPauseAsTop() {
        super.onPauseAsTop()
        compositeDisposable.clear()
    }

    companion object {
        private const val SCREEN_NAME = "Payment Card Select"
        private const val TAG = "SelectCardPresenter"
        private const val DIALOG_SAVING = 1
    }

    init {
        pageViewEvent = PageViewEvent(Anivia.PV_SELECT_CARD, Anivia.SECTION_CHECKOUT, Anivia.SECTION_CHECKOUT)
        mContext = context
        mSelectedCardNickName = selectedCardNickName ?: ""
        mIsColleagueDiscountAccepted = isColleagueDiscountAccepted
        mView = ViewUtil.inflate(mContext, R.layout.select)
        (mView.findViewById<View>(R.id.save_button) as TextView).setText(R.string.select_card_done)
        (mView.findViewById<View>(R.id.add_label) as TextView).setText(R.string.select_card_add)
        titleText = context.getString(R.string.select_card_title)
    }
}
