package com.asda.android.app.main.view

import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.exception.CaughtExceptionEvent
import com.asda.android.featureGen._Shared_Pref_Utils
import com.asda.android.firebase.RemoteConfig
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils
import org.json.JSONArray

object RemoteConfigController {

    private val TAG = "RemoteConfigController"
    fun validateStore(activity: Activity?) {
        val storeId = AsdaServiceFactory.get().lastStoreIdFromSessionMetadata

        if (activity == null || storeId == null) return
        val remoteConfigCallback: RemoteConfig.RemoteConfigCallback = object : RemoteConfig.RemoteConfigCallback {
            override fun onDownloadSuccess() {
                SharedPreferencesUtil.setRemoteConfigDetails(activity)
                _Shared_Pref_Utils.setRemoteConfigDetails(activity)
                val myVersion = ViewUtils.getApplicationVersion(activity)
                RemoteConfig.let {
                    val appMinimumVersion: Int = getAppVersion(it, "appMinimumVersionForStores")

                    val requireUpdate = isUpdateRequired(it, storeId)

                    if (myVersion != -1 && appMinimumVersion > 0 && myVersion < appMinimumVersion && requireUpdate) {
                        val intent = Intent(activity, UpgradeActivity::class.java)
                        updateMessage(intent, it)
                        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                        activity.startActivity(intent)
                        activity.finish()
                    }
                }
            }

            override fun onFailure() {
                Log.e(TAG, "Failed to retrieve APG config.")
            }
        }
        setFirebaseDefaultsAndFetch(activity, remoteConfigCallback)
    }

    private fun isUpdateRequired(remoteConfig: RemoteConfig, storeId: String): Boolean {
        var requireUpdate = false
        val minimumVersionStoreIds = remoteConfig.getValue("appMinimumVersionStoreIds")
        if (minimumVersionStoreIds.isNotEmpty()) {
            val storeIds = JSONArray(minimumVersionStoreIds)
            for (i in 0 until storeIds.length()) {
                if (storeId == storeIds.getString(i)) {
                    requireUpdate = true
                    break
                }
            }
        }
        return requireUpdate
    }

    /**
     * This function is written to avoid duplicate code in this class.
     * This function is supposed to be invoked from this particular class, by either of the stated keys.
     * @param remoteConfig = remote config instance.
     * @param key =  the key to be searched in remote config.
     * @return app version corresponding to the key from remote config
     */
    private fun getAppVersion(remoteConfig: RemoteConfig, key: String): Int {
        return if (key == "appMinimumVersion" || key == "appMinimumVersionForStores" || key == "appCurrentVersion") {
            var appVersion: Int
            try {
                appVersion = Integer.parseInt(remoteConfig.getValue(key))
            } catch (e: Exception) {
                appVersion = -1
                Log.e(TAG, e.message.toString())
            }
            return appVersion
        } else {
            -1
        }
    }

    fun getConfigForAppUpdate(activity: Activity?) {
        val storeId = AsdaServiceFactory.get().lastStoreIdFromSessionMetadata

        if (activity == null || storeId == null) return
        val remoteConfigCallback: RemoteConfig.RemoteConfigCallback = object : RemoteConfig.RemoteConfigCallback {

            override fun onDownloadSuccess() {
                SharedPreferencesUtil.setRemoteConfigDetails(activity)
                _Shared_Pref_Utils.setRemoteConfigDetails(activity) // for generated flags
                val myVersion = ViewUtils.getApplicationVersion(activity)
                RemoteConfig.let { it ->
                    val appMinimumVersion: Int = getAppVersion(it, "appMinimumVersion")

                    val appMinimumStoreVersion: Int = getAppVersion(it, "appMinimumVersionForStores")

                    val appCurrentVersion: Int = getAppVersion(it, "appCurrentVersion")

                    val requireUpdate = isUpdateRequired(it, storeId)

                    if (myVersion != -1) {

                        if (isForceUpdate(appMinimumVersion, myVersion, appMinimumStoreVersion, requireUpdate)) {
                            val intent = Intent(activity, UpgradeActivity::class.java)
                            updateMessage(intent, it)
                            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                            activity.startActivity(intent)
                            activity.finish()
                        } else if (isUpdateAvailable(appCurrentVersion, myVersion, activity)) {
                            AlertDialog.Builder(activity)
                                .setMessage(R.string.newer_version_available)
                                .setPositiveButton(activity.getText(R.string.upgrade_now_nag_button)) { dialog, which ->
                                    launchPlayStore(activity)
                                }
                                .setNegativeButton(activity.getText(R.string.upgrade_later_nag_button), null)
                                .setCancelable(true)
                                .create().show()
                        }
                    }
                }
            }

            override fun onFailure() {
                Log.e(TAG, "Failed to retrieve APG config.")
            }
        }

        setFirebaseDefaultsAndFetch(activity, remoteConfigCallback)
    }

    private fun launchPlayStore(activity: Activity) {
        try {
            activity.startActivity(Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=com.asda.android")))
        } catch (e: ActivityNotFoundException) {
            Log.e(TAG, "unable to open market")
            Tracker.get().trackException(CaughtExceptionEvent(e))
            Toast.makeText(activity, activity.getText(R.string.oops_error), Toast.LENGTH_SHORT).show()
        }
    }

    private fun isForceUpdate(appMinimumVersion: Int, myVersion: Int, appMinimumStoreVersion: Int, requireUpdate: Boolean): Boolean {
        return appMinimumVersion > 0 && (myVersion < appMinimumVersion || myVersion < appMinimumStoreVersion && requireUpdate)
    }

    private fun isUpdateAvailable(appCurrentVersion: Int, myVersion: Int, activity: Activity): Boolean {
        return appCurrentVersion > 0 && myVersion < appCurrentVersion && !activity.isFinishing
    }

    private fun updateMessage(intent: Intent, it: RemoteConfig) {
        if (it.getValue("appForceUpgradeMessage").isNotEmpty()) {
            intent.putExtra(UpgradeActivity.EXTRA_MESSAGE, it.getValue("appForceUpgradeMessage"))
        }
    }

    private fun setFirebaseDefaultsAndFetch(activity: Context, callBack: RemoteConfig.RemoteConfigCallback) {
        RemoteConfig.setdefaultConfigValues(activity, Utils.isDevBuild)
        SharedPreferencesUtil.setRemoteConfigDetails(activity)
        _Shared_Pref_Utils.setRemoteConfigDetails(activity) // for the generated flags.
        RemoteConfig.getRemoteConfigValues(activity, callBack)
    }
}
