package com.asda.android.app.messagecenter

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.text.TextUtils
import android.text.format.DateUtils
import android.util.Log
import androidx.collection.ArrayMap
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.app.messagecenter.osm.OSMMessage
import com.asda.android.app.messagecenter.view.UAMessageFragment
import com.asda.android.app.messagecenter.view.WelcomeMessageFragment
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.utils.orTrue
import com.asda.android.base.core.utils.orZero
import com.asda.android.orders.constants.OrderConstants.EMPTY
import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.fasterxml.jackson.annotation.JsonProperty
import com.urbanairship.richpush.RichPushMessage
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale
import java.util.UUID

@JsonIgnoreProperties(ignoreUnknown = true)
class Message {
    private val dateFormat =
        SimpleDateFormat("dd-MM-yy", Locale.UK)

    @JsonProperty("title")
    var title: String? = null
        private set

    @JsonProperty("messageId")
    var messageId: String?
        private set

    @JsonProperty("keyValueMap")
    private var keyValueMap: ArrayMap<String, String>? = null

    // These values should be absent on the json used to initialize messages

    val expirationDateMS: Long?

    val sentDateMS: Long

    var isDeleted = false
        private set

    var readStatus = false
        private set

    var content: String? = null
        private set

    @JsonProperty("messageType")
    var messageType: String
        private set

    private var mMessage: RichPushMessage? = null

    var priority = Int.MAX_VALUE
        private set
    var campaignId = EMPTY
        private set

    constructor() {
        // Constructor set default values that can be overridden by json
        sentDateMS = System.currentTimeMillis()
        expirationDateMS = THIRTY_DAYS_MS
        keyValueMap = ArrayMap()
        messageType = MSG_TYPE_PRE_LOAD
        messageId = EMPTY
    }

    constructor(msgId: String?) {
        sentDateMS = System.currentTimeMillis()
        expirationDateMS = THIRTY_DAYS_MS
        keyValueMap = ArrayMap()
        messageType = MSG_TYPE_PRE_LOAD
        messageId = msgId
    }

    constructor(message: RichPushMessage) {
        mMessage = message
        title = message.title
        messageId = message.messageId
        sentDateMS = message.sentDateMS
        expirationDateMS = mMessage?.expirationDateMS.orZero()
        isDeleted = mMessage?.isDeleted.orTrue()
        readStatus = mMessage?.isRead.orTrue()
        messageType = MSG_TYPE_UA
    }

    constructor(message: OSMMessage) {
        keyValueMap = ArrayMap()
        title = message.heading
        messageId = message.messageId ?: UUID.randomUUID().toString()
        sentDateMS = convertToTimeStamp(
            message.updatedAt.orEmpty(),
            DateFormats.YYYY_MM_DD_T_KK_MM_SS_SSS_SSSS
        ).orZero()
        expirationDateMS = 0L
        content = message.content
        readStatus = isOSMMessageRead(message.state)
        messageType = MSG_TYPE_OSM
        priority = message.priority?.toInt() ?: Int.MAX_VALUE
        campaignId = message.campaignId.orEmpty()
    }

    fun markDeleted() {
        mMessage?.delete()
        isDeleted = true
    }

    private fun isOSMMessageRead(state: String?): Boolean {
        return state.equals(
            STATE_READ,
            ignoreCase = true
        )
    }

    val extras: Bundle
        get() = mMessage?.let {
            mMessage?.extras
        } ?: run {
            val bundle = Bundle(keyValueMap?.size.orZero())
            for (key in keyValueMap?.keys.orEmpty()) {
                bundle.putString(key, keyValueMap?.get(key))
            }
            bundle
        }

    val isExpired: Boolean
        get() = if (mMessage != null) {
            mMessage?.isExpired.orTrue()
        } else {
            System.currentTimeMillis() >= sentDateMS + expirationDateMS.orZero()
        }

    fun markRead() {
        if (mMessage != null) {
            mMessage?.markRead()
        }
        readStatus = true
    }

    fun createMessageDetailFragment(): Fragment {
        return if (mMessage == null) {
            WelcomeMessageFragment.newInstance()
        } else UAMessageFragment.newInstance(messageId)
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) {
            return true
        }
        if (other == null || javaClass != other.javaClass) {
            return false
        }
        val message =
            other as Message
        return messageId == message.messageId
    }

    override fun hashCode(): Int {
        return if (messageId != null) messageId.hashCode() else 0
    }

    val subTitle: String?
        get() {
            val extras: Bundle? = extras
            return if (extras != null) extras.getString(SUB_TITLE_EXTRA) else ""
        }

    val iconUrl: String?
        get() {
            val extras: Bundle? = extras
            return if (extras != null && extras.containsKey(ICON_EXTRA)) {
                extras.getString(ICON_EXTRA)
            } else null
        }

    fun getExpiry(context: Context): String? {
        val extras: Bundle? = extras
        return if (extras != null && expirationDateMS != null && TYPE_PRODUCT_OFFER == extras.getString(
                MSG_TYPE_EXTRA
            )
        ) {
            context.getString(
                R.string.message_expiry, dateFormat.format(Date(expirationDateMS.orZero()))
            )
        } else null
    }

    fun getAction(context: Context): String {
        val action =
            extras.getString(ACTION_TEXT_EXTRA, EMPTY)
        return if (action.isNullOrBlank().not()) action else context.getString(R.string.view)
    }

    fun getTitleColor(context: Context): Int {
        val extras: Bundle? = extras
        var color =
            ContextCompat.getColor(context, R.color.msg_text_area_default)
        if (extras != null) {
            try {
                val colorStr =
                    extras.getString(TEXT_AREA_COLOR_EXTRA)
                if (!TextUtils.isEmpty(colorStr)) {
                    color = Color.parseColor(colorStr)
                }
            } catch (e: IllegalArgumentException) {
                Log.w("Message", "Error parsing color from message")
            }
        }
        return color
    }

    companion object {
        const val ICON_EXTRA = "ICON_EXTRA"
        const val TEXT_AREA_COLOR_EXTRA = "TEXT_AREA_COLOR_EXTRA"
        const val SUB_TITLE_EXTRA = "SUB_TITLE_EXTRA"
        const val ACTION_TEXT_EXTRA = "ACTION_TEXT_EXTRA"
        const val ACTION_URI_EXTRA = "ACTION_URI_EXTRA"
        const val ACTION_TAG_EXTRA = "ACTION_TAG_EXTRA"
        const val MSG_TYPE_EXTRA = "MSG_TYPE_EXTRA"
        const val TYPE_PRODUCT_OFFER = "PRODUCT_OFFER_TYPE"
        const val STATE_UNREAD = "UNREAD"
        const val STATE_READ = "READ"
        const val MSG_TYPE_PRE_LOAD = "PRELOADED_MESSAGE"
        const val MSG_TYPE_UA = "UA_MESSAGE"
        const val MSG_TYPE_OSM = "OSM_MESSAGE"
        private const val THIRTY_DAYS_MS = DateUtils.DAY_IN_MILLIS * 30
    }
}
