package com.asda.android.app.messagecenter

import android.content.Context
import android.text.format.DateUtils
import androidx.annotation.VisibleForTesting
import com.asda.android.app.base.airship.view.AirshipPushListener.Companion.dismissInboxNotification
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_OSM
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_PRE_LOAD
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_UA
import com.asda.android.app.messagecenter.osm.OSMMessageManager
import com.asda.android.app.settings.featureflag.FeatureSettings.isOSMMessageServiceEnabled
import com.asda.android.cmsprovider.AsdaCMSConfig
import com.asda.android.restapi.utils.SharedPreferencesUtil.isMessageCenterAvailable
import com.urbanairship.UAirship
import com.urbanairship.richpush.RichPushInbox
import com.urbanairship.richpush.RichPushMessage
import kotlinx.coroutines.flow.collect
import java.util.ArrayList
import java.util.HashSet
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine

class MessageCenterController private constructor(context: Context) {
    interface OnInboxUpdateListener {
        fun onInboxUpdate()
    }

    private val mContext: Context = context.applicationContext
    @VisibleForTesting(otherwise = VisibleForTesting.PRIVATE)
    var mPreloadedMessageManager: PreloadedMessageManager
    @VisibleForTesting(otherwise = VisibleForTesting.PRIVATE)
    val mListeners: MutableSet<OnInboxUpdateListener> =
        HashSet()
    private val mInboxListener =
        RichPushInbox.Listener { }

    private suspend fun getUAirshipSuspend() = suspendCoroutine<UAirship> {
        UAirship.shared { airship ->
            it.resume(airship)
        }
    }

    @VisibleForTesting
    suspend fun getUnreadMessageCount(): Int {
        var unreadCount = 0
        for (message in getAllMessages()) {
            if (message.readStatus.not()) {
                unreadCount++
            }
        }
        return unreadCount
    }

    fun addOnInboxUpdateListener(listener: OnInboxUpdateListener) {
        if (isMessageCenterAvailable(mContext)) {
            mListeners.add(listener)
            if (mListeners.size == 1) {
                UAirship.shared { airship: UAirship ->
                    airship.inbox.addListener(mInboxListener)
                    airship.inbox
                        .fetchMessages { b: Boolean -> notifyListeners() }
                }
            }
        }
    }

    fun removeOnInboxUpdateListener(listener: OnInboxUpdateListener) {
        val hasRemoved = mListeners.remove(listener)
        if (hasRemoved && mListeners.isEmpty()) {
            UAirship.shared { airship: UAirship ->
                airship.inbox.removeListener(mInboxListener)
            }
        }
    }

    @VisibleForTesting
    fun notifyListeners() {
        for (listener in mListeners) {
            listener.onInboxUpdate()
        }
    }

    suspend fun markRead(messageId: String?) {
        messageId?.let {
            if (!mPreloadedMessageManager.markReadById(it)) {
                val messages: MutableSet<String> = HashSet()
                messages.add(it)
                getUAirshipSuspend().inbox.markMessagesRead(messages)
                if (getUnreadMessageCount() == 0) {
                    dismissInboxNotification()
                }
            }
        }
    }

    /**
     * Function marks OSM messages as READ
     *
     * @param messageList List of OSM messages
     */
    suspend fun markOSMMessagesRead(messageList: List<Message>) {
        messageList.asSequence().filter {
            it.messageType == MSG_TYPE_OSM && it.readStatus.not()
        }.map {
            it.campaignId
        }.map {
            OSMReadMessageRequest.OSMRequestMessage(messageId = null, campaignId = it)
        }.mapTo(mutableListOf()) {
            it
        }.let {
            if (it.isNotEmpty()) {
                val readRequest = OSMReadMessageRequest(
                    OSMReadMessageRequest.ReadStateData(
                        OSMReadMessageRequest.ReadStateOsm(
                            profileId = AsdaCMSConfig.interfaceProfileIdEmitter.getSingleProfileId(),
                            messages = it
                        )
                    )
                )
                OSMMessageManager.markOSMMessageRead(readRequest).collect {
                    // Not interested in the read response
                }
            }
        }
    }

    /**
     * Function sorts messages according to read status and sent timestamp
     *
     * @param messageList List of messages to sort
     * @return sorted messages
     */
    fun sortMessageList(messageList: List<Message>): List<Message> {
        val sortedMessageSet = LinkedHashSet<Message>()
        // Filter unread messages and sort with sent time stamp && priority
        messageList.filter {
            it.readStatus.not()
        }.sortedWith(compareBy({ it.priority }, { -it.sentDateMS }))
            .apply {
                sortedMessageSet.addAll(this)
            }
        // Filter read messages and sort with sent time stamp && priority
        messageList.filterNot {
            it.readStatus.not()
        }.sortedWith(compareBy({ it.priority }, { -it.sentDateMS }))
            .apply {
                sortedMessageSet.addAll(this)
            }
        return sortedMessageSet.toList()
    }

    /**
     * Function retrieves all the messages [PRE_LOAD, UA, OSM]
     *
     * @return Collection of messages
     */
    suspend fun getAllMessages(): Collection<Message> {
        val messageList = arrayListOf<Message>()
        // Fetches UA messages and preloaded messages
        messageList.addAll(getAirShipMessages())
        if (isOSMMessageServiceEnabled(mContext)) {
            // Fetches OSM messages
            OSMMessageManager.fetchOSMMessages().collect {
                messageList.addAll(it)
            }
        }
        // Sort according to time
        return sortMessageList(messageList)
    }

    /**
     * Function retrieves UAirship inbox messages
     *
     * @return UA Message list
     */
    private suspend fun getAirShipMessages(): List<Message> {
        return suspendCoroutine { continuation ->
            UAirship.shared { uAirship: UAirship ->
                continuation.resume(
                    getAirShipMessages(
                        uAirship
                    )
                )
            }
        }
    }

    /**
     * Function retrieves local messages and UAirship messages
     *
     * @param uAirship Airship object
     * @return list of messages
     */
    fun getAirShipMessages(uAirship: UAirship): List<Message> {
        val messages =
            ArrayList<Message>()
        val uAMessages = uAirship.inbox.messages
        for (uaMessage in uAMessages) {
            if (isValid(uaMessage)) {
                messages.add(Message(uaMessage!!))
            }
        }
        messages.addAll(mPreloadedMessageManager.messages)
        return messages
    }

    fun refreshMessages() {
        UAirship.shared { uAirship: UAirship ->
            uAirship.inbox.fetchMessages()
        }
    }

    fun cleanupMessagesIfNeeded() {
        mPreloadedMessageManager.cleanupExpired()
        UAirship.shared { uAirship: UAirship ->
            // Cleanup UA messages
            val richPushInbox = uAirship.inbox
            val uAMessages =
                richPushInbox.messages
            val messagesToDelete: MutableSet<String> = HashSet()
            for (message in uAMessages) {
                if (!isValid(message)) {
                    messagesToDelete.add(message.messageId)
                }
            }
            if (messagesToDelete.isNotEmpty()) {
                richPushInbox.deleteMessages(messagesToDelete)
            }
        }
    }

    /**
     * Function deletes messages by message id
     *
     * @param messages Message list
     */
    suspend fun deleteMessagesById(messages: List<Message>, doAfterDelete: () -> Unit) {
        // Suspend function resumes with empty params, so taking local copy of the function params(strange issue)
        val newMessages = ArrayList(messages)
        newMessages.forEach { message ->
            when (message.messageType) {
                MSG_TYPE_PRE_LOAD -> {
                    mPreloadedMessageManager.deleteMessageById(message.messageId)
                }
                MSG_TYPE_OSM -> {
                    OSMMessageManager.deleteOSMMessage(message.messageId).collect {}
                }
                MSG_TYPE_UA -> {
                    getUAirshipSuspend().inbox.deleteMessages(mutableSetOf(message.messageId))
                }
            }
        }
        notifyListeners()
        doAfterDelete()
    }

    /**
     * Function finds message by message id
     *
     * @param messageId id
     * @return Message
     */
    suspend fun findMessageById(messageId: String): Message? {
        return getAllMessages()
            .firstOrNull { it.messageId == messageId }
    }

    companion object {
        private var sInstance: MessageCenterController? = null
        fun getInstance(context: Context?): MessageCenterController? {
            if (sInstance == null && context != null) {
                sInstance =
                    MessageCenterController(context.applicationContext)
            }
            return sInstance
        }

        private fun isValid(message: RichPushMessage?): Boolean {
            return message != null && !message.isDeleted && !hasExpired(
                message
            )
        }

        private fun hasExpired(message: RichPushMessage): Boolean {
            val thirtyDaysMs = DateUtils.DAY_IN_MILLIS * 30
            return (
                message.isExpired ||
                    (
                        message.expirationDateMS == null &&
                            System.currentTimeMillis() - message.sentDateMS >= thirtyDaysMs
                        )
                )
        }
    }

    init {
        mPreloadedMessageManager = PreloadedMessageManager(mContext)
    }
}
