package com.asda.android.app.messagecenter

import android.content.Context
import android.content.SharedPreferences
import android.text.TextUtils
import android.util.Log
import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.JsonMapper
import com.asda.android.base.core.view.AsyncTask
import java.io.IOException
import java.util.ArrayList

class PreloadedMessageManager(private val mContext: Context) {
    private val mPreloadedMessages: ArrayList<Message> = ArrayList()
    private val mPersistentStorage: SharedPreferences
    private fun loadFromStorage() {
        object :
            AsyncTask<Array<Message>>() {
            override fun doInBackground(): Array<Message> {
                try {
                    val json = mPersistentStorage.getString(
                        PRELOADED_KEY,
                        EMPTY_JSON_ARRAY
                    )
                    val objectMapper = JsonMapper.get()
                    return objectMapper.readValue(
                        json,
                        Array<Message>::class.java
                    )
                } catch (e: IOException) {
                    Log.w(
                        TAG,
                        "Could not load preloaded messages"
                    )
                    return emptyArray()
                }
            }

            override fun onPostExecute(messagesFromStorage: Array<Message>) {
                for (msg in messagesFromStorage) {
                    if (!msg.isExpired && hasIcon(msg)) {
                        mPreloadedMessages.add(msg)
                    }
                }
            }
        }.execute()
    }

    @VisibleForTesting
    fun hasIcon(msg: Message): Boolean {
        var hasIcon = false
        try {
            val extras = msg.extras
            val icon = extras.getString(Message.ICON_EXTRA)
            icon?.let {
                if (!TextUtils.isEmpty(icon)) {
                    val path = if (it.startsWith(ASSET_PREFIX)) it.replace(ASSET_PREFIX, "") else it
                    val inputStream = mContext.assets.open(path) // Will throw FileNotFoundException if doesn't exist
                    inputStream.close()
                    hasIcon = true
                }
            }
        } catch (ex: Exception) {
            Log.w(TAG, ex)
        }
        return hasIcon
    }

    private fun setupInitialMessages() {
        object :
            AsyncTask<Array<Message>>() {
            override fun doInBackground(): Array<Message> {
                try {
                    return JsonMapper.get()
                        .readValue(
                            mContext.assets.open("preloaded_messages.json"),
                            Array<Message>::class.java
                        )
                } catch (e: Exception) {
                    Log.w(
                        TAG,
                        "Could not load template file",
                        e
                    )
                }
                return emptyArray()
            }

            override fun onPostExecute(messagesFromStorage: Array<Message>) {
                mPreloadedMessages.addAll(messagesFromStorage)
                persist()
            }
        }.execute()
    }

    private fun persist() {
        object : AsyncTask<String?>() {
            override fun doInBackground(): String? {
                try {
                    return JsonMapper.get().writeValueAsString(mPreloadedMessages)
                } catch (e: IOException) {
                    Log.w(
                        TAG,
                        "Could not persist changes to preloaded messages",
                        e
                    )
                }
                return null
            }

            override fun onPostExecute(json: String?) {
                if (json != null) {
                    mPersistentStorage.edit()
                        .putString(PRELOADED_KEY, json).apply()
                }
            }
        }.execute()
    }

    val messages: List<Message>
        get() {
            val messages: MutableList<Message> =
                ArrayList()
            for (message in mPreloadedMessages) {
                if (!message.isExpired && !message.isDeleted) {
                    messages.add(message)
                }
            }
            return messages
        }

    private fun deleteMessage(message: Message) {
        message.markDeleted()
        mPreloadedMessages.remove(message)
        persist()
    }

    private fun findMessageById(messageId: String?): Message? {
        for (message in mPreloadedMessages) {
            if (message.messageId == messageId.orEmpty()) {
                return message
            }
        }
        return null
    }

    fun deleteMessageById(messageId: String?): Boolean {
        val messageToBeDeleted =
            findMessageById(messageId)
        return if (messageToBeDeleted != null) {
            deleteMessage(messageToBeDeleted)
            true
        } else {
            false
        }
    }

    fun markReadById(messageId: String): Boolean {
        val message = findMessageById(messageId)
        return if (message != null) {
            message.markRead()
            persist()
            true
        } else {
            false
        }
    }

    fun cleanupExpired() {
        val iterator =
            mPreloadedMessages.iterator()
        while (iterator.hasNext()) {
            val message = iterator.next()
            if (message.isExpired || message.isDeleted) {
                message.markDeleted()
                iterator.remove()
            }
        }
        persist()
    }

    private fun areMessagesUpdated(): Boolean {
        return mPersistentStorage.getBoolean(
            MESSAGES_UPDATED_KEY,
            false
        )
    }

    private fun setMessagesUpdated() {
        mPersistentStorage.edit()
            .putBoolean(MESSAGES_UPDATED_KEY, true).apply()
    }

    companion object {
        private const val TAG = "PreloadedMessageManager"
        private const val PRELOADED_MESSAGE_STORAGE = "preloaded_messages"
        private const val PRELOADED_KEY = "preloaded"
        private const val MESSAGES_UPDATED_KEY = "messages_updated"
        private const val EMPTY_JSON_ARRAY = "[]"
        private const val ASSET_PREFIX = "file:///android_asset/"
    }

    init {
        mPersistentStorage = mContext.getSharedPreferences(
            PRELOADED_MESSAGE_STORAGE,
            Context.MODE_PRIVATE
        )
        val areMessagesUpdated = areMessagesUpdated()
        if (mPersistentStorage.contains(PRELOADED_KEY) && areMessagesUpdated) {
            loadFromStorage()
        } else {
            setupInitialMessages()
            setMessagesUpdated()
        }
    }
}
