package com.asda.android.app.messagecenter.osm

import com.asda.android.app.messagecenter.Message
import com.asda.android.app.messagecenter.OSMReadMessageRequest
import com.asda.android.app.messagecenter.osm.usecase.DeleteOsmMessageUseCase
import com.asda.android.app.messagecenter.osm.usecase.GetOsmMessagesUseCase
import com.asda.android.app.messagecenter.osm.usecase.MarkReadUseCase
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.orders.constants.OrderConstants.EMPTY
import com.asda.android.restapi.config.data
import com.asda.android.restapi.config.succeeded
import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow

object OSMMessageManager {
    const val TAG = "OSMMessageManager"
    val fetchMessageUseCase = GetOsmMessagesUseCase(IO)
    val markReadUseCase = MarkReadUseCase(IO)
    val deleteOSMMessageUseCase = DeleteOsmMessageUseCase(IO)

    /**
     * Function fetches OSM message from endpoint and returns response in flow
     *
     * @return Flow of OSM Message list
     */
    fun fetchOSMMessages() = flow {
        val response = fetchMessageUseCase(Any(), 0, 0)
        if (response.succeeded) {
            response.data?.data?.osm?.messages?.let {
                val messages = it.asSequence()
                    .map { osmMessage ->
                        Message(osmMessage)
                    }.mapTo(mutableListOf()) { message ->
                        message
                    }
                emit(messages.toMutableList())
            } ?: run {
                emit(mutableListOf())
            }
        } else {
            AsdaBaseCoreConfig.asdaLogger.reportNonFatal(Exception("$TAG fetchOSMMessages failed"))
            emit(mutableListOf())
        }
    }.catch {
        emit(mutableListOf())
    }

    /**
     * Function makes message as read and returns response in flow
     *
     * @param request ReadStatusOsmResponse
     * @return Flow of Boolean (isSucceed)
     */
    fun markOSMMessageRead(request: OSMReadMessageRequest) = flow {
        val response = markReadUseCase(request, 0, 0)
        if (response.succeeded) {
            emit(true)
        } else {
            AsdaBaseCoreConfig.asdaLogger.reportNonFatal(Exception("$TAG markOSMMessageRead failed"))
            emit(false)
        }
    }.catch {
        AsdaBaseCoreConfig.asdaLogger.reportNonFatal(Exception("$TAG markOSMMessageRead failed"))
        emit(false)
    }

    /**
     * Function deletes OSM message by message id and returns flow of response
     *
     * @param messageId ID
     * @return Flow of Pair (MessageId, isSucceed)
     */
    suspend fun deleteOSMMessage(id: String?) = flow {
        id?.let { messageId ->
            val response = deleteOSMMessageUseCase(messageId, 0, 0)
            if (response.succeeded) {
                response.data?.data?.osm?.let {
                    if (it.isValidDelete(messageId)) {
                        emit(messageId to true)
                    } else {
                        emit(messageId to false)
                    }
                } ?: run {
                    emit(messageId to false)
                }
            } else {
                emit(messageId to false)
            }
        }
    }.catch {
        AsdaBaseCoreConfig.asdaLogger.reportNonFatal(Exception("$TAG deleteOSMMessage failed"))
        emit(EMPTY to false)
    }

    fun DeleteOsm.isValidDelete(deleteMessageId: String) =
        messageId == deleteMessageId && state == "DELETED"
}
