/*
Copyright 2009-2014 Urban Airship Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE URBAN AIRSHIP INC ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL URBAN AIRSHIP INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.asda.android.app.messagecenter.view

import android.content.Context
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.TextUtils
import android.text.style.DynamicDrawableSpan
import android.text.style.ImageSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.messagecenter.Message
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_OSM
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_PRE_LOAD
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_UA
import com.asda.android.app.messagecenter.formatOSMMessageDate
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.base.core.model.DeepLinkUrlParserInputModel
import com.asda.android.base.core.parser.DeepLinkUrlIdParser
import com.asda.android.base.core.parser.DeepLinkUrlParser
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.utils.setHTMLUrlSpanText
import com.asda.android.imageloader.ImageLoader.load
import com.asda.android.imageloader.ImageLoaderRequest
import com.asda.android.orders.refund.view.utils.gone
import com.asda.android.orders.refund.view.utils.visible
import com.asda.android.utils.view.ViewUtil
import java.util.ArrayList

class MessageAdapter(
    private val mContext: Context,
    private val mMessageSelector: MessageSelector
) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private val ITEM_UA_MESSAGE = 0
    private val ITEM_OSM_MESSAGE = 1

    interface MessageSelector {
        fun isMessageSelected(messageId: String): Boolean
    }

    interface OnItemClickListener {
        fun onItemClick(
            message: Message,
            position: Int,
            clickedView: View
        )
    }

    private var mOnItemClickListener: OnItemClickListener? = null
    private val mMessages: ArrayList<Message> = ArrayList()

    fun getItem(position: Int): Any {
        return mMessages[position]
    }

    override fun onCreateViewHolder(
        parent: ViewGroup,
        viewType: Int
    ): RecyclerView.ViewHolder {
        return when (viewType) {
            ITEM_UA_MESSAGE -> {
                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.inbox_list_item, parent, false)
                val spacing =
                    mContext.resources.getDimensionPixelSize(R.dimen.messages_grid_spacing)
                val numColumns =
                    mContext.resources.getInteger(R.integer.messages_grid_columns)
                val width =
                    if (numColumns > 1) ViewUtil.getScreenWidth(mContext) / numColumns - spacing / 2 else ViewUtil.getScreenWidth(
                        mContext
                    )
                val params = view.findViewById<View>(R.id.image).layoutParams
                params.width = width
                params.height = (MSG_IMAGES_ASPECT_RATIO * width).toInt()
                MessageViewHolder(view)
            }
            ITEM_OSM_MESSAGE -> {
                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.osm_inbox_list_item, parent, false)
                OSMMessageViewHolder(view)
            }
            else -> throw IllegalStateException("Illegal item added")
        }
    }

    override fun onBindViewHolder(
        holder: RecyclerView.ViewHolder,
        position: Int
    ) {
        when {
            getItemViewType(position) == ITEM_UA_MESSAGE -> {
                (holder as MessageViewHolder).bind(getMessage(position))
            }
            getItemViewType(position) == ITEM_OSM_MESSAGE -> {
                (holder as OSMMessageViewHolder).bind(getMessage(position))
            }
        }
    }

    override fun getItemId(position: Int): Long {
        return mMessages[position].messageId.hashCode().toLong()
    }

    fun getMessage(position: Int): Message {
        return mMessages[position]
    }

    override fun getItemViewType(position: Int): Int {
        val message = getMessage(position)
        return when (message.messageType) {
            MSG_TYPE_OSM -> ITEM_OSM_MESSAGE
            MSG_TYPE_UA,
            MSG_TYPE_PRE_LOAD
            -> ITEM_UA_MESSAGE
            else -> ITEM_UA_MESSAGE
        }
    }

    override fun getItemCount(): Int {
        return mMessages.size
    }

    fun set(collection: Collection<Message>) {
        mMessages.clear()
        mMessages.addAll(collection)
        notifyItemRangeChanged(0, itemCount)
    }

    fun setOnItemClickListener(listener: OnItemClickListener) {
        mOnItemClickListener = listener
    }

    inner class MessageViewHolder(itemView: View) :
        RecyclerView.ViewHolder(itemView),
        View.OnClickListener {
        val title: TextView = ViewUtil.findViewById(itemView, R.id.title)
        val subTitle: TextView = ViewUtil.findViewById(itemView, R.id.sub_title)
        val textArea: View = ViewUtil.findViewById(itemView, R.id.text_area)
        val expiry: TextView = ViewUtil.findViewById(itemView, R.id.expiry)
        val image: ImageView = ViewUtil.findViewById(itemView, R.id.image)
        val action: TextView = ViewUtil.findViewById(itemView, R.id.action)

        fun bind(message: Message) {
            val iconUrl = message.iconUrl
            if (iconUrl != null) {
                load(
                    mContext,
                    image,
                    ImageLoaderRequest.Builder().path(iconUrl).fit(true)
                        .build(),
                    null
                )
            } else {
                image.setImageBitmap(null)
            }
            title.text = message.title
            val subTitleText = message.subTitle
            subTitle.visibility = if (TextUtils.isEmpty(subTitleText)) View.GONE else View.VISIBLE
            subTitle.text = subTitleText
            val expiryText = message.getExpiry(mContext)
            if (expiryText != null) {
                expiry.text = expiryText
                expiry.visibility = View.VISIBLE
            } else {
                expiry.visibility = View.GONE
            }
            action.text = message.getAction(mContext)
            itemView.isSelected = mMessageSelector.isMessageSelected(message.messageId.orEmpty())
        }

        override fun onClick(v: View) {
            mOnItemClickListener?.let {
                val clickedPosition = adapterPosition
                if (clickedPosition != RecyclerView.NO_POSITION) {
                    val message = getMessage(clickedPosition)
                    it.onItemClick(message, clickedPosition, v)
                }
            }
        }

        init {
            itemView.setOnClickListener(this)
        }
    }

    companion object {
        private const val MSG_IMAGES_ASPECT_RATIO = 0.847f
    }

    inner class OSMMessageViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        private val titleText: TextView = ViewUtil.findViewById(itemView, R.id.titleTxt)
        private val dateText: TextView = ViewUtil.findViewById(itemView, R.id.dateText)
        private val contentText: TextView = ViewUtil.findViewById(itemView, R.id.contentText)

        init {
            itemView.setOnClickListener {
                mOnItemClickListener?.let {
                    val clickedPosition = adapterPosition
                    if (clickedPosition != RecyclerView.NO_POSITION) {
                        val message = getMessage(clickedPosition)
                        it.onItemClick(message, clickedPosition, itemView)
                    }
                }
            }
        }

        fun bind(message: Message) {
            setTitleText(message)
            if (message.content.isNullOrBlank().not()) {
                contentText.visible()
                contentText.setHTMLUrlSpanText(message.content.orEmpty()) {
                    handleCTAClick(it, itemView.context)
                }
            } else {
                contentText.gone()
            }
            dateText.apply {
                try {
                    text = formatOSMMessageDate(context, message.sentDateMS)
                    visible()
                } catch (e: Exception) {
                    gone()
                    AsdaBaseCoreConfig.asdaLogger.reportNonFatal(Exception("formatOSMMessageDate failed"))
                }
            }
            itemView.isSelected = mMessageSelector.isMessageSelected(message.messageId.orEmpty())
        }

        private fun setTitleText(message: Message) {
            message.title?.let {
                titleText.apply {
                    val secondaryRatings = SpannableStringBuilder(it)
                    if (message.readStatus.not()) {
                        secondaryRatings.append(context.getString(R.string.four_spaces))
                        secondaryRatings.setSpan(
                            ImageSpan(
                                context,
                                R.drawable.badge_red_osm,
                                DynamicDrawableSpan.ALIGN_BASELINE
                            ),
                            secondaryRatings.length - 1,
                            secondaryRatings.length,
                            Spanned.SPAN_INCLUSIVE_INCLUSIVE
                        )
                    }
                    text = secondaryRatings
                }
            }
        }

        private fun handleCTAClick(url: String, context: Context) {
            if (url.isNotBlank()) {
                context.tryCast<AsdaActivity> {
                    val deepLinkUrlParser = DeepLinkUrlParser()
                    val deepLinkUrlIdParser = DeepLinkUrlIdParser()

                    // Identifies the url type
                    val type = deepLinkUrlParser.parse(DeepLinkUrlParserInputModel(url, null))
                    // Checks for url details like order id
                    val urlDetails = deepLinkUrlIdParser.parse(
                        DeepLinkUrlParserInputModel(url, null)
                    )
                    // Function handles all the deep-link redirection in the app
                    MerchandisingHandler.handleCmsMerchandisingEvent(
                        this,
                        null,
                        null,
                        type,
                        urlDetails.first,
                        null,
                        urlDetails.second,
                        url
                    )
                }
            }
        }
    }
}
