/*
Copyright 2009-2014 Urban Airship Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE URBAN AIRSHIP INC ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL URBAN AIRSHIP INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.asda.android.app.messagecenter.view

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.content.res.Configuration
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.view.animation.DecelerateInterpolator
import android.widget.Button
import android.widget.ProgressBar
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.view.ActionMode
import androidx.appcompat.widget.PopupMenu
import androidx.core.view.ViewCompat
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.PV_NOTIFICATIONS
import com.asda.android.analytics.events.AsdaEngineeringAnalyticsEvent
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.analytics.info.AnalyticsBannerInfo
import com.asda.android.app.base.airship.view.AirshipPushListener.Companion.setTag
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.main.view.SplashActivity
import com.asda.android.app.messagecenter.Message
import com.asda.android.app.messagecenter.Message.Companion.MSG_TYPE_OSM
import com.asda.android.app.messagecenter.MessageCenterController
import com.asda.android.app.messagecenter.MessageCenterController.OnInboxUpdateListener
import com.asda.android.app.messagecenter.view.MessageAdapter.MessageSelector
import com.asda.android.app.settings.featureflag.FeatureSettings.isOSMMessageServiceEnabled
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP_OLD
import com.asda.android.base.core.constants.FragmentConfigConstants
import com.asda.android.base.core.constants.ShopConstants
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.orFalse
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.restapi.service.constants.MerchandisingConstants
import com.asda.android.restapi.service.data.PushNotificationEvent
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.utils.view.ViewUtil
import kotlinx.coroutines.launch
import java.util.ArrayList

class MessageCenterFragment :
    FeaturedFragment(),
    ActionMode.Callback,
    OnInboxUpdateListener,
    MessageSelector {
    private var mActionMode: ActionMode? = null
    private var mActionSelectionButton: Button? = null
    lateinit var mEmptyView: View
    lateinit var mRecyclerView: RecyclerView
    lateinit var mAdapter: MessageAdapter
    private val mSelectedMessageList: MutableList<Message> = mutableListOf()
    lateinit var mLayoutManager: GridLayoutManager
    private var mActivity: AsdaActivity? = null
    private var mContainer: ViewGroup? = null
    lateinit var mView: View
    lateinit var safeContext: Context
    lateinit var progressBar: ProgressBar
    override fun onAttach(activity: Context) {
        super.onAttach(activity)
        activity.tryCast<AsdaActivity> {
            mActivity = this
        }
    }

    override fun onResume() {
        super.onResume()
        val controller =
            MessageCenterController.getInstance(activity)
        controller?.addOnInboxUpdateListener(this)
        startActionModeIfNecessary()
        controller?.refreshMessages()
        Tracker.get()
            .trackPageView(
                PageViewEvent(PV_NOTIFICATIONS, null, null)
                    .putString(
                        Anivia.STOREID_KEY,
                        AsdaServiceFactory.get().lastStoreIdFromSessionMetadata
                    )
            )
    }

    override fun onPause() {
        super.onPause()
        val controller =
            MessageCenterController.getInstance(activity)
        controller?.removeOnInboxUpdateListener(this)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mContainer = container
        mView = inflater.inflate(R.layout.msg_center_fragment, container, false)
        context?.let { safeContext = it }
        mEmptyView = mView.findViewById(R.id.empty)
        mRecyclerView =
            ViewUtil.findViewById(
                mView,
                R.id.list_view
            )
        mLayoutManager = GridLayoutManager(
            activity, resources.getInteger(R.integer.messages_grid_columns)
        )
        mRecyclerView.addItemDecoration(
            object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    val spacing =
                        resources.getDimensionPixelSize(R.dimen.messages_grid_spacing)
                    val pos = mRecyclerView.getChildAdapterPosition(view)
                    val columns = mLayoutManager.spanCount
                    outRect.top = spacing
                    if (columns > 1) {
                        val mod = pos % columns
                        if (mod < columns - 1) {
                            outRect.right = spacing / 2
                        }
                        if (mod != 0 && mod < columns) {
                            outRect.left = spacing / 2
                        }
                    }
                }
            })
        mRecyclerView.layoutManager = mLayoutManager
        mAdapter = MessageAdapter(safeContext, this)
        mRecyclerView.adapter = mAdapter
        mAdapter.setOnItemClickListener(object : MessageAdapter.OnItemClickListener {
            override fun onItemClick(message: Message, position: Int, clickedView: View) {
                if (mActionMode != null) { // in selection mode
                    onMessageSelected(message)
                } else {
                    onMessageTapped(message, clickedView)
                }
            }
        })

        return mView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        progressBar = ViewUtil.findViewById(view, R.id.message_progress_bar)
        progressBar.visible()
        loadMessages()
    }

    private fun onMessageTapped(
        message: Message,
        clickedView: View
    ) {
        if (message.messageType == MSG_TYPE_OSM) return
        lifecycleScope.launch {
            val controller =
                MessageCenterController.getInstance(activity)
            controller?.markRead(message.messageId)
            val wasActionHandled = handleMessageAction(message, clickedView)
            if (!wasActionHandled) {
                MessageDetailsActivity.startActivity(requireActivity(), message.messageId)
                Tracker.get()
                    .trackEvent(
                        AsdaEngineeringAnalyticsEvent(Anivia.MESSAGE_TAP_EVENT)
                            .putString(Anivia.MSG_DEST_KEY, "landing page")
                            .putString(
                                Anivia.MSG_ID_KEY,
                                message.title
                            )
                            .putString(Anivia.MSG_TYPE_KEY, "message center message")
                    )
            }
        }
    }

    private fun handleMessageAction(
        message: Message?,
        clickedView: View
    ): Boolean {
        var activityLaunched = false
        if (message == null) {
            return activityLaunched
        }
        val extras = message.extras
        val actionUri =
            if (extras != null) extras.getString(Message.ACTION_URI_EXTRA) else ""
        if (TextUtils.isEmpty(actionUri)) {
            return activityLaunched
        }
        try {
            val uri = Uri.parse(actionUri)
            val launch = Intent()
            if (SplashActivity.prepareDeepLinkingIntent(launch, uri.scheme, uri)) {
                val pushEvent = PushNotificationEvent(launch.extras)
                if (!MerchandisingHandler.handleMerchandisingEvent(
                        mActivity,
                        null,
                        pushEvent,
                        null,
                        AnalyticsBannerInfo("notification center", "mobile banner", "slot 0")
                            .buildAnalyticsBannerInfoString()
                    )
                ) {
                    // Received push action should be handled by the SHOP fragment instead
                    val bundle = Bundle()
                    bundle.putParcelable(ShopConstants.ARG_PUSH_NOTIFICATION, launch.extras)
                    bundle.putString(
                        ShopConstants.ARG_SOURCE,
                        "notification center : product offer card"
                    )
                    bundle.putString(
                        ShopConstants.ARG_PREVIOUS_FRAGMENT, FragmentConfigConstants.TAG_MSG_CENTER
                    )
                    val fragmentSwitcher = mActivity?.getFragmentSwitcher()
                    context?.let {
                        if (AsdaBaseCoreConfig.featureSettingManager.isNewTaxonomyEnabled(it)) {
                            fragmentSwitcher?.switchToFragment(TAG_SHOP, bundle)
                        } else {
                            fragmentSwitcher?.switchToFragment(TAG_SHOP_OLD, bundle)
                        }
                    }
                }
            } else if (uri.scheme?.startsWith("http").orFalse()) {
                com.asda.android.base.core.view.ViewUtil.openWebPage(
                    actionUri,
                    activity,
                    SCREEN_NAME
                )
            }
            activityLaunched = true
            val actionTags =
                if (extras != null) extras.getString(Message.ACTION_TAG_EXTRA) else ""
            setTag(actionTags)
            Tracker.get()
                .trackEvent(
                    AsdaEngineeringAnalyticsEvent(Anivia.MESSAGE_TAP_EVENT)
                        .putString(Anivia.MSG_DEST_KEY, actionUri)
                        .putString(Anivia.MSG_ID_KEY, message.title)
                        .putString(Anivia.MSG_TYPE_KEY, "message center message")
                )
        } catch (e: Exception) {
            Log.w(
                TAG,
                "Error taking action=$actionUri",
                e
            )
        }
        return activityLaunched
    }

    private fun setTag(actionTags: String?) {
        actionTags?.let { actionTag ->
            if (TextUtils.isEmpty(actionTags).not()) {
                for (tag in actionTag.split(";")) {
                    val tagAfterTrim = tag.trim { it <= ' ' }
                    if (!TextUtils.isEmpty(tagAfterTrim)) {
                        setTag(tagAfterTrim, true)
                    }
                }
            }
        }
    }

    override fun onConfigurationChanged(configuration: Configuration) {
        super.onConfigurationChanged(configuration)
        if (mContainer != null && mEmptyView.visibility == View.VISIBLE
        ) {
            mContainer?.removeView(mView)
            mContainer?.addView(onCreateView(LayoutInflater.from(activity), mContainer, null))
        } else {
            val numColumns = resources.getInteger(R.integer.messages_grid_columns)
            if (mLayoutManager.spanCount != numColumns) {
                mLayoutManager.spanCount = numColumns
            }
            mRecyclerView.adapter = mAdapter // Force view refresh to recalculate the card width
        }
    }

    private fun clearSelection() {
        mSelectedMessageList.clear()
        mAdapter.notifyDataSetChanged()
        finishActionMode()
    }

    private fun finishActionMode() {
        mActionMode?.finish()
    }

    private fun startActionModeIfNecessary() {
        val checkedIds = selectedMessages
        if (mActionMode != null && checkedIds.isEmpty()) {
            mActionMode?.finish()
        } else if (mActionMode == null && checkedIds.isNotEmpty()) {
            mActionMode =
                (this.activity as AppCompatActivity?)?.startSupportActionMode(
                    this
                )
        }
    }

    private fun loadMessages() {
        val controller = MessageCenterController.getInstance(activity) ?: return
        lifecycleScope.launch {
            val messages = controller.getAllMessages()
            progressBar.gone()
            setMessagesToAdapter(messages)
            val isEmpty = mAdapter.itemCount == 0
            mRecyclerView.visibility = if (isEmpty) View.GONE else View.VISIBLE
            if (isEmpty && mEmptyView.visibility == View.GONE) {
                mEmptyView.visibility = View.VISIBLE
                animateMessage()
            }
            mEmptyView.visibility = if (isEmpty) View.VISIBLE else View.GONE
            // As soon as it loads fire read API for all OSM messages
            if (isOSMMessageServiceEnabled(requireContext())) {
                controller.markOSMMessagesRead(messages.toList())
            }
        }
    }

    private fun animateMessage() {
        val background = mEmptyView.findViewById<View>(R.id.background)
        val icon = mEmptyView.findViewById<View>(R.id.icon)
        val shadow = mEmptyView.findViewById<View>(R.id.shadow)
        val title = mEmptyView.findViewById<View>(R.id.empty_msg_title)
        val subTitle = mEmptyView.findViewById<View>(R.id.empty_msg_subtitle)
        val screenHeight =
            ViewUtil.getDisplayMetrics(mActivity as Context).heightPixels
        background.scaleX = 0f
        background.scaleY = 0f
        icon.translationY = -screenHeight.toFloat()
        title.translationY = screenHeight.toFloat()
        subTitle.translationY = screenHeight.toFloat()
        shadow.alpha = 0f
        val duration = 250
        ViewCompat.animate(background)
            .scaleX(1f)
            .scaleY(1f)
            .setDuration(duration.toLong())
            .withEndAction withEndAction1@{
                if (!isAdded) {
                    return@withEndAction1
                }
                val interpolator =
                    DecelerateInterpolator()
                icon.animate().translationY(0f).setInterpolator(interpolator).duration =
                    duration.toLong()
                title.animate().translationY(0f).setInterpolator(interpolator).duration =
                    duration.toLong()
                ViewCompat.animate(subTitle)
                    .translationY(0f)
                    .setInterpolator(interpolator)
                    .setDuration(duration.toLong())
                    .withEndAction withEndAction2@{
                        if (!isAdded) {
                            return@withEndAction2
                        }
                        shadow.animate().alpha(1f).duration = duration.toLong()
                    }
            }
    }

    private fun setMessagesToAdapter(messages: Collection<Message>) {
        val list: MutableCollection<Message> =
            ArrayList()
        for (message in messages) {
            if (!getMessageType(message).equals(
                    MerchandisingConstants.TYPE_RECIPES_ITEM,
                    ignoreCase = true
                )
            ) {
                list.add(message)
            }
        }
        mAdapter.set(list)
    }

    private fun getMessageType(message: Message?): String? {
        var type = ""
        if (message != null) {
            val extras = message.extras
            val actionUri =
                if (extras != null) extras.getString(Message.ACTION_URI_EXTRA) else ""
            if (!TextUtils.isEmpty(actionUri)) {
                try {
                    val uri = Uri.parse(actionUri)
                    val launch = Intent()
                    if (SplashActivity.prepareDeepLinkingIntent(launch, uri.scheme, uri)) {
                        val pushEvent =
                            PushNotificationEvent(launch.extras)
                        if (pushEvent.type != null) {
                            type = pushEvent.type.orEmpty()
                        }
                    }
                } catch (e: Exception) {
                    Log.w(
                        TAG,
                        "Error taking action=$actionUri",
                        e
                    )
                }
            }
        }
        return type
    }

    private fun selectAll() {
        mSelectedMessageList.clear()
        val count = mAdapter.itemCount
        for (i in 0 until count) {
            mSelectedMessageList.add(mAdapter.getMessage(i))
        }
        mAdapter.notifyDataSetChanged()
        mActionMode?.invalidate()
    }

    private fun onMessageSelected(message: Message) {
        mSelectedMessageList.firstOrNull {
            it.messageId == message.messageId
        }?.let {
            mSelectedMessageList.remove(message)
        } ?: kotlin.run {
            mSelectedMessageList.add(message)
        }
        mAdapter.notifyDataSetChanged()
        startActionModeIfNecessary()
        mActionMode?.invalidate()
    }

    override fun onCreateActionMode(
        mode: ActionMode,
        menu: Menu
    ): Boolean {
        mode.menuInflater.inflate(R.menu.msg_center_cab_actions_menu, menu)
        @SuppressLint("InflateParams") val customView =
            LayoutInflater.from(activity).inflate(R.layout.cab_selection_dropdown, null)
        mActionSelectionButton = customView.findViewById(R.id.selection_button)
        val popupMenu =
            PopupMenu(safeContext, customView)
        popupMenu.menuInflater.inflate(R.menu.selection, popupMenu.menu)
        popupMenu.setOnMenuItemClickListener { item: MenuItem ->
            if (item.itemId == R.id.menu_deselect_all) {
                clearSelection()
            } else {
                selectAll()
            }
            true
        }
        mActionSelectionButton?.setOnClickListener { arg0: View? ->
            val menu1 = popupMenu.menu
            menu1.findItem(R.id.menu_deselect_all).isVisible = true
            menu1
                .findItem(R.id.menu_select_all).isVisible =
                selectedMessages.size != mAdapter.itemCount
            popupMenu.show()
        }
        mode.customView = customView
        return true
    }

    override fun onPrepareActionMode(
        mode: ActionMode,
        menu: Menu
    ): Boolean {
        // If we have an action selection_popup button update the text
        if (mActionSelectionButton != null) {
            val selectionText =
                this.getString(R.string.cab_selection, selectedMessages.size)
            mActionSelectionButton?.text = selectionText
        }
        return true
    }

    override fun onActionItemClicked(
        mode: ActionMode,
        item: MenuItem
    ): Boolean {
        if (item.itemId == R.id.delete) {
            lifecycleScope.launch {
                progressBar.visible()
                val controller =
                    MessageCenterController.getInstance(this@MessageCenterFragment.safeContext)
                controller?.deleteMessagesById(selectedMessages) {
                    progressBar.gone()
                }
            }
        } else {
            return false
        }
        finishActionMode()
        return true
    }

    override fun onDestroyActionMode(mode: ActionMode) {
        if (mActionMode != null) {
            mActionMode = null
            clearSelection()
        }
    }

    override fun onInboxUpdate() {
        loadMessages()
        mActionMode?.invalidate()
    }

    override fun onDetach() {
        super.onDetach()
        val controller =
            MessageCenterController.getInstance(activity)
        controller?.cleanupMessagesIfNeeded()
    }

    override fun onStop() {
        super.onStop()
        clearSelection()
    }

    private val selectedMessages: MutableList<Message>
        get() = mSelectedMessageList

    override fun isMessageSelected(messageId: String): Boolean {
        return mSelectedMessageList.firstOrNull {
            it.messageId == messageId
        }?.let {
            true
        } ?: kotlin.run {
            false
        }
    }

    override fun onBackPressed(): Boolean {
        if (mActionMode != null) {
            mActionMode?.finish()
            return true
        }
        return super.onBackPressed()
    }

    companion object {
        private const val TAG = "MessageCenterFragment"
        private const val SCREEN_NAME = "Message Center"
    }
}
