/*
Copyright 2009-2014 Urban Airship Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE URBAN AIRSHIP INC ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL URBAN AIRSHIP INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.asda.android.app.messagecenter.view

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.annotation.SuppressLint
import android.graphics.Color
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.ConsoleMessage
import android.webkit.JavascriptInterface
import android.webkit.WebChromeClient
import android.webkit.WebChromeClient.CustomViewCallback
import android.webkit.WebSettings
import android.webkit.WebView
import android.widget.FrameLayout
import android.widget.ProgressBar
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.base.core.utils.orFalse
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants.debug
import com.asda.android.utils.view.ViewUtil
import com.urbanairship.UAirship
import com.urbanairship.richpush.RichPushMessage
import com.urbanairship.widget.UAWebView
import com.urbanairship.widget.UAWebViewClient

class UAMessageFragment : Fragment() {

    private class AsdaJavaScriptBridge {
        @JavascriptInterface
        fun trackAnalyticEvent(jsonEvent: String?) {
            Handler(Looper.getMainLooper()).post {
                Tracker.get().trackAniviaJsonEvent(jsonEvent)
            }
        }
    }

    lateinit var mBrowser: UAWebView
    lateinit var mProgressBar: ProgressBar
    private var mMessage: RichPushMessage? = null
    private var mCustomViewContainer: FrameLayout? = null
    private var mCustomViewCallback: CustomViewCallback? = null
    private var mCustomView: View? = null
    private var mWebChromeClient: CustomWebChromeClient? = null
    private var mMessageId: String? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
        mMessageId = arguments?.getString(MESSAGE_ID_KEY)
    }

    @SuppressLint("AddJavascriptInterface")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(R.layout.fragment_message, container, false)
        mProgressBar =
            ViewUtil.findViewById(
                view,
                R.id.progress_bar
            )
        mBrowser =
            ViewUtil.findViewById(view, R.id.message_view)
        mCustomViewContainer = ViewUtil.findViewById(
            view,
            R.id.customViewContainer
        )
        mBrowser.alpha = 0f

        // Set a custom RichPushWebViewClient view client to listen for the page finish
        // Note: UAWebViewClient is required to load the proper auth and to
        // inject the Urban Airship Javascript interface.  When overriding any methods
        // make sure to call through to the super's implementation.
        mBrowser.webViewClient = object : UAWebViewClient() {
            override fun onPageFinished(view: WebView?, url: String?) {
                super.onPageFinished(view, url)
                showMessage()
            }

            override fun shouldOverrideUrlLoading(view: WebView, url: String?): Boolean {
                // Prevent webview from navigating away
                return url?.startsWith("http").orFalse() || super.shouldOverrideUrlLoading(view, url)
            }
        }
        mWebChromeClient = CustomWebChromeClient()
        mBrowser.webChromeClient = mWebChromeClient
        mBrowser.settings?.setAppCacheEnabled(true)
        mBrowser.settings?.cacheMode = WebSettings.LOAD_CACHE_ELSE_NETWORK
        mBrowser.addJavascriptInterface(AsdaJavaScriptBridge(), "AsdaBridge")
        return view
    }

    override fun onStart() {
        super.onStart()
        UAirship.shared { uAirship: UAirship ->
            mMessage = uAirship.inbox.getMessage(mMessageId)
            if (mMessage == null) {
                Log.w(
                    TAG,
                    "Couldn't retrieve message for ID: $mMessageId"
                )
                activity?.finish()
            } else {
                mBrowser.loadRichPushMessage(mMessage)
                if (debug) {
                    val user = uAirship.inbox.user
                    Log.d(
                        TAG,
                        "Message URL = " +
                            mMessage?.messageUrl +
                            " id = " +
                            user.id +
                            " pw = " +
                            user.password
                    )
                }
            }
        }
    }

    override fun onResume() {
        super.onResume()
        mBrowser.onResume()
    }

    override fun onPause() {
        super.onPause()
        mBrowser.onPause()
    }

    private fun showMessage() {
        mBrowser.animate()?.alpha(1f)?.setDuration(200)?.setListener(null)

        // Animate the loading view to 0% opacity. After the animation ends,
        // set its visibility to GONE as an optimization step (it won't
        // participate in layout passes, etc.)
        mProgressBar
            .animate()
            .alpha(0f)
            .setDuration(200)
            .setListener(
                object : AnimatorListenerAdapter() {
                    override fun onAnimationEnd(animation: Animator) {
                        mProgressBar.visibility = View.GONE
                    }
                })
    }

    internal inner class CustomWebChromeClient : WebChromeClient() {
        private var mVideoProgressView: View? = null
        override fun onShowCustomView(
            view: View,
            callback: CustomViewCallback
        ) {
            enterFullScreelMode()

            // if a view already exists then immediately terminate the new one
            if (mCustomView != null) {
                callback.onCustomViewHidden()
                return
            }
            mCustomView = view
            mCustomView?.setBackgroundColor(Color.BLACK)
            mBrowser.visibility = View.GONE
            mCustomViewContainer?.visibility = View.VISIBLE
            mCustomViewContainer?.addView(view)
            mCustomViewCallback = callback
            view.isFocusableInTouchMode = true
            view.requestFocus()
            view.setOnKeyListener { v: View?, keyCode: Int, event: KeyEvent? ->
                if (keyCode == KeyEvent.KEYCODE_BACK) {
                    if (inCustomView()) {
                        hideCustomView()
                        return@setOnKeyListener true
                    }
                    if (mCustomView == null && mBrowser.canGoBack()) {
                        mBrowser.goBack()
                        return@setOnKeyListener true
                    }
                }
                false
            }
        }

        @SuppressLint("InflateParams")
        override fun getVideoLoadingProgressView(): View? {
            if (mVideoProgressView == null) {
                val inflater = LayoutInflater.from(activity)
                mVideoProgressView = inflater.inflate(R.layout.video_progress, null)
            }
            return mVideoProgressView
        }

        override fun onHideCustomView() {
            super.onHideCustomView()
            leaveFullScreenMode()
            if (mCustomView == null) {
                return
            }
            mBrowser.visibility = View.VISIBLE
            mCustomViewContainer?.visibility = View.GONE

            // Hide the custom view.
            mCustomView?.visibility = View.GONE
            mCustomView?.setOnKeyListener(null)

            // Remove the custom view from its container.
            mCustomViewContainer?.removeView(mCustomView)
            mCustomViewCallback?.onCustomViewHidden()
            mCustomView = null
        }

        override fun onConsoleMessage(consoleMessage: ConsoleMessage): Boolean {
            if (debug) {
                Log.w(
                    TAG,
                    "MessageFragment console = " + consoleMessage.message()
                )
            }
            return super.onConsoleMessage(consoleMessage)
        }

        private fun enterFullScreelMode() {
            activity
                ?.window
                ?.decorView
                ?.systemUiVisibility = (
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                )
        }

        private fun leaveFullScreenMode() {
            activity
                ?.window
                ?.decorView
                ?.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
        }

        private fun inCustomView(): Boolean {
            return mCustomView != null
        }

        private fun hideCustomView() {
            mWebChromeClient?.onHideCustomView()
        }
    }

    companion object {
        private const val MESSAGE_ID_KEY = "message_id"
        private const val TAG = "UAMessageFragment"
        fun newInstance(messageId: String?): UAMessageFragment {
            val message = UAMessageFragment()
            val arguments = Bundle()
            arguments.putString(MESSAGE_ID_KEY, messageId)
            message.arguments = arguments
            return message
        }
    }
}
