package com.asda.android.app.orders

import android.content.Context
import android.graphics.Paint
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.cncgeofence.utils.Formatters.sDateFormat
import com.asda.android.app.orders.ods.convertTime
import com.asda.android.app.orders.ods.getOrderService
import com.asda.android.app.orders.ods.mapOdsHistoryToViewOrder
import com.asda.android.app.orders.ods.mapOdsOrderToWismoOrder
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.ViewConstants
import com.asda.android.base.core.view.adapter.HeaderRecyclerViewAdapter
import com.asda.android.cxo.CartManager
import com.asda.android.home.AsdaHomeConfig
import com.asda.android.orders.wismo.constants.WismoCardConstants
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants.debug
import com.asda.android.restapi.service.constants.SlotConstants
import com.asda.android.restapi.service.constants.ViewOrderConstants
import com.asda.android.restapi.service.data.OdsOrderModel
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.service.base.AsdaServiceFactory.get
import com.asda.android.service.base.Authentication
import com.asda.android.service.constants.CartManagerConstants
import com.asda.android.utils.Utils.convertTimeFormat
import com.asda.android.utils.Utils.fullDateFormatter
import com.asda.android.utils.view.ViewUtil.findViewById
import com.jakewharton.retrofit2.adapter.rxjava2.HttpException
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers
import java.text.ParseException

class OrderListAdapter(private val mContext: Context, filterDpOrders: Boolean) :
    HeaderRecyclerViewAdapter() {
    private val mOrders: ArrayList<ViewOrderResponse.Order>
    private val mInflater: LayoutInflater
    private var mLoadingEntry: View? = null
    private var mLoadingEntryPosition = 0
    private var mIsLoading: Boolean
    private var mIsInitialized: Boolean = false
    private var mHasMoreItems: Boolean

    // offset logic is used with ODS
    private var offset: Int

    // page logic is used with legacy API
    private var mNextPage: Int
    private var mOnInitialItemsLoadedListener: OnInitialItemsLoadedListener? = null
    private var mFilterDpOrders: Boolean
    private var mOnItemClickListener: OnItemClickListener? = null
    private val mLoadFinish: OnLoadFinished?
    private val mOrdersQueue: ArrayList<String?>
    fun reload(filterDpOrders: Boolean) {
        mOrders.clear()
        mOrdersQueue.clear()
        offset = 0
        mNextPage = 1
        mIsLoading = false
        mHasMoreItems = true
        mIsInitialized = false
        mFilterDpOrders = filterDpOrders
        loadMoreEntries()
        notifyDataSetChanged()
    }

    override fun getItemId(position: Int): Long {
        return 0
    }

    override fun getItemCount(): Int {
        var count = super.getItemCount()
        count += mOrders.size
        mLoadingEntryPosition = if (mHasMoreItems) {
            ++count
            count - 1
        } else {
            -1
        }
        return count
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val viewHolder = super.onCreateViewHolder(parent, viewType)
        if (viewHolder != null) {
            return viewHolder
        }
        val view: View?
        return if (viewType == TYPE_LOADING_ENTRY) {
            if (mLoadingEntry == null) {
                mLoadingEntry = mInflater.inflate(R.layout.store_list_loading_entry, parent, false)
            }
            view = mLoadingEntry
            BaseViewHolder(view, viewType)
        } else {
            view = mInflater.inflate(R.layout.order_list_entry, parent, false)
            val baseViewHolder: BaseViewHolder = OrderItemViewHolder(view, viewType)
            baseViewHolder.itemView.setOnClickListener {
                if (mOnItemClickListener != null) {
                    mOnItemClickListener?.onItemClick(
                        translatePositionToBase(baseViewHolder.adapterPosition)
                    )
                }
            }
            baseViewHolder
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, recyclerPosition: Int) {
        if (holder.itemViewType == TYPE_ORDER_ENTRY) {
            val position = translatePositionToBase(recyclerPosition)
            val orderItemViewHolder = holder as OrderItemViewHolder
            populateFields(position, orderItemViewHolder)
        }
    }

    override fun onViewRecycled(holder: RecyclerView.ViewHolder) {
        val order = getOrder(holder.adapterPosition)
        if (order != null && WismoCardConstants.ORDER_LOADED_TO_VAN == order.actualOrderState) {
            get().cancelRequest(order.orderId)
        }
    }

    override fun getItemViewType(position: Int): Int {
        if (isHeaderPosition(position)) {
            return ViewConstants.TYPE_LIST_HEADERS - position
        } else if (isFooterPosition(position)) {
            return ViewConstants.TYPE_LIST_FOOTER
        }
        return if (position == mLoadingEntryPosition) {
            TYPE_LOADING_ENTRY
        } else {
            TYPE_ORDER_ENTRY
        }
    }

    fun getOrder(position: Int): ViewOrderResponse.Order? {
        try {
            return mOrders[position]
        } catch (e: Exception) {
            Log.e(TAG, "no order at $position")
        }
        return null
    }

    val ordersCount: Int
        get() = mOrders.size

    private fun populateFields(position: Int, viewHolder: OrderItemViewHolder) {
        val order = mOrders[position]
        val dateText = viewHolder.dateText
        val orderIdText = viewHolder.orderIdText
        val slotTypeText = viewHolder.slotTypeText
        val orderStatusText = viewHolder.orderStatusText
        val orderCostText = viewHolder.orderCostText
        if (order._slotDateFull == null) {
            if (!order.isDeliveryPassPurchase) { // delivery pass purchasing orders have
                // deliveryStartTime & deliveryEndTime equal to "00:00"
                order.slotDate = convertTime(order.slotDate)
                order._slotDateFull = order.slotDate
                order._slotDateFull += (
                    ", " +
                        convertTimeFormat(
                            DateFormats.HH_MM, DateFormats.H_MM_AA, order.deliveryStartTime
                        )
                    )
                if (!TextUtils.isEmpty(order.deliveryEndTime)) {
                    order._slotDateFull += (
                        "-" +
                            convertTimeFormat(
                                DateFormats.HH_MM, DateFormats.H_MM_AA, order.deliveryEndTime
                            )
                        )
                }
            } else {
                try {
                    order._slotDateFull = sDateFormat
                        .format(fullDateFormatter.parse(order.orderedDate))
                    order._slotDateFull += (
                        ", " +
                            convertTimeFormat(
                                DateFormats.HH_MM, DateFormats.H_MM_AA, order.deliveryStartTime
                            )
                        )
                    if (!TextUtils.isEmpty(order.deliveryEndTime)) order._slotDateFull += (
                        "-" +
                            convertTimeFormat(
                                DateFormats.HH_MM, DateFormats.H_MM_AA, order.deliveryEndTime
                            )
                        )
                } catch (e: ParseException) {
                    Log.e(TAG, "Failed to parse order date for DP: '" + order.orderedDate + "'", e)
                    order._slotDateFull = order.orderedDate
                }
            }
        }
        dateText.text = order._slotDateFull
        orderIdText.text = order.orderId
        orderCostText.text = order.totalOrderCost
        val clickAndCollect =
            SlotConstants.DELIVERY_OPTION_CLICK_AND_COLLECT == order.deliveryOption || SlotConstants.DELIVERY_OPTION_REMOTE_CLICK_AND_COLLECT == order.deliveryOption && !order.isThirdPersonDelivery
        val mainTextColor: Int
        val orderCostColor: Int
        val statusTextColor: Int
        var orderStatus = if (order.orderStatus == null) "" else order.orderStatus
        val actualOrderState = if (order.actualOrderState == null) "" else order.actualOrderState
        if (orderStatus == ViewOrderConstants.ORDER_STATUS_CANCELLED || WismoCardConstants.ORDER_REJECTED == actualOrderState) {
            dateText.paintFlags = dateText.paintFlags or Paint.STRIKE_THRU_TEXT_FLAG
            orderIdText.paintFlags = orderIdText.paintFlags or Paint.STRIKE_THRU_TEXT_FLAG
            orderCostText.paintFlags = orderCostText.paintFlags or Paint.STRIKE_THRU_TEXT_FLAG
            mainTextColor = ContextCompat.getColor(mContext, R.color.dark_grey)
            orderCostColor = mainTextColor
            statusTextColor = orderCostColor
            orderStatus = mContext.getString(R.string.order_list_cancelled)
        } else {
            dateText.paintFlags = dateText.paintFlags and Paint.STRIKE_THRU_TEXT_FLAG.inv()
            orderIdText.paintFlags = orderIdText.paintFlags and Paint.STRIKE_THRU_TEXT_FLAG.inv()
            orderCostText.paintFlags =
                orderCostText.paintFlags and Paint.STRIKE_THRU_TEXT_FLAG.inv()
            mainTextColor = ContextCompat.getColor(mContext, R.color.black)
            orderCostColor = ContextCompat.getColor(mContext, R.color.almost_black)
            val isDelivered =
                orderStatus == ViewOrderConstants.ORDER_STATUS_DELIVERED || orderStatus == ViewOrderConstants.ORDER_STATUS_PROCESSING && order.isDeliveryPassPurchase
            if (isDelivered) {
                statusTextColor = mainTextColor
                orderStatus = if (clickAndCollect) {
                    if (WismoCardConstants.FULFILLMENT_TYPE_RCNC.equals(order.fulfillmentType, ignoreCase = true)) {
                        // TODO Review this status for FMO - replicating CNC behavior for completion
                        // orderStatus = mContext.getString(R.string.hs_order_view_details);
                        mContext.getString(R.string.status_order_cnc_collected)
                    } else if (order.isDeliveryPassPurchase) {
                        mContext.getString(R.string.status_dp_order_delivered)
                    } else {
                        mContext.getString(R.string.status_order_cnc_collected)
                    }
                } else if (order.isDeliveryPassPurchase) {
                    mContext.getString(R.string.status_dp_order_delivered)
                } else {
                    mContext.getString(R.string.order_list_delivered)
                }
            } else {
                statusTextColor = ContextCompat.getColor(mContext, R.color.asda_dark_green)
                val cartManager = CartManager.instance
                when (actualOrderState) {
                    WismoCardConstants.ORDER_CREATED, WismoCardConstants.ORDER_CUTOFF, WismoCardConstants.ORDER_SUBMITTED, WismoCardConstants.ORDER_AWAITING_AUTHORIZATION, WismoCardConstants.ORDER_EXTRACTED ->
                        orderStatus =
                            mContext.getString(
                                if (cartManager.isOrderBeingAmended(order.orderId)) R.string.status_order_amending else R.string.hs_order_placed
                            )
                    WismoCardConstants.ORDER_AUTHORIZATION_FAILURE ->
                        orderStatus =
                            mContext.getString(R.string.order_list_payment_failure)
                    WismoCardConstants.ORDER_AUTHORIZATION_SUCCESS, WismoCardConstants.ORDER_SENT_TO_PAM, WismoCardConstants.ORDER_RECD_AT_STORE ->
                        orderStatus =
                            mContext.getString(R.string.preparing_order)
                    WismoCardConstants.ORDER_EPOS_COMPLETE -> if (WismoCardConstants.FULFILLMENT_TYPE_HD.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        ) ||
                        WismoCardConstants.FULFILLMENT_TYPE_RCNC.equals(order.fulfillmentType, ignoreCase = true)
                    ) {
                        orderStatus = mContext.getString(R.string.preparing_order)
                    } else if (WismoCardConstants.FULFILLMENT_TYPE_CNC.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        )
                    ) {
                        orderStatus = mContext.getString(R.string.order_list_ready_collection)
                    }
                    WismoCardConstants.ORDER_LOADED_TO_VAN -> if (WismoCardConstants.FULFILLMENT_TYPE_HD.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        )
                    ) {
                        orderStatus = mContext.getString(R.string.order_list_out_for_delivery)
                    } else if (WismoCardConstants.FULFILLMENT_TYPE_RCNC.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        ) ||
                        WismoCardConstants.FULFILLMENT_TYPE_CNC.equals(order.fulfillmentType, ignoreCase = true)
                    ) {
                        orderStatus = mContext.getString(R.string.order_list_ready_collection)
                    }
                    WismoCardConstants.READY_FOR_STORE ->
                        orderStatus =
                            mContext.getString(R.string.preparing_order)
                    WismoCardConstants.ORDER_NOT_FULFILLED_SETTLEMENT_EXCEPTION -> if (WismoCardConstants.FULFILLMENT_TYPE_HD.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        )
                    ) {
                        orderStatus = mContext.getString(R.string.order_list_out_for_delivery)
                    } else if (WismoCardConstants.FULFILLMENT_TYPE_RCNC.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        ) ||
                        WismoCardConstants.FULFILLMENT_TYPE_CNC.equals(order.fulfillmentType, ignoreCase = true)
                    ) {
                        orderStatus = mContext.getString(R.string.order_list_ready_collection)
                    }
                    WismoCardConstants.ORDER_PICK_COMPLETE -> if (WismoCardConstants.FULFILLMENT_TYPE_HD.equals(
                            order.fulfillmentType,
                            ignoreCase = true
                        ) ||
                        WismoCardConstants.FULFILLMENT_TYPE_RCNC.equals(order.fulfillmentType, ignoreCase = true) ||
                        WismoCardConstants.FULFILLMENT_TYPE_CNC.equals(order.fulfillmentType, ignoreCase = true)
                    ) {
                        orderStatus = mContext.getString(R.string.preparing_order)
                    }
                    else -> orderStatus = mContext.getString(
                        if (cartManager.isOrderBeingAmended(order.orderId)) R.string.status_order_amending else R.string.status_order_confirmed
                    )
                }
            }
        }
        orderStatusText.visibility = View.VISIBLE
        orderStatusText.text = orderStatus
        dateText.setTextColor(mainTextColor)
        orderIdText.setTextColor(mainTextColor)
        slotTypeText.setTextColor(mainTextColor)
        orderStatusText.setTextColor(statusTextColor)
        orderCostText.setTextColor(orderCostColor)
        if (AsdaHomeConfig.bookSlotHelper.getSlotTypeByDeliveryOption(order.deliveryOption)
            == CartManagerConstants.SLOT_TYPE_CC &&
            !order.isThirdPersonDelivery
        ) {
            slotTypeText.setText(R.string.slot_type_cc)
        } else if (order.isDeliveryPassPurchase) {
            slotTypeText.setText(R.string.slot_type_dp)
        } else if (AsdaHomeConfig.bookSlotHelper.getSlotTypeByDeliveryOption(order.deliveryOption)
            == CartManagerConstants.SLOT_TYPE_DELIVERY
        ) {
            slotTypeText.setText(R.string.slot_type_hd)
        }
    }

    fun loadMoreItemsIfNecessary(firstVisibleItem: Int, visibleItemCount: Int) {
        val adapterCount = itemCount
        val userSeesEndOfList =
            visibleItemCount == 0 || firstVisibleItem + visibleItemCount >= adapterCount
        if (!mIsLoading && mHasMoreItems && userSeesEndOfList) {
            loadMoreEntries()
        }
    }

    fun loadMoreEntries() {
        if (!mIsLoading) {
            mIsLoading = true
            if (mOnInitialItemsLoadedListener != null && !mIsInitialized) mOnInitialItemsLoadedListener?.onLoadingStarted()
            if (debug) Log.i(
                TAG,
                "Loading from offset: $offset"
            )
            val pair = Authentication.getInstance().identifierAndProfileId
            if (getOrderService() != null) {
                pair?.first?.let {
                    pair.second?.let { it1 ->
                        getOrderService()
                            ?.getOrders(
                                it,
                                it1, PAGE_SIZE, offset
                            )
                            ?.map { odsOrderModel: OdsOrderModel? ->
                                odsOrderModel?.let { it2 ->
                                    mapOdsHistoryToViewOrder(
                                        it2, offset / PAGE_SIZE + 1
                                    )
                                }
                            }
                            ?.subscribeOn(Schedulers.io())
                            ?.observeOn(AndroidSchedulers.mainThread())
                            ?.subscribe(viewOrderObserver)
                    }
                }
            }
        }
    }

    private val viewOrderObserver: ResourceSingleObserver<ViewOrderResponse?>
        get() = object : ResourceSingleObserver<ViewOrderResponse?>() {
            override fun onSuccess(viewOrderResponse: ViewOrderResponse) {
                onViewOrderSuccess(viewOrderResponse)
            }

            override fun onError(e: Throwable) {
                onViewOrderError(e)
            }
        }

    private fun onViewOrderError(throwable: Throwable) {
        // V4 api will throw 404 for no orders
        if (throwable is HttpException) {
            if (throwable.code() == 404) {
                val viewOrderResponse = ViewOrderResponse()
                viewOrderResponse.setStatus("200")
                viewOrderResponse.currentPage = 0
                viewOrderResponse.maxPages = 0
                viewOrderResponse.orders = arrayOfNulls(0)
                onViewOrderSuccess(viewOrderResponse)
            }
        } else {
            mIsLoading = false
            mHasMoreItems = false
            notifyDataSetChanged()
            if (mOnInitialItemsLoadedListener != null) {
                mOnInitialItemsLoadedListener?.onError()
            }
        }
    }

    private fun onViewOrderSuccess(data: ViewOrderResponse) {
        offset += PAGE_SIZE
        mNextPage = data.currentPage + 1
        if (data.maxPages == data.currentPage) {
            mHasMoreItems = false
        }
        if (data.orders != null && data.orders?.size!! > 0) {
            for (order in data.orders!!) {
                if (mFilterDpOrders && order.isDeliveryPassPurchase) {
                    continue
                }
                val clickAndCollect =
                    (
                        AsdaHomeConfig.bookSlotHelper.getSlotTypeByDeliveryOptionforODS(order.deliveryOption)
                            == CartManagerConstants.SLOT_TYPE_CC
                        )
                mOrders.add(order)
                if (clickAndCollect) {
                    mOrdersQueue.add(order.orderId)
                    order.orderId?.let {
                        getOrderService()
                            ?.getOrderDetails(it)
                            ?.map { odsOrderModel: OdsOrderModel? ->
                                odsOrderModel?.let { it1 ->
                                    mapOdsOrderToWismoOrder(
                                        it1
                                    )
                                }
                            }
                            ?.subscribeOn(Schedulers.io())
                            ?.observeOn(AndroidSchedulers.mainThread())
                            ?.subscribe(getOrderListObserver(order))
                    }
                } else {
                    if (mOrdersQueue.isEmpty() && mLoadFinish != null) {
                        mLoadFinish.onFinish()
                    }
                }
            }
        } else {
            mLoadFinish?.onFinish()
        }
    }

    private fun getOrderListObserver(order: ViewOrderResponse.Order): ResourceSingleObserver<WismoOrderResponse?> {
        return object : ResourceSingleObserver<WismoOrderResponse?>() {
            override fun onSuccess(wismoOrderResponse: WismoOrderResponse) {
                if (wismoOrderResponse.payload != null && wismoOrderResponse.payload?.fulfillmentType != null) {
                    order.fulfillmentType = wismoOrderResponse.payload?.fulfillmentType
                }
                mOrdersQueue.remove(order.orderId)
                if (mOrdersQueue.isEmpty() && mLoadFinish != null) {
                    mLoadFinish.onFinish()
                }
            }

            override fun onError(e: Throwable) {
                // No op
            }
        }
    }

    fun setOnInitialItemsLoadedListener(
        onInitialItemsLoadedListener: OnInitialItemsLoadedListener?
    ) {
        mOnInitialItemsLoadedListener = onInitialItemsLoadedListener
    }

    fun setOnItemClickListener(onItemClickListener: OnItemClickListener?) {
        mOnItemClickListener = onItemClickListener
    }

    interface OnInitialItemsLoadedListener {
        fun onInitialItemsLoaded()
        fun onLoadingStarted()
        fun onError()
    }

    interface OnItemClickListener {
        fun onItemClick(position: Int)
    }

    interface OnLoadFinished {
        fun onFinish()
    }

    class OrderItemViewHolder(itemView: View, viewType: Int) :
        BaseViewHolder(itemView, viewType) {
        val dateText: TextView = findViewById(itemView, R.id.date_text)
        val orderIdText: TextView = findViewById(itemView, R.id.order_id)
        val slotTypeText: TextView = findViewById(itemView, R.id.slot_type)
        val orderStatusText: TextView = findViewById(itemView, R.id.order_status)
        val orderCostText: TextView = findViewById(itemView, R.id.order_cost)
    }

    companion object {
        private const val TYPE_ORDER_ENTRY = 0
        private const val TYPE_LOADING_ENTRY = 1
        private const val TAG = "OrderListAdapter"
        private const val PAGE_SIZE = 20 // Maximum page size for ODS calls.
    }

    init {
        mInflater = LayoutInflater.from(mContext)
        mOrders = ArrayList()
        mOrdersQueue = ArrayList()
        mFilterDpOrders = filterDpOrders
        mLoadFinish = object : OnLoadFinished {
            override fun onFinish() {
                mIsLoading = false
                notifyDataSetChanged()

                // Notify that initial items have been loaded and that list is initialized
                if (!mIsInitialized) {
                    mIsInitialized = true
                    if (mOnInitialItemsLoadedListener != null) {
                        mOnInitialItemsLoadedListener?.onInitialItemsLoaded()
                    }
                }
            }
        }
        offset = 0
        mNextPage = 1
        mIsLoading = false
        mHasMoreItems = true
        mIsInitialized = false
    }
}
