package com.asda.android.app.orders

import android.content.Context
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.dp.view.ActiveDPTabbedFragment.Companion.newInstance
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.OrderManager.Companion.get
import io.reactivex.ObservableSource
import io.reactivex.Observer

class OrderListFragment :
    FeaturedFragment(),
    ObservableSource<String?> {
    private var mNeedsReload = false
    private var mView: View? = null
    private var mListView: RecyclerView? = null
    private var mListAdapter: OrderListAdapter? = null
    private var mFilterDpOrders = false
    private var mContext: Context? = null
    private var mDPOrderTip: View? = null
    private var mObserver: Observer<in String>? = null
    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mFilterDpOrders = bundle.getBoolean(OrderFragment.FILTER_DP_ORDERS_KEY)
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context
    }

    override fun onDetach() {
        super.onDetach()
        if (mObserver != null) {
            mObserver?.onComplete()
            mObserver = null
        }
    }

    override fun getInternalTag(): String {
        return OrderFragment.ORDER_FRAGMENT_TAG
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        if (mView == null || mView?.parent == null) {
            mView = inflater.inflate(R.layout.order_history_list_layout, container, false)
            mListView = mView?.findViewById(R.id.order_list)
            mListView?.layoutManager = LinearLayoutManager(mContext)
            mListAdapter = mContext?.let { OrderListAdapter(it, mFilterDpOrders) }
            mListView?.addOnScrollListener(
                object : RecyclerView.OnScrollListener() {
                    override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                        if (mListAdapter != null) {
                            val layoutManager = mListView?.layoutManager
                            if (layoutManager is LinearLayoutManager) {
                                val visibleItemCount = layoutManager.getChildCount()
                                val firstVisibleItem = layoutManager.findFirstVisibleItemPosition()
                                mListAdapter?.loadMoreItemsIfNecessary(
                                    firstVisibleItem,
                                    visibleItemCount
                                )
                            }
                        }
                    }
                })
            mListAdapter?.setOnInitialItemsLoadedListener(
                object : OrderListAdapter.OnInitialItemsLoadedListener {
                    override fun onLoadingStarted() {
                        mView?.findViewById<View>(R.id.retry_button)?.visibility = View.GONE
                        mView?.findViewById<View>(R.id.loading_page)?.visibility = View.VISIBLE
                    }

                    override fun onError() {
                        if (mListAdapter?.ordersCount == 0) {
                            mView?.findViewById<View>(R.id.loading_page)?.visibility = View.GONE
                            mView?.findViewById<View>(R.id.no_content_container)?.visibility =
                                View.VISIBLE
                            mView?.findViewById<View>(R.id.retry_button)?.visibility = View.VISIBLE
                            val emptyView = mView?.findViewById<View>(R.id.no_content) as TextView
                            emptyView.setText(R.string.oops_order_history_error)
                            emptyView.visibility = View.VISIBLE
                        }
                    }

                    override fun onInitialItemsLoaded() {
                        mView?.findViewById<View>(R.id.loading_page)?.visibility = View.GONE
                        if (mListAdapter?.ordersCount == 0) {
                            val emptyView = mView?.findViewById<View>(R.id.no_content) as TextView
                            emptyView.setText(R.string.order_history_no_orders)
                            emptyView.visibility = View.VISIBLE
                        } else {
                            mView?.findViewById<View>(R.id.no_content_container)?.visibility =
                                View.GONE
                        }
                    }
                })
            mListAdapter?.setOnItemClickListener(
                object : OrderListAdapter.OnItemClickListener {
                    override fun onItemClick(position: Int) {
                        val order = mListAdapter?.getOrder(position)
                        if (order != null) {
                            if (order.isDeliveryPassPurchase) {
                                push(newInstance())
                            } else {
                                if (mObserver != null) {
                                    order.orderId?.let { mObserver?.onNext(it) }
                                } else {
                                    val isLegacyCheckin =
                                        get().isLegacyCheckinOrder(order.orderId)
                                    val fragment =
                                        OrderDetailsPresenter.newInstance(
                                            order.orderId,
                                            !isLegacyCheckin
                                        )
                                    fragment.setOnOrderCancelledListener { canceled: Boolean ->
                                        if (canceled) {
                                            fragment.pop()
                                            reload(mFilterDpOrders)
                                        } else {
                                            mNeedsReload = true
                                        }
                                    }
                                    push(fragment)
                                }
                            }
                        }
                    }
                })
            mView?.findViewById<View>(R.id.retry_button)?.setOnClickListener {
                reload(
                    mFilterDpOrders
                )
            }
            mListView?.adapter = mListAdapter
            mDPOrderTip = null
            handleDPOrderTip()
            val pageViewEvent = PageViewEvent(
                Anivia.PV_ORDER_LIST, Anivia.SECTION_ORDER_HISTORY, Anivia.SECTION_HOMEPAGE
            )
            Tracker.get().trackEvent(pageViewEvent)
        }
        return mView!!
    }

    private fun reload(filterDpOrders: Boolean) {
        if (mView != null && mListAdapter != null) {
            mView?.findViewById<View>(R.id.loading_page)?.visibility = View.VISIBLE
            mListAdapter?.reload(filterDpOrders)
        }
    }

    private fun handleDPOrderTip() {
        if (mListAdapter != null) {
            if (mFilterDpOrders ||
                (Authentication.getInstance() as Authentication).getActiveDeliveryPass() == null
            ) {
                if (mDPOrderTip != null) {
                    mListAdapter?.removeHeaderView()
                    mDPOrderTip = null
                }
            } else if (mDPOrderTip == null) {
                mDPOrderTip = LayoutInflater.from(mContext)
                    .inflate(R.layout.order_history_list_header, mListView, false)
                val textView = mDPOrderTip?.findViewById<View>(R.id.view_dp_details) as TextView
                val text = mContext?.getString(R.string.dp_order_tip_text)
                val sb = SpannableStringBuilder(text)
                text?.indexOf('\n')?.let {
                    sb.setSpan(
                        mContext?.let { it1 ->
                            ContextCompat.getColor(
                                it1, R.color.asda_dark_green
                            )
                        }?.let { it2 ->
                            ForegroundColorSpan(
                                it2
                            )
                        },
                        it,
                        text.length,
                        Spanned.SPAN_INCLUSIVE_INCLUSIVE
                    )
                }
                textView.text = sb
                mDPOrderTip?.let {
                    mListAdapter?.addHeaderView(it)
                }
                mDPOrderTip?.setOnClickListener {
                    push(
                        newInstance()
                    )
                }
            }
        }
    }

    private fun refresh(filterDpOrders: Boolean) {
        mFilterDpOrders = filterDpOrders
        reload(filterDpOrders)
        val fragment = findFragmentByTag(OrderFragment.ORDER_FRAGMENT_TAG)
        if (fragment is OrderDetailsPresenter) {
            fragment.refresh()
        }
    }

    override fun onStart() {
        super.onStart()
        refreshIfNecessary()
    }

    private fun refreshIfNecessary() {
        if (mNeedsReload) {
            mNeedsReload = false
            refresh(mFilterDpOrders)
        }
    }

    override fun onStop() {
        mNeedsReload = true
        super.onStop()
    }

    override fun onShown() {
        super.onShown()
        handleDPOrderTip()
        refreshIfNecessary()
    }

    override fun subscribe(p0: Observer<in String?>) {
        mObserver = p0
    }
    // region ActionBar-AccountFragment Usage
    /** Following methods are only used by BaseAccountActivity, not AsdaActivity.  */
    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return if (isAdded) {
            if (mObserver == null) getString(R.string.your_orders) else getString(R.string.select_order)
        } else null
    } // endregion

    companion object {
        const val SCREEN_NAME = "Order List Page"

        @JvmStatic
        fun newInstance(filterDpOrders: Boolean): OrderListFragment {
            val orderListFragment = OrderListFragment()
            val args = Bundle()
            args.putBoolean(OrderFragment.FILTER_DP_ORDERS_KEY, filterDpOrders)
            orderListFragment.arguments = args
            return orderListFragment
        }
    }
}
