package com.asda.android.app.orders

import android.app.Activity
import android.app.Dialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.TextView
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.orders.ods.getOrderService
import com.asda.android.app.orders.ods.mapOdsOrderToWismoOrder
import com.asda.android.app.shop.AddToTrolleyBaseFragment
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.constants.EXTRA_ORIGIN
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.constants.ErrorCodes
import com.asda.android.restapi.service.data.OdsOrderModel
import com.asda.android.restapi.service.data.ShelfItem
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.singleprofile.features.login.view.LoginSpActivity
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.ViewUtil.setValueWithCurrency
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers
import java.net.ConnectException
import java.net.SocketTimeoutException

class ReorderFragment : AddToTrolleyBaseFragment() {
    private var mOrderId: String? = null
    private val mRecipeIngredientsMap = HashMap<String?, String?>()
    override fun getScreenName(): String {
        return SCREEN_NAME
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        mOrderId = bundle?.getString(ORDER_ID)
    }

    override fun onStart() {
        super.onStart()
        val pageViewEvent = PageViewEvent(Anivia.PV_REORDER, Anivia.SECTION_ACCOUNT, Anivia.SECTION_ACCOUNT)
        Tracker.get().trackPageView(pageViewEvent)
    }

    override fun createLoginDialog(activity: Activity, isAddToLists: Boolean): Dialog {
        val clickListener =
            DialogInterface.OnClickListener { dialog: DialogInterface?, which: Int ->
                val intent = Intent(
                    activity,
                    LoginSpActivity::class.java
                )
                intent.putExtra(EXTRA_ORIGIN, "ReorderFragment")
                startActivityForResult(
                    intent,
                    ActivityRequestCodes.REQUEST_CODE_LOGIN
                )
            }
        val r = activity.resources
        val msg = r.getString(
            if (isAddToLists) R.string.login_to_shopping_list_popup_description else R.string.login_dialog_msg
        )
        val negativeText = r.getString(R.string.login_dialog_continue_button)
        val positiveText = r.getString(R.string.login_dialog_login_button)
        return DialogFactory.createAlertDialog(
            msg, positiveText, clickListener, negativeText, activity, SCREEN_NAME
        )
    }

    override fun loadItems() {
        val context = context
        if (context != null) {
            mOrderId?.let {
                getOrderService()
                    ?.getOrderDetails(it)
                    ?.map { odsOrderModel: OdsOrderModel? ->
                        odsOrderModel?.let { it1 ->
                            mapOdsOrderToWismoOrder(
                                it1
                            )
                        }
                    }
                    ?.subscribeOn(Schedulers.io())
                    ?.observeOn(AndroidSchedulers.mainThread())
                    ?.subscribe(wismoObserver)
            }
        }
    }

    private val wismoObserver: ResourceSingleObserver<WismoOrderResponse?>
        get() = object : ResourceSingleObserver<WismoOrderResponse?>() {
            override fun onSuccess(data: WismoOrderResponse) {
                onSuccessWismo(data)
            }

            override fun onError(e: Throwable) {
                Log.e(
                    TAG,
                    "Error getting ordered items for orderId #$mOrderId"
                )
                if (isAdded) {
                    handleApiFailure(isItNetworkProblem(e))
                }
            }
        }

    private fun onSuccessWismo(data: WismoOrderResponse) {
        mTotalItems = 0
        if (isAdded) {
            if (data.payload != null) {
                if (data.payload?.containsRecipe == true &&
                    data.payload?.recipes != null && data.payload?.recipes?.size!! > 0
                ) {
                    for ((name, ingredients) in data.payload?.recipes!!) {
                        if (ingredients == null) {
                            continue
                        }
                        for ((_, _, _, _, skuId) in ingredients) {
                            if (!mRecipeIngredientsMap.containsKey(skuId)) {
                                mRecipeIngredientsMap[skuId] = name
                            }
                        }
                    }
                }
                val shelfListItems = ArrayList<ShelfListItem>()
                if (data.payload?.shoppingItemList != null) {
                    val hasCheckedItems: Boolean = mCheckedItems.size > 0
                    for (orderItem in data.payload?.shoppingItemList!!) {
                        val shelfListItem = ShelfListItem(orderItem)
                        shelfListItem.imageURL = ViewUtil.pickImageUrl(
                            orderItem.extraLargeImageURL,
                            orderItem.imageURL
                        )
                        if (TextUtils.isEmpty(shelfListItem.maxQty)) {
                            shelfListItem.maxQty =
                                orderItem.qty // Assume user can only decrease qty if necessary
                        }
                        if (shelfListItem.availability == ShelfItem.AVAILABLE) {
                            if (hasCheckedItems && !mCheckedItems.containsKey(shelfListItem.id)) {
                                shelfListItem.checked = false
                            } else {
                                shelfListItem.checked = true
                                mTotalItems++
                                mCheckedItems[shelfListItem.id] = shelfListItem.priceToDisplay
                            }
                        } else {
                            shelfListItem.checked = false
                        }
                        shelfListItems.add(shelfListItem)
                    }
                }
                if (data.payload?.unavailableItemList != null) {
                    for (orderItem in data.payload?.unavailableItemList!!) {
                        val shelfListItem = ShelfListItem(orderItem)
                        if (TextUtils.isEmpty(shelfListItem.maxQty)) {
                            shelfListItem.maxQty =
                                orderItem.ordered // Assume user can only decrease qty if necessary
                        }
                        shelfListItem.checked = true
                        mTotalItems++
                        mCheckedItems[shelfListItem.id] = shelfListItem.priceToDisplay
                        shelfListItems.add(shelfListItem)
                    }
                }
                if (data.payload?.substitutedItemList != null) {
                    for (orderItem in data.payload?.substitutedItemList!!) {
                        val shelfListItem = ShelfListItem(orderItem)
                        shelfListItem.checked = true
                        mTotalItems++
                        mCheckedItems[shelfListItem.id] = shelfListItem.priceToDisplay
                        shelfListItems.add(shelfListItem)
                    }
                }
                if (shelfListItems.isEmpty()) {
                    Log.e(
                        TAG,
                        "Empty ordered items for orderId #$mOrderId"
                    )
                    handleApiFailure(ErrorCodes.ERROR_CODE_UNKNOWN_ERROR)
                } else {
                    addShelfItems(shelfListItems)
                    itemsLoaded()
                }
            } else {
                Log.e(
                    TAG,
                    "Empty ordered items for orderId #$mOrderId"
                )
                handleApiFailure(ErrorCodes.ERROR_CODE_UNKNOWN_ERROR)
            }
        }
    }

    private fun handleApiFailure(error: Int) {
        DialogHelper.displayErrorDialog(
            null,
            error,
            mActivity,
            false,
            {
                if (isAdded) {
                    if (mActivity.supportFragmentManager.backStackEntryCount == 1) {
                        mActivity.setResult(Activity.RESULT_OK)
                        mActivity.finish()
                    } else {
                        mActivity.onBackPressed()
                    }
                }
            },
            SCREEN_NAME
        )
    }

    override fun allowUnavailableCheck(): Boolean {
        return true
    }

    override fun processListViewItem(listEntryView: View, itemData: ShelfListItem) {
        super.processListViewItem(listEntryView, itemData)
        setValueWithCurrency(
            R.id.shelf_item_cost,
            listEntryView,
            (listEntryView.findViewById<View>(R.id.shelf_item_cost) as TextView).text
        )
    }

    override fun hasTotalCost(): Boolean {
        return false
    }

    override fun addSelectedItemsToTrolley() {
        super.addSelectedItemsToTrolley(
            Anivia.CATEGORY_MERCHANDISING_REORDER, mRecipeIngredientsMap, null
        )
    }

    private fun isItNetworkProblem(throwable: Throwable): Int {
        return if (throwable is SocketTimeoutException || throwable is ConnectException) ErrorCodes.ERROR_CODE_NETWORK_ERROR else ErrorCodes.ERROR_CODE_JSON_ERROR
    }

    companion object {
        private const val SCREEN_NAME = "Reorder"
        private const val TAG = "ReorderFragment"
        private const val ORDER_ID = "order_id"

        @JvmStatic
        fun newInstance(orderId: String?): ReorderFragment {
            val fragment = ReorderFragment()
            val bundle = Bundle()
            bundle.putString(ORDER_ID, orderId)
            fragment.arguments = bundle
            return fragment
        }
    }
}
