package com.asda.android.app.orders.ods

import android.util.Log
import com.asda.android.app.bookslot.PreviewBookDataSource
import com.asda.android.base.core.utils.orFalse
import com.asda.android.orders.constants.*
import com.asda.android.orders.orders.model.OrderStatus
import com.asda.android.orders.utils.convertStatus
import com.asda.android.orders.wismo.constants.WismoCardConstants
import com.asda.android.recipes.view.fragments.ImpactedRecipeListFragment
import com.asda.android.recipes.view.fragments.UnavailableRecipesFragment
import com.asda.android.restapi.constants.BffConstants.API_BFF_QL_MATCHER
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.OrderAPI
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.constants.SlotConstants
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.cart.CartResponse
import com.asda.android.restapi.service.data.recipes.RecipeCartAddResponse
import com.asda.android.service.base.Authentication
import okhttp3.Request
import java.math.BigDecimal
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

fun addAuthenticationToken(requestUrl: String, requestBuilder: Request.Builder) {
    if (requestUrl.contains("graphql") || requestUrl.contains(API_BFF_QL_MATCHER)) {
        requestBuilder.url(requestUrl)
        addAuthenticationToken(requestBuilder)
    }
}

fun addAuthenticationToken(requestBuilder: Request.Builder) {
    val authToken = Authentication.getInstance()?.autoLoginCookieStrings?.authTokenCookieString
    if (authToken.isNullOrEmpty()) {
        requestBuilder.removeHeader("WM_SEC.AUTH_TOKEN")
        return
    }
    requestBuilder.header("WM_SEC.AUTH_TOKEN", authToken)
}

fun getOrderService(): OrderAPI? {
    return ServiceGenerator.createService(
        AsdaServiceSettings.BASE_URL, "",
        OrderAPI::class.java
    )
}

fun mapOdsHistoryToViewOrder(odsOrder: OdsOrderModel, pageNum: Int): ViewOrderResponse {
    val order = ViewOrderResponse()
    val header = odsOrder.header?.headerAttributes

    val pageCount = header?.pageCount ?: 0
    val totalCount = header?.totalCount ?: 0

    order.maxPages = (totalCount / pageCount) + if (totalCount % pageCount == 0) 0 else 1
    order.currentPage = pageNum

    val orderList: MutableList<ViewOrderResponse.Order> = ArrayList()

    odsOrder.payload?.let {
        for (payload in it) {
            orderList.add(mapOdsOrderToOldOrder(payload))
        }
    }
    order.orders = orderList.toTypedArray()
    return order
}

fun mapOdsOrderToOldOrder(payload: Payload?): ViewOrderResponse.Order {
    val order = ViewOrderResponse.Order()
    val deliveryReservationDetail = payload?.deliveryReservationDetails?.get(0)
    val summary = payload?.orderSummary

    order.bookedUntilTime = deliveryReservationDetail?.slotCutOffTime
    order.carrierBagCharge = summary?.carrierBagCharge?.currencyUnit
    order.deliveryEndTime = deliveryReservationDetail?.endTime
    order.timeSlotISO = deliveryReservationDetail?.fulfillmentDate + deliveryReservationDetail
        ?.startTime + deliveryReservationDetail?.endTime

    order.deliveryOption = mapSlotType(
        deliveryReservationDetail?.dispenseType
            ?: "",
        payload?.orderInfo?.thirdPersonDelivery ?: false
    )
    order.deliveryStartTime = deliveryReservationDetail?.startTime
    order.deliveryCost = summary?.deliveryCharge?.currencyAmount.toString() + summary?.deliveryCharge?.currencyUnit
    order.orderId = payload?.orderNo
    order.storeID = payload?.orderCustomAttributes?.storeId
    order.orderStatus = if (payload?.statusText == PREPARING_FOR_COLLECTION && payload.status == OrderStatus.ORDER_EPOS_COMPLETE.status && deliveryReservationDetail?.dispenseType == DISPENSE_TYPE_CNC || (deliveryReservationDetail?.dispenseType == DISPENSE_TYPE_RCNC && !payload.status.equals(OrderStatus.ORDER_NO_PENDING_ACTION.status) && !payload.status.equals(OrderStatus.ORDER_CANCELLED.status)))
        OrderStatus.READY_FOR_STORE.readableValue
    else convertStatus(
        payload?.status,
        mapfulfillmentType(
            deliveryReservationDetail?.dispenseType
                ?: "",
            payload?.orderInfo?.thirdPersonDelivery ?: false
        )
    )
    order.actualOrderState = payload?.status
    order.orderedDate = payload?.orderDate
    order.postCode = payload?.paymentMethods?.get(0)?.personInfoBillTo?.address?.postalCode
    order.slotDate = deliveryReservationDetail?.fulfillmentDate ?: ""
    order.subTotalCost = payload?.orderTotals?.orderSubTotal?.currencyAmount.toString() + payload
        ?.orderTotals?.orderSubTotal?.currencyUnit
    order.totalOrderCost = RestUtils.addPoundIfNecessary(summary?.totalAmount?.currencyAmount?.setScale(2, BigDecimal.ROUND_HALF_UP)?.toPlainString())
    order.deliverySurcharge = summary?.minBasketCharge?.currencyAmount?.toString() + payload
        ?.orderTotals?.orderSubTotal?.currencyUnit

    order.fulfillmentType = mapfulfillmentType(deliveryReservationDetail?.dispenseType ?: "", payload?.orderInfo?.thirdPersonDelivery ?: false)
    order.isAmendable = payload?.orderCustomAttributes?.isAmendable
    order.setIsLegacyCheckIn("false")
    order.isThirdPersonDelivery = payload?.orderInfo?.thirdPersonDelivery ?: false

    val list: MutableList<ViewOrderResponse.Order.OrderItem> = ArrayList()

    val shouldShowUnavailableItemList = shouldShowUnavailableItemList(payload?.statusText)
    for (orderLine in payload?.orderLines.orEmpty()) {
        constructShoppingItem(orderLine, shouldShowUnavailableItemList)?.let { list.add(it) }
    }

    val address = payload?.shipToAddress?.get(0)

    if (address?.address?.addressType.equals(WismoCardConstants.FULFILLMENT_TYPE_HD, true) || order.isThirdPersonDelivery) {
        order.deliveryAddress = constructDeliveryAddress(payload)
    } else if (address?.address?.addressType.equals(WismoCardConstants.FULFILLMENT_TYPE_CNC, true) || address?.address?.addressType.equals(WismoCardConstants.FULFILLMENT_TYPE_RCNC, true)) {
        order.cncStoreAddress = constructCncAddress(payload)
    }

    // TODO : Delivery pass fields
    return order
}

fun constructCncAddress(payload: Payload?): SlotResponse.CncStoreAddress? {
    val address = SlotResponse.CncStoreAddress()
    val odsAddress = payload?.shipToAddress?.get(0)
    val storeFrontId = odsAddress?.storeFrontId

    address.storeid = storeFrontId?.storeId
    address.storename = storeFrontId?.storeName
    address.postalCode = odsAddress?.address?.postalCode
    address.city = odsAddress?.address?.city
    address.addressLine1 = odsAddress?.address?.addressLineOne
    address.addressLine2 = odsAddress?.address?.addressLineTwo
    address.phoneNumber = odsAddress?.storeFrontId?.phone?.completeNumber
    address.fulfillmentType = mapfulfillmentType(
        payload?.deliveryReservationDetails?.get(0)?.dispenseType
            ?: "",
        payload?.orderInfo?.thirdPersonDelivery ?: false
    )
    address.latitude = odsAddress?.address?.geoPoint?.latitude.toString()
    address.longitude = odsAddress?.address?.geoPoint?.longitude.toString()
    address.country = odsAddress?.address?.country

    return address
}

fun constructDeliveryAddress(payload: Payload?): SlotResponse.DeliveryAddress? {
    val deliveryAddres = SlotResponse.DeliveryAddress()
    val odsAddress = payload?.shipToAddress?.get(0)

    deliveryAddres.address = odsAddress?.address?.addressLineOne
    deliveryAddres.address2 = odsAddress?.address?.addressLineTwo
    deliveryAddres.phoneNumber = odsAddress?.storeFrontId?.phone?.completeNumber
    deliveryAddres.postalCode = odsAddress?.address?.postalCode
    deliveryAddres.city = odsAddress?.address?.city
    deliveryAddres.county = odsAddress?.address?.county
    deliveryAddres.state = odsAddress?.address?.stateOrProvinceName
    deliveryAddres.country = odsAddress?.address?.country
    deliveryAddres.latitude = odsAddress?.address?.geoPoint?.latitude.toString()
    deliveryAddres.longitude = odsAddress?.address?.geoPoint?.longitude.toString()

    val primaryContact = payload?.buyerInfo?.primaryContact

    deliveryAddres.firstName = primaryContact?.name?.firstName
    deliveryAddres.lastName = primaryContact?.name?.lastName
    deliveryAddres.middleName = primaryContact?.name?.middleName

    deliveryAddres.storeId = odsAddress?.storeFrontId?.storeId
    deliveryAddres.mobilePhoneNumber = primaryContact?.phone?.completeNumber

    return deliveryAddres
}

fun mapOdsOrderToWismoOrder(odsResponse: OdsOrderModel): WismoOrderResponse {
    val wismoOrder = WismoOrderResponse()
    wismoOrder.status = odsResponse.responseStatus
    wismoOrder.payload = mapOdstoWismoPayload(odsResponse.payload?.get(0))
    wismoOrder.payload?.orderStatus = odsResponse.payload?.get(0)?.statusText
    wismoOrder.payload?.isExpress = odsResponse.payload?.get(0)?.isExpressOrder
    wismoOrder.payload?.isCancellable = odsResponse.payload?.get(0)?.isCancellable
//  wismoOrder.payload?.orderStatus = convertStatus(odsResponse.payload[0]?.statusText, mapfulfillmentType(odsResponse.payload.get(0)?.deliveryReservationDetails?.get(0)?.dispenseType ?: "")
    return wismoOrder
}

fun mapOdstoWismoPayload(odsPayload: Payload?): WismoOrderResponse.Payload {
    val payload = WismoOrderResponse.Payload()

    val reservationDetails = odsPayload?.deliveryReservationDetails?.get(0)
    payload.slotStartTime = reservationDetails?.startTime
    payload.slotEndTime = reservationDetails?.endTime
    payload.orderStatus = convertStatus(
        odsPayload?.status,
        odsPayload?.deliveryReservationDetails?.get(0)?.dispenseType
            ?: "",
        odsPayload?.orderInfo?.thirdPersonDelivery
    )
    payload.dateOfDelivery = convertTime(reservationDetails?.fulfillmentDate)
    payload.bookedUntilTime = reservationDetails?.slotCutOffTime
    payload.etaTrackingEnd = reservationDetails?.etaEnd
    payload.etaTrackingStart = reservationDetails?.etaStart
    payload.refundAvailabilityPeriod = odsPayload?.orderCustomAttributes?.refundAvailabilityPeriod
    payload.refundAvailable = if (odsPayload?.orderCustomAttributes?.refundAvailable == true) "true" else "false"
    payload.fulfillmentType = mapfulfillmentType(
        reservationDetails?.dispenseType ?: "",
        odsPayload?.orderInfo?.thirdPersonDelivery
            ?: false
    )
    payload.webOrderID = odsPayload?.orderNo
    payload.storeID = odsPayload?.accessPointInfo?.get(0)?.contactInfo?.storeFrontId?.storeId
    payload.showGcDetails = odsPayload?.showGiftCardDetail ?: false
    payload.orderStatusText = odsPayload?.statusText
    payload.status = odsPayload?.status
    payload.amendedOrderID = odsPayload?.orderCustomAttributes?.amendedOrderID
    payload.vanRegistrationId = odsPayload?.deliveryTracking?.vanRegistrationId
    payload.driverName = odsPayload?.deliveryTracking?.driverName
    payload.driverPhoneNumber = odsPayload?.deliveryTracking?.driverPhoneNumber
    payload.vehicleMake = odsPayload?.deliveryTracking?.vehicleMake
    payload.vehicleModel = odsPayload?.deliveryTracking?.vehicleModel
    payload.numberOfStopsAway = odsPayload?.deliveryTracking?.numberOfStopsAway.toString()
    payload.storeID = odsPayload?.orderCustomAttributes?.storeId
    payload.orderDate = odsPayload?.orderDate
    payload.thirdPartyDeliveryOrder = odsPayload?.orderInfo?.thirdPersonDelivery ?: false
    payload.thirdPartyDeliveryProvider = odsPayload?.orderInfo?.thirdPartyProviderDetails?.thirdPartyDeliveryProvider.orEmpty()
    payload.trackingId = odsPayload?.deliveryTracking?.trackingId

    val list: MutableList<ViewOrderResponse.Order.OrderItem?> = ArrayList()
    val subList: MutableList<WismoOrderResponse.SubstitutedItem> = ArrayList()
    val unavailableList: MutableList<WismoOrderResponse.UnavailableItem> = ArrayList()

    val shouldShowUnavailableItemList = shouldShowUnavailableItemList(odsPayload?.statusText)

    for (orderLine in odsPayload?.orderLines.orEmpty()) {
        constructShoppingItem(orderLine, shouldShowUnavailableItemList).let { list.add(it) }
        if (orderLine?.pickedItem != null) {
            subList.add(constructSubstituteItem(orderLine))
        }
        if (orderLine?.orderLineCustomAttributes?.nilPickQty != "0") {
            unavailableList.add(mapUnavailableItemList(orderLine))
        }
    }

    if (subList.size > 0) {
        payload.substitutedItemList = subList.toTypedArray()
    }
    if (shouldShowUnavailableItemList && unavailableList.size > 0) {
        payload.unavailableItemList = unavailableList.toTypedArray()
    }
    payload.isReturnAvailable = odsPayload?.returnDetails?.itemRefundList.isNullOrEmpty().not()
    payload.refundItemList = mapRefundItemfromODS(odsPayload)

    list.sortBy { it?.deptName }
    payload.shoppingItemList = list.toTypedArray()

    payload.customerAddress?.setAddress(odsPayload?.shipToAddress?.get(0)?.address?.addressLineOne)

    payload.payment = constructPaymentInfo(odsPayload)
    payload.payment?.colleagueDiscountAmount?.let { payload.colleagueDiscountEligible = true }

    payload.giftCards = constructGiftCardPaymentInfo(odsPayload)

    updatePayloadAddress(odsPayload, payload)

    val orderCustomAttributes = odsPayload?.orderCustomAttributes
    payload.orderAuthStatus = orderCustomAttributes?.authStatus
    payload.authStatusMsg = orderCustomAttributes?.authStatusMsg
    payload.allowCardEdit = orderCustomAttributes?.allowCardEdit?.toBoolean() ?: false
    payload.recipes = constructRecipeList(odsPayload)

    payload.containsRecipe = payload.recipes?.size ?: 0 > 0
    payload.isAmendable = odsPayload?.isAmendable

    return payload
}

fun updatePayloadAddress(odsPayload: Payload?, payload: WismoOrderResponse.Payload) {
    when (odsPayload?.deliveryReservationDetails?.firstOrNull()?.dispenseType) {
        DISPENSE_TYPE_DELIVERY -> payload.customerAddress = constructDeliveryAddressForWismo(odsPayload)
        DISPENSE_TYPE_CNC, DISPENSE_TYPE_RCNC ->
            if (payload.thirdPartyDeliveryOrder) {
                payload.customerAddress = constructDeliveryAddressForWismo(odsPayload)
            } else {
                payload.cncStoreAddress = constructCncAddressforWismo(odsPayload)
            }
    }
}

fun mapRefundItemfromODS(odsPayload: Payload?): WismoOrderResponse.Refund? {

    val refundList = WismoOrderResponse.Refund()

    val approvedList = ArrayList<WismoOrderResponse.RefundItem>()
    val pendingList = ArrayList<WismoOrderResponse.RefundItem>()
    val cancelledList = ArrayList<WismoOrderResponse.RefundItem>()

    val odsRefundList = odsPayload?.returnDetails?.itemRefundList.orEmpty()

    for (odsRefundItem in odsRefundList) {
        val refundItem = WismoOrderResponse.RefundItem()
        refundItem.name = odsRefundItem?.description
        refundItem.desc = odsRefundItem?.description
        refundItem.itemName = odsRefundItem?.description
        refundItem.price = odsRefundItem?.refundAmount.toString()
        refundItem.quantityRefunded = odsRefundItem?.quantityToReturn.toString()
            .plus(" of ")
            .plus(odsRefundItem?.deliveredQty?.toString())
        refundItem.refundReason = odsRefundItem?.reason
        refundItem.refundDate = odsRefundItem?.submittedDate
        refundItem.imageURL = odsRefundItem?.imageUrl

        when {
            odsRefundItem?.state.equals(REFUND_APPROVED, false) || odsRefundItem?.state.equals(REFUND_COMPLETE, false) -> approvedList.add(refundItem)
            odsRefundItem?.state.equals(REFUND_CANCELLED, false) || odsRefundItem?.state.equals(REFUND_AUTO_CANCELLED, false) -> cancelledList.add(refundItem)
            else -> pendingList.add(refundItem)
        }
    }

    refundList.approved = approvedList.toTypedArray()
    refundList.pending = pendingList.toTypedArray()
    refundList.cancelled = cancelledList.toTypedArray()

    return refundList
}

fun mapUnavailableItemList(orderLine: OrderLine?): WismoOrderResponse.UnavailableItem {

    val nilPick = orderLine?.orderLineCustomAttributes?.nilPickQty?.toInt() ?: 0
    val orderedQty = orderLine?.orderedQty
    val unavailableItem = WismoOrderResponse.UnavailableItem()

    val orderProduct = orderLine?.orderProduct
    val offerLogistics = orderProduct?.offerLogistics

    unavailableItem.description = orderProduct?.description
    unavailableItem.id = offerLogistics?.skuId
    unavailableItem.imageURL = offerLogistics?.itemImageURL
    unavailableItem.pricePerWt = orderProduct?.pricePerUOM
    unavailableItem.totalPrice = orderLine?.orderLineCustomAttributes?.totalPrice?.toString()
    unavailableItem.ordered = orderedQty?.measurementValue?.toPlainString()
    unavailableItem.delivered = orderedQty?.measurementValue?.toInt()?.minus(nilPick).toString()

    return unavailableItem
}

fun shouldShowUnavailableItemList(statusText: String?): Boolean {
    return when (statusText) {
        ORDER_STATUS_PLACED -> false
        ORDER_STATUS_PROCESSING -> false
        ORDER_STATUS_AUTHORIZED -> false
        else -> true
    }
}

fun mapSlotType(type: String, is3POrder: Boolean = false): String {
    return when {
        type.equals(DISPENSE_TYPE_DELIVERY, true) -> SlotConstants.DELIVERY_OPTION_HOME_DELIVERY
        type.equals(DISPENSE_TYPE_CNC, true) && is3POrder -> SlotConstants.DELIVERY_OPTION_HOME_DELIVERY
        type.equals(DISPENSE_TYPE_CNC, true) -> SlotConstants.DELIVERY_OPTION_CLICK_AND_COLLECT
        type.equals(DISPENSE_TYPE_RCNC, true) -> SlotConstants.DELIVERY_OPTION_REMOTE_CLICK_AND_COLLECT
        else -> type
    }
}

fun mapfulfillmentType(type: String, is3POrder: Boolean = false): String {
    return when {
        type.equals(DISPENSE_TYPE_DELIVERY, true) -> WismoCardConstants.FULFILLMENT_TYPE_HD
        type.equals(DISPENSE_TYPE_CNC, true) && is3POrder -> WismoCardConstants.FULFILLMENT_TYPE_HD
        type.equals(DISPENSE_TYPE_CNC, true) -> WismoCardConstants.FULFILLMENT_TYPE_CNC
        type.equals(DISPENSE_TYPE_RCNC, true) -> WismoCardConstants.FULFILLMENT_TYPE_RCNC
        type.equals(DISPENSE_DELIVERY_PASS, true) -> WismoCardConstants.DELIVERY_PASS
        else -> type
    }
}

fun constructCncAddressforWismo(odsPayload: Payload?): WismoOrderResponse.CncStoreAddress? {
    val cncStoreResponse = WismoOrderResponse.CncStoreAddress()

    val contactInfo = odsPayload?.accessPointInfo?.get(0)?.contactInfo
    val storeFrontId = contactInfo?.storeFrontId

    cncStoreResponse.setAddressLine1(contactInfo?.address?.addressLineOne)
    cncStoreResponse.setAddressLine2(contactInfo?.address?.addressLineTwo)
    cncStoreResponse.city = contactInfo?.address?.city
    cncStoreResponse.postalCode = contactInfo?.address?.postalCode
    cncStoreResponse.country = contactInfo?.address?.country
    cncStoreResponse.setStoreName(storeFrontId?.storeName)
    cncStoreResponse.latitude = contactInfo?.address?.geoPoint?.latitude.toString()
    cncStoreResponse.longitude = contactInfo?.address?.geoPoint?.longitude.toString()
    cncStoreResponse.phoneNumber = contactInfo?.storeFrontId?.phone?.completeNumber

    return cncStoreResponse
}

fun constructDeliveryAddressForWismo(odsPayload: Payload?): WismoOrderResponse.DeliveryAddress? {
    val wismoDeliveryAddress = WismoOrderResponse.DeliveryAddress()

    val odsAddress = odsPayload?.shipToAddress?.get(0)

    wismoDeliveryAddress.setAddress(odsPayload?.shipToAddress?.get(0)?.address?.addressLineOne)
    wismoDeliveryAddress.setAddress(odsAddress?.address?.addressLineOne)
    wismoDeliveryAddress.address2 = odsAddress?.address?.addressLineTwo
    wismoDeliveryAddress.city = odsAddress?.address?.city
    wismoDeliveryAddress.postalCode = odsAddress?.address?.postalCode
    wismoDeliveryAddress.country = odsAddress?.address?.country
    wismoDeliveryAddress.latitude = odsAddress?.address?.geoPoint?.latitude.toString()
    wismoDeliveryAddress.longitude = odsAddress?.address?.geoPoint?.longitude.toString()
    wismoDeliveryAddress.phoneNumber = odsAddress?.storeFrontId?.phone?.completeNumber

    val primaryContact = odsPayload?.buyerInfo?.primaryContact

    wismoDeliveryAddress.firstName = primaryContact?.name?.firstName
    wismoDeliveryAddress.lastName = primaryContact?.name?.lastName
    wismoDeliveryAddress.middleName = primaryContact?.name?.middleName

    wismoDeliveryAddress.storeId = odsAddress?.storeFrontId?.storeId
    wismoDeliveryAddress.mobilePhoneNumber = primaryContact?.phone?.completeNumber
    return wismoDeliveryAddress
}

fun constructRecipeList(odsPayload: Payload?): MutableList<RecipeCartAddResponse.Recipe>? {
    val recipeList: MutableList<RecipeCartAddResponse.Recipe> = ArrayList()

    for (odsRecipe in odsPayload?.orderRecipeInfo.orEmpty()) {

        val ingredientList: MutableList<RecipeCartAddResponse.Ingredients> = ArrayList()

        for (odsIngredient in odsRecipe?.orderRecipeIngredients.orEmpty()) {
            val ingredient = RecipeCartAddResponse.Ingredients()
            ingredient.ingredientId = odsIngredient?.ingredientId
            ingredient.ingredientName = odsIngredient?.ingredientName
            ingredient.quantity = odsIngredient?.itemQty.toString()
            ingredient.skuId = odsIngredient?.skuId
            ingredient.qtyNeededForRecipe = odsIngredient?.itemQty.toString()
            ingredient.recipeId = odsRecipe?.recipeId
            ingredient.scene7AssetId = odsRecipe?.s7AssetId

            ingredientList.add(ingredient)
        }

        val recipe = RecipeCartAddResponse.Recipe()

        recipe.name = odsRecipe?.name
        recipe.ingredients = ingredientList
        recipe.recipeId = odsRecipe?.recipeId
        recipe.source = odsRecipe?.recipeSource
        recipe.servingSize = odsRecipe?.servingSize.toString()
        recipe.scene7AssetId = odsRecipe?.s7AssetId
        recipe.costPerServing = odsRecipe?.costPerServing.toString()
        recipe.costOfRecipe = odsRecipe?.costOfRecipe.toString()
        recipe.readyIn = odsRecipe?.readyIn

        /*recipe.imageUrl = odsRecipe
        recipe.totalIngredients = odsRecipe*/

        recipeList.add(recipe)
    }
    return recipeList
}

fun constructPaymentInfo(odsPayload: Payload?): WismoOrderResponse.Payment? {
    val payment = WismoOrderResponse.Payment()
    val odsPayment = odsPayload?.paymentMethods

    odsPayment?.let {
        for (paymentMethod in it) {
            paymentMethod?.personInfoBillTo?.let {
                payment.cardHolderName = it.name?.firstName
            }
        }
    }
    payment.costBeforePromo = odsPayload?.orderSummary?.amountBeforePromo?.currencyAmount?.toPlainString()
    payment.carrierBagCharge = odsPayload?.orderSummary?.carrierBagCharge?.currencyAmount?.toPlainString()
    payment.deliveryCharge = odsPayload?.orderSummary?.deliveryCharge?.currencyAmount?.toPlainString()
    payment.evoucherTotalAmount = odsPayload?.orderSummary?.evoucherTotalAmount?.currencyAmount?.toPlainString()
    payment.totalCost = odsPayload?.orderSummary?.totalAmount?.currencyAmount?.toPlainString()
    payment.deliverySurcharge = odsPayload?.orderSummary?.minBasketCharge?.currencyAmount?.toPlainString()
    payment.gcTotal = odsPayload?.orderSummary?.amountOnGiftCards?.currencyAmount?.toDouble()
    payment.incentiveDiscount = odsPayload?.orderSummary?.totalMultiSaveDiscount?.currencyAmount?.toPlainString()

    val paymentMethod = odsPayload?.paymentMethods

    paymentMethod?.let {
        for (paymentCard in it) {
            if (paymentCard?.paymentType != GIFTCARD_PAYMENT_TYPE) {
                payment.orderCardNickName = paymentCard?.cardId
                payment.cardType = paymentCard?.paymentType
                payment.maskedCardNumber = paymentCard?.last4DigitsOfCard
                payment.expirationMonth = paymentCard?.instrumentExpiry?.substring(MONTH_START_INDEX)
                payment.expirationYear = paymentCard?.instrumentExpiry?.substring(YEAR_START_INDEX, MONTH_START_INDEX)
                payment.amount = paymentCard?.amount
            }
        }
    }

    val customAttributes = odsPayload?.orderCustomAttributes

    payment.paymentStatus = customAttributes?.authStatus
    payment.failureReason = customAttributes?.authFailureReason
    payment.paymentStatusMsg = customAttributes?.paymentStatusMsg
    payment.colleagueDiscountAmount = RestUtils.addPoundIfNecessary(
        odsPayload?.orderSummary?.colleagueDiscountAmount?.currencyAmount?.toString()
            ?: "0.0"
    )
    payment.minOrderValue = customAttributes?.minOrderAmount

    return payment
}

fun constructGiftCardPaymentInfo(odsPayload: Payload?): List<WismoOrderResponse.GiftCardPayment?> {

    val giftCardPaymentList = ArrayList<WismoOrderResponse.GiftCardPayment>()

    val paymentMethod = odsPayload?.paymentMethods

    paymentMethod?.let {
        for (paymentCard in it) {
            if (paymentCard?.paymentType == GIFTCARD_PAYMENT_TYPE) {
                val giftCardPayment = WismoOrderResponse.GiftCardPayment()
                giftCardPayment.maskedGcNum = paymentCard.last4DigitsOfCard
                giftCardPayment.gcNum = paymentCard.cardNumber
                giftCardPayment.panelCode = paymentCard.panelCode
                giftCardPayment.amtApplied = paymentCard.amount
                giftCardPaymentList.add(giftCardPayment)
            }
        }
    }

    return giftCardPaymentList
}

fun constructShoppingItem(orderLine: OrderLine?, shouldShowUnavailableItemList: Boolean): ViewOrderResponse.Order.OrderItem? {
    orderLine?.let {
        val item = ViewOrderResponse.Order.OrderItem()
        val orderProduct = it.orderProduct
        val orderCustomAttributes = it.orderLineCustomAttributes

        item.cost = orderCustomAttributes?.totalPrice
        item.desc = orderProduct?.description

        // TODO : Not Used
        // item.itemName = orderProduct
        // item.brandName = orderProduct
        // bundleDiscount
        // primaryShelf

        val offerLogistics = orderProduct?.offerLogistics

        item.id = offerLogistics?.skuId
        item.cin = offerLogistics?.cin
        item.imageURL = offerLogistics?.itemImageURL
        item.promoType = orderProduct?.offerAttributes?.offerType

        item.pricePerWt = if (it.unitOfMeasurement?.equals(KILOGRAMS).orFalse()
        ) ProductDetails.PRICE_PER_WEIGHT_WEIGHTED else orderProduct?.pricePerUOM
        item.pricePerUOM = orderProduct?.pricePerUOM

        val nilPick = orderCustomAttributes?.nilPickQty?.toInt() ?: 0
        var orderedQty = it.orderedQty?.measurementValue?.toInt()

        if (shouldShowUnavailableItemList) {
            orderedQty = it.orderedQty?.measurementValue?.toInt()?.minus(nilPick)
        }

        item.qty = if (it.unitOfMeasurement?.equals("KG", true) == true)
            it.weight + it.orderedQty?.unitOfMeasure
        else orderedQty?.toString()
        item.maxQty = orderProduct?.maxQty?.toString()
        item.bundledItemCount = orderProduct?.bundleItemCount ?: 0
        item.deptName = it.deptName
        // TODO : below calculation can be removed once V4 api sends the pice in the response.
        item.price = it.orderedQty?.measurementValue?.times((it.unitPrice?.currencyAmount) as BigDecimal).toString()
        item.wasPrice = orderCustomAttributes?.wasPrice
        item.weight = orderCustomAttributes?.skuSize
        item.substitutionOption = it.customerPreferences?.isSubstitutable ?: false
        item.associatedWithDish = orderCustomAttributes?.associatedWithDish ?: false
        item.numberOfRecipes = orderCustomAttributes?.numberOfRecipes ?: 0
        return item
    }
    return null
}

fun constructSubstituteItem(orderLine: OrderLine?): WismoOrderResponse.SubstitutedItem {
    val substitutedItem = WismoOrderResponse.SubstitutedItem()
    val pickedItem = orderLine?.pickedItem

    substitutedItem.id = pickedItem?.id
    substitutedItem.description = pickedItem?.description
    substitutedItem.quantity = pickedItem?.substitutedQuantity.toString()
    substitutedItem.price = pickedItem?.unitPrice?.toString()
    substitutedItem.totalPrice = orderLine?.orderLineCustomAttributes?.substitutedPrice?.toString()

    val subItemList: MutableList<WismoOrderResponse.SubItem> = ArrayList()
    for (subItem in pickedItem?.substitutedItemList.orEmpty()) {
        subItemList.add(mapSubItem(subItem))
    }
    substitutedItem.sub = subItemList.toTypedArray()

    return substitutedItem
}

fun mapSubItem(subItem: SubstitutedItem?): WismoOrderResponse.SubItem {
    val item = WismoOrderResponse.SubItem()
    item.subdescription = subItem?.description
    item.imageUrl = subItem?.imageURL
    item.pricePerUOM = subItem?.pricePerUOM
    item.subprice = subItem?.unitPrice?.times((subItem.quantity ?: 0.0) as BigDecimal).toString()
    item.subquantity = subItem?.quantity.toString()
    item.totalPrice = subItem?.priceCharged?.toPlainString()
    return item
}

fun convertTime(time: String?): String {
    if (time != null) {
        try {
            val inputFormat = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS", Locale.UK)
            val outputFormat = SimpleDateFormat("EEEE, dd MMMM yyyy", Locale.UK)
            inputFormat.timeZone = TimeZone.getTimeZone("UTC")
            outputFormat.timeZone = TimeZone.getTimeZone("Europe/London")
            val date = inputFormat.parse(time)
            return outputFormat.format(date)
        } catch (e: ParseException) {
            Log.e(TAG, "error parsing the date :" + e.message)
        }
    }
    return ""
}

fun isChangesToPreviewNeeded(cartResponse: CartResponse?): Boolean {
    val deliveryImpactFragmentTag = getDeliveryImpactFragmentTag(cartResponse)
    return cartResponse?.data?.cart?.changesToPreview != null && deliveryImpactFragmentTag.isNotEmpty()
}

fun getDeliveryImpactFragmentTag(cartResponse: CartResponse?): String {
    val deliveryImpact = cartResponse?.data?.cart?.changesToPreview
    return when {
        deliveryImpact?.unavailableRecipes?.isNotEmpty() == true -> UnavailableRecipesFragment.TAG
        deliveryImpact?.ingredientsUnavailableRecipes?.isNotEmpty() == true -> ImpactedRecipeListFragment.TAG
        deliveryImpact?.priceChangedItems?.isNotEmpty() == true || deliveryImpact?.promoChangedItems?.isNotEmpty() == true ||
            deliveryImpact?.delistedItems?.isNotEmpty() == true -> PreviewBookDataSource.TAG
        else -> ""
    }
}
