package com.asda.android.app.recurringslot

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.asda.android.R
import com.asda.android.analytics.AnalyticsConfig
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.view.AccountFragment
import com.asda.android.app.account.view.BaseAccountActivity
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.base.core.view.ViewUtil.openWebPage
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.databinding.AccountRecurringSlotsBinding
import com.asda.android.home.AsdaHomeConfig
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.features.account.view.WelcomeFragment
import com.asda.android.slots.bookslot.adapter.SubscriptionAdapter
import com.asda.android.slots.bookslot.bookslotv3.service.BookSlotRepository
import com.asda.android.slots.recurringslot.repository.SlotSubscriptionRepository
import com.asda.android.slots.utils.SlotSubscriptionManager
import com.asda.android.slots.utils.SlotUtils
import io.reactivex.observers.ResourceSingleObserver

class RecurringSlotDetailsFragment : FeaturedFragment() {
    private lateinit var mModel: RecurringSlotAccountViewModel
    private var slotSubscriptionRepository: SlotSubscriptionRepository? = null
    private lateinit var fragmentBinder: AccountRecurringSlotsBinding
    private val CALLED_FROM_SKIP = "CALLED_FROM_SKIP"
    private val CALLED_FROM_CANCEL = "CALLED_FROM_CANCEL"

    override fun onAttach(context: Context) {
        super.onAttach(context)
        pageViewEvent = PageViewEvent(Anivia.PV_RECURRING_SLOT_DETAILS, Anivia.SECTION_ACCOUNT, Anivia.SECTION_ACCOUNT)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val binding = AccountRecurringSlotsBinding.inflate(inflater, container, false)
        if (RestUtils.isTablet(context) && parentFragment is AccountFragment) { // If on split screen need to use wrap_content width
            val params = binding.content.layoutParams
            params.width = ViewGroup.LayoutParams.WRAP_CONTENT
            binding.content.layoutParams = params
        }
        mModel = ViewModelProviders.of(this).get(RecurringSlotAccountViewModel::class.java)
        mModel.attach(this)
        binding.model = mModel
        fragmentBinder = binding

        return fragmentBinder.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        slotSubscriptionRepository = SlotSubscriptionRepository()
        context?.let {
            fetchSubscriptionForMigratedUsers()
        }
    }

    private fun fetchSubscriptionForMigratedUsers() {
        SingleProfile.getProfileResponse()?.profileId?.let { profileId ->
            slotSubscriptionRepository?.getSubscriptions(profileId, true)?.subscribe(object : ResourceSingleObserver<SubscriptionsResponse>() {
                override fun onSuccess(subscriptionsResponse: SubscriptionsResponse) {
                    dismissProgress()
                    if (isAdded && !mModel.setData(subscriptionsResponse)) {
                        closeRecurringSlotScreen()
                    }
                }

                override fun onError(e: Throwable) {
                    dismissProgress()
                    if (isAdded) {
                        showDialog(null, null)
                    }
                }
            })
        }
    }

    override fun hasActionBar(): Boolean {
        val activity = activity
        return activity is AsdaActivity || activity is DeliveryPassPurchaseActivity || activity is BaseAccountActivity || parentFragment is AccountFragment || activity is DeliveryImpactActivity
    }

    override fun getActionBarTitle(): String? {
        return if (activity != null) getString(R.string.recurring_slot_title) else ""
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    //region SKIP SLOT
    fun onClickSkip(id: String?, bookingId: String?, date: String?) {
        context?.let {
            val builder = AlertDialog.Builder(it)
            builder.setTitle(R.string.are_you_sure)
                .setMessage(getString(R.string.skip_recurring, date))
                .setPositiveButton(R.string.ok) { _, _ -> doSkip(bookingId, id) }
                .setNegativeButton(R.string.cancel, null)
                .create()
                .show()
        }
    }

    private fun doSkip(bookingId: String?, id: String?) {
        showProgress(R.string.please_wait)
        context?.let {
            if (bookingId != null) {
                skipRecurringSlotForMigratedUser(bookingId)
            }
        }
    }

    private fun skipRecurringSlotForMigratedUser(bookingId: String) {
        val data = getSubscriptionCreation(bookingId)
        slotSubscriptionRepository?.cancelSubscriptionMigratedUser(bookingId, data)
            ?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                override fun onSuccess(t: SubscriptionCreation) {
                    if (isAdded) {
                        SlotSubscriptionManager.refreshSubscriptionEligibility()
                        serviceEligibilityCallToUpdateStoreId(CALLED_FROM_SKIP)
                    }
                    dismissProgress()
                    mModel.subscriptionAdapter?.updateToSkipState(bookingId)
                    mModel.triggerRsEvent(Tracker.get(), AsdaAnalyticsEvent(Anivia.RS_SKIP_REC_SLOT))
                }

                override fun onError(e: Throwable) {
                    dismissProgress()
                }
            })
    }

    private fun getSubscriptionCreation(bookingId: String): SubscriptionCreation {
        val subscription = SubscriptionCreation()
        val subscriptionCreationData = SubscriptionCreationData()
        val subscriptionInfo = SubscriptionInfo()
        subscriptionInfo.subscription_bookings = listOf(SubscriptionBooking(booking_id = bookingId, status = SubscriptionBooking.SKIP_INITIATED))
        subscriptionCreationData.subscription_info = subscriptionInfo
        subscription.data = subscriptionCreationData
        return subscription
    }

    fun setAdapter(subscriptionAdapter: SubscriptionAdapter?) {
        subscriptionAdapter?.let { _adapter ->
            fragmentBinder.rsRecurringSlots.apply {
                layoutManager = LinearLayoutManager(context)
                adapter = _adapter
            }
            _adapter.notifyDataSetChanged()
        }
    }

    private fun serviceEligibilityCallToUpdateStoreId(calledFrom: String) {
        BookSlotRepository().makeServiceEligibilityCall(
            SlotUtils.getDefaultPostCodeFromSingleProfile(),
            includeAccessPointList = false
        )?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
            override fun onSuccess(serviceEligibilityData: ServiceEligibilityData) {
                val serviceEligibility = serviceEligibilityData.data?.service_eligibility
                val accessPointList = serviceEligibility?.first()?.accesspoint_list
                if (serviceEligibility?.isNullOrEmpty() == false &&
                    accessPointList?.isNullOrEmpty() == false &&
                    accessPointList.first().fulfill_store_id.isNullOrEmpty().not()
                ) {
                    serviceEligibility.first().accesspoint_list?.first()?.fulfill_store_id?.let {
                        if (it.isNotEmpty()) {
                            AsdaHomeConfig.asdaService.setStoreId(it)
                        }
                    }
                    SlotSubscriptionManager.refreshSubscriptionEligibilityForATGUsers()
                }
                if (calledFrom == CALLED_FROM_SKIP) {
                    refreshCXOCart()
                    SlotSubscriptionManager.refreshSubscriptionEligibility()
                } else { // CALLED_FROM_CANCEL
                    refreshCXOCart()
                    closeRecurringSlotScreen()
                }
            }

            override fun onError(e: Throwable) {
                // Do nothing
            }
        })
    }

    //endregion SKIP SLOT

    override fun onCreateDialog(dialog: Int): Dialog? {
        when (dialog) {
            DIALOG_CANCEL -> {
                context?.let {
                    val builder = AlertDialog.Builder(it)
                    return builder.setTitle(R.string.are_you_sure)
                        .setMessage(getString(R.string.dp_cancel_recurring_slot))
                        .setPositiveButton(R.string.keep_slot, null)
                        .setNegativeButton(R.string.cancel_slot) { _, _ ->
                            cancelSlot()
                        }
                        .create()
                }
            }
        }
        return null
    }

    //region CANCEL SLOT

    private fun cancelSlot() {
        showProgress(R.string.please_wait)
        context?.let {
            onSubscriptionCancel()
        }
    }

    private fun refreshCXOCart() {
        com.asda.android.cxo.CartManager.instance.refreshCart()
    }

    private fun onSubscriptionCancel() {
        val subscription = SlotSubscription(mModel.subscriptionInfo)
        subscription.subscription_info?.status = SubscriptionSlotStatus.CANCEL_INITIATED.name

        val singleProfileId = SingleProfile.getProfileResponse()?.profileId
        if (singleProfileId.isNullOrEmpty()) return

        slotSubscriptionRepository?.cancelSubscription(
            singleProfileId,
            subscription.subscription_info?.subscription_id
                ?: "",
            SubscriptionCreation(SubscriptionCreationData(subscription.subscription_info))
        )?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
            override fun onSuccess(t: SubscriptionCreation) {
                if (isAdded) {
                    SlotSubscriptionManager.refreshSubscriptionEligibility()
                    serviceEligibilityCallToUpdateStoreId(CALLED_FROM_CANCEL)
                    AnalyticsConfig.activeRecurringSlot = false
                    mModel.triggerRsEvent(Tracker.get(), AsdaAnalyticsEvent(Anivia.RS_CANCEL_REC_SLOT))
                }
            }

            override fun onError(e: Throwable) {
                dismissProgress()
            }
        })
    }

    private fun closeRecurringSlotScreen() {
        if (!RestUtils.isTablet(context) && activity != null) {
            activity?.finish()
        } else if (activity is AsdaActivity) {
            val fragmentSwitcher = (activity as AsdaActivity).getFragmentSwitcher()
            if (fragmentSwitcher != null) {
                val fragment = fragmentSwitcher.getTopFragment()
                if (fragment is AccountFragment) {
                    fragment.pop()
                    fragment.onPushed(WelcomeFragment.newInstance(), true)
                    fragment.reset()
                }
            }
        }
    }

    fun onClickCancel() {
        showDialog(DIALOG_CANCEL)
    }
    //endregion CANCEL SLOT

    fun onClickFAQ() {
        push(RecurringSlotInfoFragment.newInstance())
    }

    fun onClickLearnMore(faqUrl: String) {
        context?.let { _context ->
            openWebPage(faqUrl, _context, TAG)
        }
    }

    companion object {

        const val TAG = "RecurringSlotDetailsFragment"
        private const val DIALOG_CANCEL = 1

        fun newInstance(): RecurringSlotDetailsFragment {
            return RecurringSlotDetailsFragment()
        }
    }
}
