package com.asda.android.app.settings.featureflag

import android.content.Context
import com.asda.android.R
import com.asda.android.abtestingframework.implementation.sitespect.constants.SitespectP13NMetrics
import com.asda.android.abtestingframework.implementation.sitespect.variations.Variations
import com.asda.android.app.main.AsdaApplication
import com.asda.android.authorization.BootstrapManager
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils

object FeatureSettings {
    const val SETTING_FORCE_OFF = 0
    const val SETTING_FORCE_ON = 1
    const val SETTING_HONOR_REMOTE_SETTING = 2 // This could be sitespect or killswitch or siteconfig.

    interface RemoteSetting {
        fun isFeatureEnabled(): Boolean
    }

    fun getFeatureSetting(context: Context, featureKey: String, defaultSetting: Int): Int {
        if (Utils.isDevBuild) {
            // NOTE: While the defaultSetting is used here, the setting for the feature in FeatureFlag item in settings.xml will override this value.
            return SharedPreferencesUtil.getSetting(context, featureKey, defaultSetting)
        } else {
            // for production, always return remote setting (sitespect or killswitch or siteconfig)
            return SETTING_HONOR_REMOTE_SETTING
        }
    }

    fun isFeatureEnabled(
        context: Context,
        featureKey: String,
        defaultSetting: Int = SETTING_FORCE_OFF,
        remoteSetting: RemoteSetting?
    ): Boolean {
        var enabled = false
        when (getFeatureSetting(context, featureKey, defaultSetting)) {
            SETTING_FORCE_ON -> enabled = true
            SETTING_FORCE_OFF -> enabled = false
            SETTING_HONOR_REMOTE_SETTING -> enabled = remoteSetting?.isFeatureEnabled() ?: false
        }
        return enabled
    }

    fun isCriteoEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_criteo),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getRemoteSettingCriterion(context)
        )
    }

    fun isGoogleAdEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_google_ad),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getRemoteSettingGoogleAds(context)
        )
    }

    fun isDynamicElementEnableOnStaticAds(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_dynamic_elements_on_static_ads),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = object : RemoteSetting {
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isDynamicElementsOnStaticAdEnabled(context)
                }
            }
        )
    }

    fun isRmpIntegrationEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_rmp_integration),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getRemoteSettingRmpIntegration(context)
        )
    }

    fun getRemoteSettingCriterion(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsCriteoEnabled(context)
        }
    }

    fun getRemoteSettingGoogleAds(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsGoogleAdsEnabled(context)
        }
    }

    fun getRemoteSettingRmpIntegration(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsRmpEnabled(context)
        }
    }

    fun isRelatedSearchesEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_related_searches),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsRelatedSearchesEnabled()
        )
    }

    fun getIsRelatedSearchesEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                BootstrapManager.getExperimentConfig().isRelatedSearchEnabled == true
            } else {
                false
            }
        }
    }

    fun isFMOEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_fmo),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsFmoenabled(context)
        )
    }

    fun getIsFmoenabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsFMOEnabled(context)
        }
    }

    fun isSolrEACEngineEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_solr_eac),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getIsSolrEACEngineEnabled()
        )
    }

    fun getIsSolrEACEngineEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(Variations.EACSolrVariation)
            } else {
                false
            }
        }
    }

    fun isSolrSearchEngineEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_solr_search),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getIsSolrRemoteSetting()
        )
    }

    fun getIsSolrRemoteSetting() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(Variations.SearchSolrVariation)
            } else {
                false
            }
        }
    }

    fun isFirebaseAnalyticsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_firebase_analytics),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getFirebaseAnalyticsEnabled(context)
        )
    }

    fun getFirebaseAnalyticsEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns killswitch value
            return SharedPreferencesUtil.getFirebaseAnalyticsMobileConfigEnabledFlag(context)
        }
    }

    fun isFirebasePerformanceEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_firebase_performance_metrics),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getisFirebasePerformanceEnabled(context)
        )
    }

    fun getisFirebasePerformanceEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns killswitch value
            return SharedPreferencesUtil.getFirebasePerformanceEnabledFlag(context)
        }
    }

    fun isBottomNavOrderTestEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_bottom_nav_order),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisBottomNavOrderTestEnabled()
        )
    }

    fun getisBottomNavOrderTestEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return BootstrapManager.getExperimentConfig().isBottomNavOrderTestEnabled == true
        }
    }

    fun isAllOffersP13NEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_all_offers_p13n),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsAllOffersP13Enabled()
        )
    }

    fun getIsAllOffersP13Enabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return if (Authentication.getInstance() != null) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasCampaign(SitespectP13NMetrics.P13N_ALL_OFFERS_CAMPAIGN_ID) &&
                    (
                        AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_ALL_OFFERS_VARIATION_ID) ||
                            AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_ALL_OFFERS_ORCH_VARIATION_ID)
                        )
            } else {
                false
            }
        }
    }

    fun isTempoFavoriteEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_tempo_fav),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getIsTempoFavouriteEnabled(context)
        )
    }

    fun isHooklogicOnNewFavoritesEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_new_fav_hooklogic),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getIsHooklogicOnNewFavoritesEnabled(context)
        )
    }

    fun isAllOffersPageNewType(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.all_offers_tempo_page_type),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsNewPageTypeAllOffers(context)
        )
    }

    fun getIsNewPageTypeAllOffers(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isNewPageTypeAllOffers(context)
        }
    }

    fun getIsTempoFavouriteEnabled(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isTempoFavoriteEnabled(context) &&
                BootstrapManager.getExperimentConfig().isTempoFavorites == true
        }
    }

    fun getIsHooklogicOnNewFavoritesEnabled(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsHooklogicOnNewFavoritesEnabled(context) &&
                BootstrapManager.getExperimentConfig().isHooklogicOnNewFavoritesEnabled == true
        }
    }

    fun isBFFforODSEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_bff_for_ods),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getIsBFFforODSEnabled(context)
        )
    }

    fun getIsBFFforODSEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isBFFforODSEnabled(context)
        }
    }

    fun isBFFBootstrapEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_bff_for_bootstrap),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getIsBFFforBootstrapEnabled(context)
        )
    }

    fun isNewHomePageEnable(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_new_homepage),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getIsNewHomePageEnable(context)
        )
    }

    fun getIsBFFforBootstrapEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isBFFBootstrapEnabled(context)
        }
    }

    fun getIsNewHomePageEnable(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isNewHomePageEnable(context) && BootstrapManager.getExperimentConfig().isNewHomepageEnabled ?: false
        }
    }

    fun isSingleProfileIdCheckinEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_is_single_profile_id_checkin_enabled),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isSingleProfileIdCheckinEnabled(context)
                }
            }
        )
    }

    fun isDPOrderSummaryBannerEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_dp_order_summary_banner),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = getisDPOrderSummaryBannerEnabled(context)
        )
    }

    fun isCardinalSessionUITypeHTMLEnable(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_is_cardinal_session_uitype_html_enable),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isCardinalSessionUITypeHTMLEnable(context)
                }
            }
        )
    }

    fun getisDPOrderSummaryBannerEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isDPOrderSummaryBannerEnabled(context)
        }
    }

    fun getisP13NBYGRecipeEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return AsdaApplication.getComponent().sitespectAbTestEngine.hasCampaign(SitespectP13NMetrics.P13N_BYG_RECIPE_CAMPAIGN_ID) &&
                AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_BYG_RECIPE_VARIATION_ID)
        }
    }

    fun isSelfIsolationEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_self_isolation),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getisSelfIsolationEnabled(context)
        )
    }

    fun getisSelfIsolationEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns firebase value
            return SharedPreferencesUtil.getIsSelfIsolationEnabled(context)
        }
    }

    fun isSubstitutionMessageEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_substitution_policy_update),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getisSubstitutionMessageEnabled(context)
        )
    }

    fun getisSubstitutionMessageEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns firebase value
            return SharedPreferencesUtil.getIsSubstitutionPolicyUpdateEnabled(context)
        }
    }

    fun isSlotBlockedByAmends(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_slot_blocked_by_amends),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisSlotBlockedByAmends(context)
        )
    }

    fun getisSlotBlockedByAmends(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsSlotBlockedByAmends(context)
        }
    }

    fun isCovid19BannerOnBookSlotPageEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_c19_banner_on_bookslot),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisCovid19BannerOnBookSlotPageEnabled(context)
        )
    }

    fun getisCovid19BannerOnBookSlotPageEnabled(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsCOVID19BannerEnabledOnBookSlot(context)
        }
    }

    fun isDeliveryPassBlocked(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_delivery_pass_blocked),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisDeliveryPassBlocked(context)
        )
    }

    fun getisDeliveryPassBlocked(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getDeliveryPassBlocked(context)
        }
    }

    fun isPaymentAuthBannerVisible(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_payment_auth_banner_setting),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisPaymentAuthBannerVisible(context)
        )
    }

    fun getisPaymentAuthBannerVisible(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getPaymentAuthBannerVisibility(context)
        }
    }

    fun isThreedsEnabledForDp(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_three_ds_dp),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getIsThreedsEnabledForDp(context)
        )
    }

    fun isThreedsEnabledForEditDp(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_three_ds_edit_dp),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getIsThreedsEnabledForEditDp(context)
        )
    }

    fun getIsThreedsEnabledForEditDp(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getThreeDSV2EnabledFlagForEditDp(context)
        }
    }

    fun getIsThreedsEnabledForDp(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getThreeDSV2EnabledFlagForDp(context)
        }
    }

    fun isThreedsEnabledAmex(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_three_ds_amex),
            defaultSetting = SETTING_FORCE_ON,
            remoteSetting = getisThreedsEnabledAmex(context)
        )
    }

    fun getisThreedsEnabledAmex(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getAmexCardsEnabledFlag(context)
        }
    }

    // This flag is added to start passing single profile Id instead ATG profile id
    fun isProfileIdAcceptedInBookSlot(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_profile_id_accepted_for_bookslot),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getIsProfileIdAcceptedForBookSlot(context)
                }
            }
        )
    }

    fun isSlotEligibilityMinPurchaseEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_service_eligibility_light_weight),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getIsSlotEligibilityLighWeightEnabled(context)
                }
            }
        )
    }

    fun isNewOrdersModuleEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_new_orders_module),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isNewOrdersModuleEnabled(context) && BootstrapManager.getExperimentConfig().isNewOrdersModuleEnabled == true
                }
            }
        )
    }

    fun isChatBotModuleEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_chat_bot),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isChatBotEnabled(context)
                }
            }
        )
    }

    fun isOSMMessageServiceEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_osm_message),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isOSMMessageEnabled(context)
                }
            }
        )
    }

    fun isRecurringSlotDisabledForNormalDP(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_recurring_slot_disabled_for_normal_dp),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisRecurringSlotDisabledForNormalDP(context)
        )
    }

    fun getisRecurringSlotDisabledForNormalDP(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isRecurringSlotDisabledForNormalDP(context)
        }
    }

    fun isExpressHDEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_express_hd),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getisExpressHDEnabled(context)
        )
    }

    fun getisExpressHDEnabled(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsExpressHDEnabled(context)
        }
    }

    fun isFavoriteBffApiEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_favorite_api),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsFavouriteBffApiEnabled(context)
        )
    }

    fun getIsFavouriteBffApiEnabled(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsBffFavoriteApiEnabled(context)
        }
    }

    fun isSponsoredLabelEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_sponsored_label),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = getIsSponsoredLabel(context)
        )
    }

    fun getIsSponsoredLabel(context: Context) = object : RemoteSetting {
        // Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsSponsoredLabelEnabled(context)
        }
    }

    fun isSlotsDateTimeAddressForReservedStatusEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_use_addr_date_time_for_reserved_slot_indicator),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getIsSlotsDateTimeAddressForReservedStatusEnabled(context)
                }
            }
        )
    }

    fun isEGiftCardsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_e_gift_cards_feature),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isEGiftCardsEnabled(context)
                }
            }
        )
    }

    fun isAdsDecisionEngineEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_ads_decision_engine),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isAdsDecisionEngineEnabled(context) && BootstrapManager.getExperimentConfig().isAdsDecisionEngineEnabled == true
                }
            }
        )
    }

    fun isSlotSecondaryStoreEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_is_slot_secondary_store_enabled),
            defaultSetting = SETTING_FORCE_OFF,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getIsSlotSecondaryStoreEnabled(context)
                }
            }
        )
    }

    fun isStoreIdForListDpEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_list_dp_store_id),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isStoreIdForListDpEnabled(context)
                }
            }
        )
    }

    fun isSubscriptionAPIEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_is_subscription_api_enabled),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.isSubscriptionAPIEnabled(context)
                }
            }
        )
    }

    fun isTrackOrderEnabled(context: Context) = isFeatureEnabled(
        context = context,
        featureKey = context.getString(R.string.debug_settings_track_order_enabled),
        remoteSetting = object : RemoteSetting {
            // Firebase config value
            override fun isFeatureEnabled() = SharedPreferencesUtil.isTrackOrderEnabled(context)
        }
    )

    fun isDPBannerForSlotsEnabled(context: Context) = isFeatureEnabled(
        context = context,
        featureKey = context.getString(R.string.debug_settings_dp_banner_slot_enabled),
        remoteSetting = object : RemoteSetting {
            // Firebase config value
            override fun isFeatureEnabled() = SharedPreferencesUtil.isFlexSlotEnabled(context)
        }
    )

    fun isFlexSlotEnabled(context: Context) = isFeatureEnabled(
        context = context,
        featureKey = context.getString(R.string.debug_settings_flex_slot_enabled),
        remoteSetting = object : RemoteSetting {
            // Firebase config value
            override fun isFeatureEnabled() = SharedPreferencesUtil.isFlexSlotEnabled(context)
        }
    )

    fun isOffersForYouEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_settings_p13n_offers_for_you_feature),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getP13NOffersEnabledFlag(context) &&
                        (
                            "variantA".equals(BootstrapManager.getExperimentConfig().isOffersForYouEnabled) ||
                                "variantB".equals(BootstrapManager.getExperimentConfig().isOffersForYouEnabled)
                            )
                }
            }
        )
    }

    fun isMobileAppPreviewEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_mobile_preview),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getMobileAppPreviewFlag(context)
                }
            }
        )
    }

    fun isSearchExpEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_search_experiment_enabled),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getSearchExperiment(context)
                }
            }
        )
    }

    fun isP13NUserSegmentsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
            context = context,
            featureKey = context.getString(R.string.debug_p13n_user_seg_experiment_enabled),
            defaultSetting = SETTING_HONOR_REMOTE_SETTING,
            remoteSetting = object : RemoteSetting {
                // Firebase config value
                override fun isFeatureEnabled(): Boolean {
                    return SharedPreferencesUtil.getIsP13NUserSegEnabled(context)
                }
            }
        )
    }
}
