package com.asda.android.app.storelocator

import androidx.annotation.VisibleForTesting
import androidx.collection.ArrayMap
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.model.AnalyticsEvent
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.storelocator.constants.StoreLocatorConstants.filters
import com.asda.android.app.storelocator.constants.StoreLocatorConstants.sActiveFilters
import com.asda.android.base.core.utils.orFalse
import com.asda.android.service.EventUtils.broadcast
import com.asda.android.service.constants.EventConstants
import java.util.*

class StoresFilterManager : FilterManager() {

    @VisibleForTesting
    fun getMFiltersToChange() = ArrayMap<String, Boolean>()

    override fun startEditing() {
        getMFiltersToChange().clear()
        getTracker().onAnalyticsEvent(AnalyticsEvent(Anivia.PV_STOREFINDER_FILTER, "", ""))
    }

    override fun cancelEditing() {
        getMFiltersToChange().clear()
    }

    override fun stopEditingAndSave() {
        var filterChanged = false
        val selectedFilters = StringBuilder()
        for (filter in filters) {
            val currentlyActive = sActiveFilters.contains(filter)
            val requiredState = getMFiltersToChange()[filter.name]
            if (requiredState != null && requiredState != currentlyActive) {
                filterChanged = true
                if (java.lang.Boolean.TRUE == requiredState) {
                    sActiveFilters.add(filter)
                    if (selectedFilters.isNotEmpty()) {
                        selectedFilters.append("|")
                    }
                    selectedFilters.append(filter.name)
                } else {
                    sActiveFilters.remove(filter)
                }
            }
        }
        getMFiltersToChange().clear()
        if (filterChanged) {
            onFilterSettingsUpdated()
            broadcast(
                AsdaApplication.getInstance(), EventConstants.STORE_FILTER_CHANGED
            )
            getTracker()
                .trackEvent(
                    AsdaAnalyticsEvent(Anivia.FILTER_APPLY_EVENT)
                        .putString(Anivia.FILTER_SELECTED_KEY, selectedFilters.toString())
                        .putString(
                            Anivia.ENGAGEMENT_REPORT_KEY,
                            "find a store : filter : $selectedFilters"
                        )
                )
        }
    }

    override fun clearAll() {
        val hadFilters = isFilterActive().orFalse()
        sActiveFilters.clear()
        if (hadFilters) {
            onFilterSettingsUpdated()
            broadcast(
                AsdaApplication.getInstance(), EventConstants.STORE_FILTER_CHANGED
            )
        }
    }

    override fun setServiceCheck(i: Int, check: Boolean) {
        val serviceName = filters[i].name
        getMFiltersToChange()[serviceName] = check
    }

    override fun getServicesChecks(): BooleanArray {
        val checks = BooleanArray(filters.size)
        for (i in checks.indices) {
            checks[i] = isFilterActive(filters[i].name)
        }
        return checks
    }

    override fun isFilterActive(): Boolean {
        return sActiveFilters.isNotEmpty()
    }

    override fun getServicesNames(): Array<String?> {
        val serviceNames = arrayOfNulls<String>(filters.size)
        var i = 0
        for (filter in filters) {
            serviceNames[i++] = filter.name
        }
        return serviceNames
    }

    override fun getActiveFilters(): ArrayList<String> {
        val activeFilterNames = ArrayList<String>(sActiveFilters.size)
        for (filter in sActiveFilters) {
            activeFilterNames.add(filter.name)
        }
        return activeFilterNames
    }

    @VisibleForTesting
    fun getTracker(): Tracker {
        return Tracker.get()
    }

    class Filter(val name: String)

    @VisibleForTesting
    fun onFilterSettingsUpdated() {
        val listener = getListener()
        listener?.onFilterSettingsUpdated()
    }

    @VisibleForTesting
    fun isFilterActive(filterName: String?): Boolean {
        if (filterName != null) {
            for (filter in sActiveFilters) {
                if (filterName == filter.name) {
                    return true
                }
            }
        }
        return false
    }

    companion object {
        private var sInstance: StoresFilterManager? = null
        @JvmStatic
        val instance: StoresFilterManager?
            get() {
                if (sInstance == null) {
                    sInstance = StoresFilterManager()
                }
                return sInstance
            }
    }
}
