package com.asda.android.exceptionHandlers

import android.app.Application
import android.util.Log
import com.asda.android.analytics.events.GenericErrorEvent
import com.asda.android.analytics.events.exception.AndroidLogWrapper
import com.asda.android.analytics.interfaces.ITracker
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.interfaces.IAsdaLogger
import com.asda.android.restapi.interfaces.IFeatureSettingManager
import com.asda.android.restapi.service.base.RxFailure
import com.google.firebase.crashlytics.FirebaseCrashlytics

/**
 * Use this class to log exceptions or in-app stats to crashlytics or splunk(via anivia tracking)
 */
class AsdaLogger : IAsdaLogger {

    override fun reportNonFatal(ex: Exception, customMessage: String?) =
        FirebaseCrashlytics.getInstance().recordException(
            Exception(
                customMessage.orEmpty() + ex.message, ex
            )
        )

    override fun log(
        eventName: String?,
        error: IAsdaLogger.AsdaError?,
        additionalInfo: MutableMap<String, Any>,
        loggerConfig: IAsdaLogger.LoggerConfig,
        message: String?
    ) {

        if (getFeatureSettingsManager().isInAppReportingEnabled(getApplication()).not()) {
            return
        }

        error?.exception?.tryCast<RxFailure> {
            additionalInfo[ASDA_RESPONSE] = this.asdaResponse?.toString().orEmpty()
        }

        composeError(error, additionalInfo)

        val genericErrorEvent = GenericErrorEvent(
            eventName,
            getStackTrace(loggerConfig),
            message,
            additionalInfo
        )

        Log.d(TAG, genericErrorEvent.toString())

        getTracker()?.trackEvent(
            genericErrorEvent
        )
    }

    private fun composeError(
        error: IAsdaLogger.AsdaError?,
        additionalInfo: MutableMap<String, Any>
    ) {
        error?.let { asdaError ->
            val errorMap = mutableMapOf<String, Any>()
            asdaError.exception?.let {
                errorMap[GenericErrorEvent.EXCEPTION] = it
            }
            asdaError.desc?.let {
                errorMap[GenericErrorEvent.ERROR_DESC] = it
            }
            asdaError.statusCode?.let {
                errorMap[GenericErrorEvent.STATUS_CODE] = it
            }
            asdaError.statusCode?.let {
                errorMap[GenericErrorEvent.ERROR_MESSAGE] = it
            }
            asdaError.exceptionStackTrace?.let {
                errorMap[GenericErrorEvent.ERROR_TRACE] = it
            }
            additionalInfo[GenericErrorEvent.ERROR] = errorMap
        }
    }

    private fun getStackTrace(loggerConfig: IAsdaLogger.LoggerConfig): String? {
        if (loggerConfig.skipStackTrace) {
            return null
        }
        return AndroidLogWrapper.getStackTrace()
    }

    private fun getTracker(): ITracker? {
        return AsdaBaseCoreConfig.tracker
    }

    private fun getApplication(): Application {
        return AsdaBaseCoreConfig.application
    }

    private fun getFeatureSettingsManager(): IFeatureSettingManager {
        return AsdaBaseCoreConfig.featureSettingManager
    }

    companion object {
        private const val ASDA_RESPONSE = "asdaResponse"
        private const val TAG = "AsdaLogger"
    }
}
