package com.asda.android.lib

import android.content.Context
import android.os.Build
import android.preference.PreferenceManager
import com.asda.android.app.main.AsdaApplication
import com.kazakago.cryptore.BlockMode
import com.kazakago.cryptore.CipherAlgorithm
import com.kazakago.cryptore.Cryptore
import com.kazakago.cryptore.EncryptionPadding
import java.io.IOException
import java.security.*
import java.security.cert.CertificateException
import javax.crypto.NoSuchPaddingException

class CryptoUtils private constructor(private val mContext: Context) {

    private enum class Alias(var value: String) {
        RSA("CIPHER_RSA"), AES("CIPHER_AES");
    }

    private var mCryptoCore: Cryptore? = null

    @Throws(KeyStoreException::class, NoSuchPaddingException::class, InvalidAlgorithmParameterException::class, NoSuchAlgorithmException::class, IOException::class, UnrecoverableEntryException::class, NoSuchProviderException::class, InvalidKeyException::class)
    private fun encryptRSA(plainStr: String): String {
        if (mCryptoCore == null) {
            throw KeyStoreException(KEY_STORE_NOT_AVAILABLE)
        }
        val plainByte = plainStr.toByteArray()
        val result = mCryptoCore?.encrypt(plainByte)
        return Base64.encodeToString(result?.bytes, Base64.DEFAULT)
    }

    @Throws(KeyStoreException::class, UnrecoverableKeyException::class, InvalidAlgorithmParameterException::class, NoSuchAlgorithmException::class, InvalidKeyException::class, IOException::class)
    fun decryptRSA(encryptedStr: String?): String {
        if (mCryptoCore == null) {
            throw KeyStoreException(KEY_STORE_NOT_AVAILABLE)
        }
        val encryptedByte = Base64.decode(encryptedStr, Base64.DEFAULT)
        val result = mCryptoCore?.decrypt(encryptedByte, null)
        return String(result?.bytes ?: ByteArray(0))
    }

    @Throws(KeyStoreException::class, NoSuchPaddingException::class, InvalidAlgorithmParameterException::class, NoSuchAlgorithmException::class, IOException::class, UnrecoverableEntryException::class, NoSuchProviderException::class, InvalidKeyException::class)
    private fun encryptAES(plainStr: String): String {
        if (mCryptoCore == null) {
            throw KeyStoreException(KEY_STORE_NOT_AVAILABLE)
        }
        val plainByte = plainStr.toByteArray()
        val result = mCryptoCore?.encrypt(plainByte)
        saveCipherIV(result?.cipherIV)
        return Base64.encodeToString(result?.bytes, Base64.DEFAULT)
    }

    @Throws(KeyStoreException::class, UnrecoverableKeyException::class, InvalidAlgorithmParameterException::class, NoSuchAlgorithmException::class, InvalidKeyException::class, IOException::class)
    private fun decryptAES(encryptedStr: String?): String {
        if (mCryptoCore == null) {
            throw KeyStoreException(KEY_STORE_NOT_AVAILABLE)
        }
        val encryptedByte = Base64.decode(encryptedStr, Base64.DEFAULT)
        val cipherIV = loadCipherIV()
        val result = mCryptoCore?.decrypt(encryptedByte, cipherIV)
        return String(result?.bytes ?: ByteArray(0))
    }

    @Throws(UnrecoverableKeyException::class, NoSuchAlgorithmException::class, KeyStoreException::class, InvalidKeyException::class, InvalidAlgorithmParameterException::class, IOException::class)
    fun decrypt(str: String?): String {
        return if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP_MR1) {
            decryptRSA(str)
        } else {
            decryptAES(str)
        }
    }

    @Throws(NoSuchPaddingException::class, InvalidKeyException::class, NoSuchAlgorithmException::class, KeyStoreException::class, UnrecoverableEntryException::class, NoSuchProviderException::class, InvalidAlgorithmParameterException::class, IOException::class)
    fun encrypt(plainStr: String): String {
        return if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP_MR1) {
            encryptRSA(plainStr)
        } else {
            encryptAES(plainStr)
        }
    }

    companion object {
        private const val TAG = "CryptoUtils"
        private const val KEY_STORE_NOT_AVAILABLE = "Android KeyStore not available"
        private var sInstance: CryptoUtils? = null

        @JvmStatic
        @Synchronized
        fun getInstance(context: Context): CryptoUtils? {
            if (sInstance == null) {
                sInstance = CryptoUtils(context)
            }
            return sInstance
        }

        @Throws(KeyStoreException::class, CertificateException::class, NoSuchPaddingException::class, NoSuchAlgorithmException::class, IOException::class, NoSuchProviderException::class, InvalidAlgorithmParameterException::class)
        private fun getCryptoreRSA(context: Context): Cryptore {
            val builder = Cryptore.Builder(Alias.RSA.value, CipherAlgorithm.RSA)
            builder.context = context // Need Only RSA on below API Lv22.
            builder.blockMode = BlockMode.ECB
            builder.encryptionPadding = EncryptionPadding.RSA_PKCS1
            return builder.build()
        }

        // If Needed.
        // If Needed.
        @get:Throws(KeyStoreException::class, CertificateException::class, NoSuchPaddingException::class, NoSuchAlgorithmException::class, IOException::class, NoSuchProviderException::class, InvalidAlgorithmParameterException::class)
        private val cryptoreAES: Cryptore
            get() {
                val builder = Cryptore.Builder(Alias.AES.value, CipherAlgorithm.AES)
                builder.blockMode = BlockMode.CBC // If Needed.
                builder.encryptionPadding = EncryptionPadding.PKCS7 // If Needed.
                return builder.build()
            }

        private fun saveCipherIV(cipherIV: ByteArray?) {
            val preferences = PreferenceManager.getDefaultSharedPreferences(AsdaApplication.getInstance())
            val editor = preferences.edit()
            editor.putString("cipher_iv", Base64.encodeToString(cipherIV, Base64.DEFAULT))
            editor.apply()
        }

        private fun loadCipherIV(): ByteArray? {
            val preferences = PreferenceManager.getDefaultSharedPreferences(AsdaApplication.getInstance())
            val cipherIVStr = preferences.getString("cipher_iv", null)
            return if (cipherIVStr != null) Base64.decode(cipherIVStr, Base64.DEFAULT) else null
        }
    }

    init {
        try {
            mCryptoCore = if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP_MR1) {
                getCryptoreRSA(mContext)
            } else {
                cryptoreAES
            }
        } catch (e: Exception) {
            // no op
        }
    }
}
