package com.asda.android.service.base

import android.text.format.DateUtils
import android.util.Log
import com.asda.android.R
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.main.view.DynamicAppShortcuts
import com.asda.android.app.model.createCheckinNotification
import com.asda.android.app.orders.ods.getOrderService
import com.asda.android.app.orders.ods.mapOdsHistoryToViewOrder
import com.asda.android.lastmile.ASDALastMileCheckinManager
import com.asda.android.lastmile.interfaces.LastMileCheckinManager
import com.asda.android.orders.wismo.constants.WismoCardConstants.FULFILLMENT_TYPE_CNC
import com.asda.android.restapi.service.constants.ViewOrderConstants
import com.asda.android.restapi.service.data.OdsOrderModel
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.utils.Utils
import com.jakewharton.retrofit2.adapter.rxjava2.HttpException
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.observers.ResourceObserver
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers
import java.lang.ref.WeakReference
import java.util.*
import java.util.concurrent.CopyOnWriteArrayList

class OrderManager {
    private var mTotal = 0
    private var mPending = 0
    private val mOrders: MutableList<ViewOrderResponse.Order> = CopyOnWriteArrayList()
    private val mProcessedOrders: MutableList<ViewOrderResponse.Order> = CopyOnWriteArrayList()
    private val mCheckinEligibleOrders: MutableList<ViewOrderResponse.Order> = CopyOnWriteArrayList()
    private var mIsOrdersLoaded = false
    private var mLastLoadTime: Long = 0
    private var mIsLoading = false
    private val mCompositeDisposable = CompositeDisposable()

    interface Callback {
        fun onFinished(
            total: Int,
            pending: Int,
            orders: List<ViewOrderResponse.Order>?,
            processedOrders: List<ViewOrderResponse.Order>?
        )

        fun onFailed()
    }

    private val mCallbacks: MutableSet<WeakReference<Callback>> = HashSet()

    /**
     * * It will be stored as weak reference, so it should keep a strong reference to listener
     * outside.
     *
     * @param cb keep a strong reference to it or it will not be called.
     */
    fun fetchOrders(cb: Callback?) {
        if (!Authentication.getInstance().isLoggedIn && cb != null) {
            cb.onFinished(mTotal, mPending, mOrders, mProcessedOrders)
            return
        }
        if (cb != null) {
            mCallbacks.add(WeakReference(cb))
        }
        if (!mIsLoading) {
            var cacheTime = CACHE_TIME
            if (Utils.isDebug) {
                // Cache for 10 mins if its a debug build
                cacheTime *= 10
            }
            if (mIsOrdersLoaded && System.currentTimeMillis() - mLastLoadTime <= cacheTime) {
                returnData()
            } else {
                mIsLoading = true
                clearInternal()
                fetchOrder(PAGE_SIZE, PAGE_NUM)
            }
        }
    }

    private fun fetchOrder(pageSize: Int, num: Int) {
        val pair = Authentication.getInstance().identifierAndProfileId
        getOrderService()?.getOrders(pair?.first.orEmpty(), pair?.second.orEmpty(), pageSize, num - 1)
            ?.map { odsOrderModel: OdsOrderModel? ->
                odsOrderModel?.let {
                    mapOdsHistoryToViewOrder(it, num)
                }
            }?.subscribeOn(Schedulers.io())?.observeOn(AndroidSchedulers.mainThread())?.subscribe(fetchOrdersObservable)
    }

    private val fetchOrdersObservable: ResourceSingleObserver<ViewOrderResponse?>
        get() = object : ResourceSingleObserver<ViewOrderResponse?>() {
            override fun onSuccess(viewOrderResponse: ViewOrderResponse) {
                onFetchOrderSuccess(viewOrderResponse)
            }

            override fun onError(throwable: Throwable) {
                if (throwable is HttpException) {
                    if (throwable.code() == 404) {
                        val viewOrderResponse = ViewOrderResponse()
                        viewOrderResponse.setStatus("200")
                        viewOrderResponse.currentPage = 0
                        viewOrderResponse.maxPages = 0
                        viewOrderResponse.orders = arrayOfNulls(0)
                        onFetchOrderSuccess(viewOrderResponse)
                    }
                } else {
                    for (cb in mCallbacks) {
                        val callback = cb.get()
                        callback?.onFailed()
                    }
                    mCallbacks.clear()
                    mIsLoading = false
                }
            }
        }

    private fun onFetchOrderSuccess(data: ViewOrderResponse) {
        mLastLoadTime = System.currentTimeMillis()
        var anyCheckinEligibleOrderPresent = false
        if (data.orders != null) {
            for (order in data.orders!!) {
                if (!order.isLegacyCheckin && !anyCheckinEligibleOrderPresent) {
                    anyCheckinEligibleOrderPresent = true
                }
                if (isOrderPending(order)) {
                    mPending++
                }
                if (!order.isDeliveryPassPurchase) {
                    mTotal++
                    if (ViewOrderConstants.ORDER_STATUS_PLACED == order.orderStatus || ViewOrderConstants.ORDER_STATUS_PROCESSING == order.orderStatus || ViewOrderConstants.ORDER_STATUS_PICK_COMPLETE.equals(
                            order.orderStatus, ignoreCase = true
                        )
                    ) {
                        mOrders.add(order)
                    } else {
                        mProcessedOrders.add(order)
                    }
                }
            }
            DynamicAppShortcuts.setupShortcuts(AsdaApplication.getInstance(), data.orders)
        }
        if (!anyCheckinEligibleOrderPresent) {
            mIsOrdersLoaded = true
            mIsLoading = false
            returnData()
            return
        }
        val context: AsdaApplication = AsdaApplication.getInstance() ?: return
        var checkinManager: LastMileCheckinManager? = null
        checkinManager = ASDALastMileCheckinManager.getInstance(
            context,
            context.getString(R.string.checkin_sdk_asda_consumer_id),
            context.getString(R.string.checkin_sdk_asda_env),
            context.getString(R.string.checkin_sdk_cine_version_asda),
            context.getString(R.string.checkin_sdk_pegasus_version_asda),
            context.getString(R.string.checkin_sdk_maps_api_key),
            context.getString(R.string.checkin_sdk_asda_tenant_id),
            context.getString(R.string.checkin_sdk_asda_vertical_id),
            Authentication.getInstance(),
            AsdaActivity::class.java.name
        )
        val profileId = Authentication.getInstance().identifierAndProfileId?.second
        if (profileId.isNullOrEmpty().not() && checkinManager != null) {
            mCompositeDisposable.add(
                checkinManager
                    .silentCheckin(
                        orders,
                        profileId.orEmpty(),
                        createCheckinNotification(context)
                    )
                    .subscribe()
            )
            val asdaDisposable: Disposable = checkinManager
                .getEligibleOrders(profileId.orEmpty())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribeWith(
                    object : ResourceObserver<Any?>() {
                        override fun onNext(mappedPegasusOrder: Any) {
                            if (Utils.isDebug) {
                                Log.e(
                                    TAG,
                                    "Success on getting pegasus orders " +
                                        mappedPegasusOrder.toString()
                                )
                            }
                            if (mappedPegasusOrder is ViewOrderResponse.Order) {
                                val newOrder = getCheckinEleigibleOrderFromLegacyOrders(
                                    mappedPegasusOrder.orderId
                                )
                                if (newOrder != null) {
                                    newOrder.orderId = mappedPegasusOrder.orderId
                                    newOrder.isCheckinEligible = mappedPegasusOrder.isCheckinEligible
                                    if (mappedPegasusOrder.orderStatus == null) {
                                        newOrder.orderStatus = mappedPegasusOrder.checkinEligibleOrderDisplayState
                                    } else {
                                        newOrder.orderStatus = mappedPegasusOrder.orderStatus
                                    }
                                    newOrder.accessPointInfos = mappedPegasusOrder.accessPointInfos
                                    newOrder.reservation = mappedPegasusOrder.reservation
                                    mCheckinEligibleOrders.add(newOrder)
                                } else {
                                    if (Utils.isDebug) {
                                        Log.d(
                                            TAG,
                                            "Checkin Eligible Order does not exist in view order response!!"
                                        )
                                    }
                                }
                            }
                            returnData()
                        }

                        override fun onError(e: Throwable) {
                            mIsOrdersLoaded = true
                            mIsLoading = false
                            if (Utils.isDebug) {
                                Log.e(TAG, "error getting eligible orders " + e.message)
                            }
                            returnData()
                            mCompositeDisposable.clear()
                        }

                        override fun onComplete() {
                            mIsOrdersLoaded = true
                            mIsLoading = false
                            returnData()
                            mCompositeDisposable.clear()
                        }
                    })
            mCompositeDisposable.add(asdaDisposable)
        } else {
            Log.d(
                TAG,
                "profileId or customerId is null here so we will not be able to get the eligible orders without customerId, falling back to legacy Wismo card"
            )
            returnData()
        }
    }

    private fun returnData() {
        for (cb in mCallbacks) {
            val callback = cb.get()
            callback?.onFinished(mTotal, mPending, mOrders, mProcessedOrders)
        }
        mCallbacks.clear()
    }

    val orders: ArrayList<ViewOrderResponse.Order>
        get() {
            val list = ArrayList(mOrders)
            list.addAll(mProcessedOrders)
            return list
        }

    fun isLegacyCheckinOrder(orderId: String?): Boolean {
        if (orderId == null) {
            return true
        }
        val orders: List<ViewOrderResponse.Order> = mCheckinEligibleOrders
        for (order in orders) {
            if (orderId.equals(order.orderId, ignoreCase = true)) {
                if (Utils.isDebug) {
                    Log.e(TAG, "orderId=" + orderId + " order.orderId=" + order.orderId)
                }
                return false
            }
        }
        return true
    }

    fun isEligibleForCNC(orderId: String?): Boolean {
        for (order in mOrders) {
            if (order.orderId != null && order.orderId.equals(orderId, ignoreCase = true)) {
                return if (order.fulfillmentType == null) {
                    false
                } else {
                    order.fulfillmentType.equals(FULFILLMENT_TYPE_CNC, ignoreCase = true)
                }
            }
        }
        return false
    }

    fun reset() {
        clearInternal()
        if (mIsLoading) {
            mIsLoading = false
            fetchOrders(null)
        }
    }

    private fun getCheckinEleigibleOrderFromLegacyOrders(orderId: String?): ViewOrderResponse.Order? {
        for (legacyOrder in mOrders) {
            if (legacyOrder.orderId != null && legacyOrder.orderId.equals(orderId, ignoreCase = true)) {
                return legacyOrder
            }
        }
        return null
    }

    private fun clearInternal() {
        mTotal = 0
        mPending = 0
        mIsOrdersLoaded = false
        mOrders.clear()
        mProcessedOrders.clear()
        mLastLoadTime = 0
    }

    fun getViewOrder(orderId: String): ViewOrderResponse.Order? {
        for (order in orders) {
            if (orderId.equals(order.orderId, ignoreCase = true)) {
                return order
            }
        }
        return null
    }

    companion object {
        private const val CACHE_TIME = DateUtils.HOUR_IN_MILLIS
        private const val TAG = "OrderManager"
        private const val PAGE_SIZE = 20
        private const val PAGE_NUM = 1

        @JvmStatic
        fun get(): OrderManager {
            return AsdaApplication.getComponent().orderManager
        }

        private fun isOrderPending(order: ViewOrderResponse.Order): Boolean {
            var pending = false
            if (!order.isDeliveryPassPurchase &&
                (ViewOrderConstants.ORDER_STATUS_PLACED == order.orderStatus || ViewOrderConstants.ORDER_STATUS_PROCESSING == order.orderStatus)
            ) {
                pending = true
            }
            return pending
        }
    }
}
