package com.asda.android.utils

import android.content.Context
import android.content.SharedPreferences
import android.location.LocationManager
import android.os.Bundle
import android.os.Parcel
import android.text.Html
import android.text.TextUtils
import android.text.format.DateUtils
import android.util.Log
import android.view.View
import androidx.annotation.IntDef
import androidx.annotation.MainThread
import androidx.annotation.VisibleForTesting
import androidx.collection.ArrayMap
import com.asda.android.BuildConfig
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.ErrorEvent
import com.asda.android.analytics.info.FilterAnalytics
import com.asda.android.app.cncgeofence.CNCGeofenceConfigManager
import com.asda.android.app.cncgeofence.CNCGeofenceConfigManager.FeatureStatus
import com.asda.android.app.settings.featureflag.FeatureSettings.isThreedsEnabledAmex
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.utils.ISO8601TimeStampDecoder.decode
import com.asda.android.base.core.utils.orZero
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.cxo.CartManager.Companion.instance
import com.asda.android.favourites.features.shoppinglists.model.Item
import com.asda.android.products.ui.filters.model.AsdaFilter
import com.asda.android.restapi.app.cncgeofence.constants.CNCConstants.CNC_GEOFENCE_SETTINGS
import com.asda.android.restapi.app.cncgeofence.constants.CNCConstants.FEATURE_STATUS_KEY
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.AsdaServiceSettings.getServiceMode
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.amex
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.asda
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.delta
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.maestro
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.mastercard
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.unknown
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.visa
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.visaElectron
import com.asda.android.restapi.service.data.CreditCardsListResponse.CreditCardType
import com.asda.android.restapi.service.data.ListItemsResponse
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.restapi.service.data.WismoOrderResponse.SubItem
import com.asda.android.restapi.utils.InputStreamUtils.closeQuietly
import com.asda.android.restapi.utils.InputStreamUtils.toString
import com.asda.android.restapi.utils.SharedPreferencesUtil.getThreeDSV2EnabledFlag
import com.asda.android.restapi.utils.SimpleCalendar
import com.asda.android.service.base.Authentication
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.GoogleApiAvailability
import java.io.IOException
import java.lang.ref.WeakReference
import java.math.BigDecimal
import java.text.DateFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*
import java.util.regex.Pattern

object Utils {
    private const val TAG = "Utils"
    private const val TAG_REST_UTILS = "RestUtils"
    private var sFullDateFormat: SimpleDateFormat? = null
    private const val VISA = "Visa"
    private const val MASTER_CARD = "MasterCard"
    private const val MAESTRO = "Maestro"
    private const val AMEX = "AMEX"
    private const val VISA_ELECTRON = "Visa Electron"
    private const val ASDA_MONEY_CARD = "ASDA Money Credit"
    val isDevBuild: Boolean
        get() = BuildConfig.IS_DEV

    @JvmStatic
    fun isThreeDs2XEnabled(context: Context, cardType: String): Boolean {
        var isThreeDSEnabled = getThreeDSV2EnabledFlag(context, isDevBuild)
        if (!isThreeDSEnabled) {
            return false
        }
        if (AMEX == cardType) {
            isThreeDSEnabled = isThreedsEnabledAmex(context)
        }
        return isThreeDSEnabled
    }

    @JvmStatic
    fun isQAServer(context: Context?): Boolean {
        return (
            getServiceMode(context!!)
                != AsdaServiceSettingsConstants.SERVICE_MODE_PRODUCTION
            )
    }

    val isDebug: Boolean
        get() = BuildConfig.DEBUG

    @JvmStatic
    fun toString(charSequence: CharSequence?): String? {
        return charSequence?.toString()
    }

    @JvmStatic
    fun equals(s1: CharSequence?, s2: String?): Boolean {
        return (
            s1 == null && s2 == null ||
                TextUtils.isEmpty(s1) && TextUtils.isEmpty(s2) ||
                !(s1 == null || s2 == null) && s1.toString() == s2
            )
    }

    @VisibleForTesting
    fun getPreferences(context: Context): SharedPreferences {
        return context.getSharedPreferences(CNC_GEOFENCE_SETTINGS, Context.MODE_PRIVATE)
    }

    @VisibleForTesting
    fun getFeatureStatus(context: Context): FeatureStatus {
        return FeatureStatus.fromValue(
            getPreferences(context).getString(FEATURE_STATUS_KEY, CNCGeofenceConfigManager.DEFAULT_FEATURE_STATUS.toValue())
        )
    }

    @JvmStatic
    fun isGPlayAvailable(context: Context?): Boolean {
        return (
            GoogleApiAvailability.getInstance().isGooglePlayServicesAvailable(context)
                == ConnectionResult.SUCCESS
            )
    }

    @JvmStatic
    fun isFeatureEnabled(context: Context): Boolean {
        return (
            isGPlayAvailable(context) &&
                getFeatureStatus(context) != FeatureStatus.DISABLED
            )
    }

    @JvmStatic
    fun addPoundIfNecessary(value: String?): String? {
        var copiedValue = value
        if (copiedValue != null) {
            copiedValue = if (copiedValue.startsWith("£")) copiedValue else "£$copiedValue"
        }
        return copiedValue
    }

    @JvmStatic
    fun getCartQuantity(quantity: String?): BigDecimal {
        try {
            return BigDecimal(RestUtils.removeKg(quantity).toString())
        } catch (e: NumberFormatException) {
            Log.w(TAG, "unable to parse quantity", e)
        }
        return BigDecimal.ZERO
    }

    @JvmStatic
    fun compare(lhs: Int, rhs: Int): Int {
        return lhs.compareTo(rhs)
    }

    @VisibleForTesting
    fun formatTimeBetween(
        context: Context,
        startTs: Long,
        endTs: Long,
        timeFormat: SimpleDateFormat,
        dateTimeFormat: SimpleDateFormat,
        dateFormat: Int
    ): String {
        val calendarStart = Calendar.getInstance(Locale.UK)
        calendarStart.timeInMillis = startTs
        val calendarEnd = Calendar.getInstance(Locale.UK)
        calendarEnd.timeInMillis = endTs
        return if (calendarStart[Calendar.YEAR] == calendarEnd[Calendar.YEAR] && calendarStart[Calendar.MONTH] == calendarEnd[Calendar.MONTH] && calendarStart[Calendar.DAY_OF_MONTH] == calendarEnd[Calendar.DAY_OF_MONTH]) {
            context.getString(
                R.string.between_same_date,
                SimpleDateFormat.getDateInstance(dateFormat, Locale.UK).format(endTs),
                timeFormat.format(startTs),
                timeFormat.format(endTs)
            )
        } else {
            context.getString(
                R.string.between, dateTimeFormat.format(startTs), dateTimeFormat.format(endTs)
            )
        }
    }

    @JvmStatic
    val userEmail: String?
        get() = if (Authentication.getInstance().loginResponse != null) Authentication.getInstance().loginResponse!!.email else ""

    @JvmStatic
    fun convertToParseableString(poundString: String?): String? {
        var poundString = poundString
        if (poundString != null) {
            poundString = poundString.trim { it <= ' ' }.replace(",", "") // Remove "," from a string like "$1,255.81"
            if (poundString.length >= 1) {
                if (poundString.startsWith("£")) {
                    return poundString.substring(1)
                }
                if (poundString.endsWith("p")) { // like "50p"
                    return parsePound(poundString)
                }
                return if (poundString.startsWith("??")) {
                    poundString.substring(2) // sometimes they encode pound sign as "??"
                } else poundString
            }
        }
        return null
    }

    private fun parsePound(poundString: String): String {
        var poundString = poundString
        if (poundString.startsWith("0.")) {
            poundString = poundString.replace("0.", "0.0") // If it is decimal p, we need one more 0
        }
        val result = poundString.substring(0, poundString.length - 1)
        return if (poundString.startsWith("0.")) result else "0.$result"
    }

    @JvmStatic
    fun poundStringToDouble(poundString: String?): Double {
        var cost = 0.0
        val parsableString = convertToParseableString(poundString)
        if (parsableString != null) {
            try {
                cost = parsableString.toDouble()
            } catch (e: NumberFormatException) {
                Log.e(TAG_REST_UTILS, "Error converting: $poundString", e)
            }
        }
        return cost
    }

    // position/index starts with 0 in Java, this method translate 0 to 1, 1 to 2, etc
    @JvmStatic
    fun addPositionByOne(position: String?): String {
        var updatedPosition = "0"
        try {
            updatedPosition = (Integer.valueOf(position!!) + 1).toString()
        } catch (e: Exception) {
            Log.e(TAG, "addPositionByOne() exception... ", e)
        }
        return updatedPosition
    }

    @JvmStatic
    fun isZero(poundString: String?): Boolean {
        return isZero(RestUtils.poundStringToBigDecimal(poundString))
    }

    @JvmStatic
    fun isZero(bigDecimal: BigDecimal): Boolean {
        return bigDecimal.compareTo(BigDecimal.ZERO) == 0
    }

    @JvmStatic
    fun convertTimeFormat(
        inputDateFormat: String?,
        outputDateFormat: String?,
        dateTime: String?
    ): String? {
        var outputTimeStr: String?
        val inputFormatter: DateFormat = SimpleDateFormat(inputDateFormat, Locale.UK)
        val outputFormatter: DateFormat = SimpleDateFormat(outputDateFormat, Locale.UK)
        val date: Date
        try {
            date = inputFormatter.parse(dateTime)
            outputTimeStr = outputFormatter.format(date)
        } catch (ignore: ParseException) {
            outputTimeStr = dateTime
            Log.e("Utils.java error", "exception occured at convertTimeFormat")
        }
        return outputTimeStr
    }

    @JvmStatic
    fun convertUTCtoUKDateTimeFormat(
        inputDateFormat: String?,
        outputDateFormat: String?,
        dateTime: String?
    ): String? {
        var outputTimeStr: String?
        val inputFormatter = SimpleDateFormat(inputDateFormat)
        val outputFormatter = SimpleDateFormat(outputDateFormat, Locale.UK)
        val date: Date
        try {
            inputFormatter.timeZone = TimeZone.getTimeZone(TimeZoneFormat.UTC)
            outputFormatter.timeZone = TimeZone.getTimeZone(TimeZoneFormat.UK)
            date = inputFormatter.parse(dateTime)
            outputTimeStr = outputFormatter.format(date)
        } catch (ignore: ParseException) {
            outputTimeStr = dateTime
            Log.e("Utils.java error", "exception occured at convertUTCtoUKDateTimeFormat")
        }
        return outputTimeStr
    }

    @JvmStatic
    @VisibleForTesting
    fun getDayOfMonthSuffix(n: Int): String {
        return if (n >= 11 && n <= 13) {
            "th"
        } else when (n % 10) {
            1 -> "st"
            2 -> "nd"
            3 -> "rd"
            else -> "th"
        }
    }

    @JvmStatic
    fun removeRoundBrackets(text: CharSequence?): String {
        var mainTitle = text?.toString() ?: ""
        if (mainTitle.endsWith(")")) {
            val countStartIndex = mainTitle.lastIndexOf('(')
            if (countStartIndex > 0) {
                mainTitle = mainTitle.substring(0, countStartIndex - 1)
            }
        }
        return mainTitle
    }

    @JvmStatic
    fun removeHtmlTagsAndLineBreaks(str: String?): String {
        if (!str.isNullOrEmpty()) {
            val plainText = RestUtils.extractPlainText(str)
            if (!TextUtils.isEmpty(plainText)) {
                return Html.fromHtml(plainText).toString().replace("\n".toRegex(), " ")
            }
        }
        return ""
    }

    @JvmStatic
    fun getTotalItemsCount(shelfListItems: List<ShelfListItem>): Int {
        var count = 0
        for (item in shelfListItems) {
            if (item.qty != null) {
                if (item.qty!!.matches(Regex("[0-9]*"))) {
                    count += item.qty!!.toInt()
                } else {
                    count++
                }
            }
        }
        return count
    }

    @JvmStatic
    fun getTotalSubItemsCount(
        shelfListItems: Array<WismoOrderResponse.SubstitutedItem?>
    ): Int {
        var count = 0
        for (item in shelfListItems) {
            if (item != null && item.sub != null) {
                count = count + getTotalSubItemsCount(item.sub)
            }
        }
        return count
    }

    private fun getTotalSubItemsCount(items: Array<SubItem>?): Int {
        var count = 0
        for (subItem in items!!) {
            if (subItem.subquantity != null) {
                if (subItem.subquantity!!.matches(Regex("[0-9]*"))) {
                    count += subItem.subquantity!!.toInt()
                } else {
                    count++
                }
            }
        }
        return count
    }

    @JvmStatic
    fun setVisibility(visibility: Int, container: View?, vararg viewIds: Int) {
        if (container != null) {
            for (viewId in viewIds) {
                val view = container.findViewById<View>(viewId)
                if (view != null) {
                    view.visibility = visibility
                }
            }
        }
    }

    @JvmStatic
    fun isValidPassword(password: CharSequence?): Boolean {
        return password?.length.orZero() >= 8
    }

    @JvmStatic
    @Throws(ParseException::class)
    fun parseCreditCardDate(expiryDate: String?): SimpleCalendar {
        // If the expirydate is like 4/10 or 13/4 or 4/5 will be taken care
        var expiryDate = expiryDate
        if (expiryDate != null && expiryDate.length < 5 && expiryDate.contains("/")) {
            val months = expiryDate.split("/".toRegex()).toTypedArray()
            if (months[0].length < 2) {
                expiryDate = "0$expiryDate"
            }
            if (months[1].length < 2) {
                expiryDate = expiryDate.split("/".toRegex()).toTypedArray()[0] + "/0" + months[1]
            }
        }
        if (expiryDate == null || expiryDate.length != 5 || Character.isDigit(expiryDate[2])) {
            throw ParseException("Unexpected credit card expiry format: '$expiryDate'", 0)
        }
        val year = expiryDate.substring(3, 5).toInt()
        val month = expiryDate.substring(0, 2).toInt()
        if (year < 0 || month <= 0 || month > 12) {
            throw ParseException("Unexpected credit card expiry value: '$expiryDate'", 0)
        }
        return SimpleCalendar(2000 + year, month, null, null, null, null, null, null, null)
    }

    @JvmStatic
    @Throws(ParseException::class)
    fun parseNewCreditCardDate(expiryDate: String?): SimpleCalendar {
        // If the expirydate is like 04/2019 will be taken care
        var expiryDate = expiryDate
        if (expiryDate != null && expiryDate.length < 5 && expiryDate.contains("/")) {
            val months = expiryDate.split("/".toRegex()).toTypedArray()
            if (months[0].length < 2) {
                expiryDate = "0$expiryDate"
            }
        }
        if (expiryDate == null || expiryDate.length != 5 || Character.isDigit(expiryDate[2])) {
            throw ParseException("Unexpected credit card expiry format: '$expiryDate'", 0)
        }
        val year = expiryDate.substring(3, 5).toInt()
        val month = expiryDate.substring(0, 2).toInt()
        if (year < 0 || month <= 0 || month > 12) {
            throw ParseException("Unexpected credit card expiry value: '$expiryDate'", 0)
        }
        return SimpleCalendar(2000 + year, month, null, null, null, null, null, null, null)
    }

    /**
     * http://en.wikipedia.org/wiki/Luhn_algorithm
     *
     * @param cardNumber card number to check. May have spaces.
     * @return true if the card number looks valid
     */
    @JvmStatic
    fun isValidCardNumber(cardNumber: String?): Boolean {
        var cardNumber = cardNumber
        if (cardNumber != null) {
            cardNumber = cardNumber.replace(" ", "")
            if (cardNumber.length >= 13) {
                val cc = IntArray(cardNumber.length)
                for (i in 0 until cardNumber.length) {
                    try {
                        cc[i] = Character.digit(cardNumber[i], 10)
                    } catch (nfe: NumberFormatException) {
                        return false
                    }
                }
                return isValidCardNumber(cc)
            }
        }
        return false
    }

    private fun isValidCardNumber(cardNumber: IntArray): Boolean {
        var sum = 0
        val length = cardNumber.size
        for (j in 0 until length) {
            // get digits in reverse order
            var digit = cardNumber[length - j - 1]

            // every 2nd number multiply with 2
            if (j % 2 == 1) {
                digit *= 2
            }
            sum += if (digit > 9) digit - 9 else digit
        }
        return sum % 10 == 0
    }

    /**
     * Additional util method to validate credit card security code, this method does not performs
     * card issuer specific checks which are not performed by the [.validateCsc]
     *
     * @param csc
     * @param cardType
     * @return true if csc code length matches to that of card issuer otherwise false
     */
    @JvmStatic
    fun validateCsc(csc: String?, cardType: String?): Boolean {
        var isValid = false
        val length = csc?.length ?: 0
        if (cardType != null) {
            isValid = when (cardType.toLowerCase()) {
                "amex" -> length == 4
                "visa", "mastercard", "asda money credit", "maestro" -> length == 3
                else -> length == 3
            }
        }
        return isValid
    }

    /**
     * Helper method to get CreditCardsListResponse.CreditCardType from api response cardType string
     *
     * @param cardType
     * @return CreditCardsListResponse.CreditCardType
     */
    @JvmStatic
    fun getCardIssuer(cardType: String?): CreditCardType {
        return if (cardType == null) {
            unknown
        } else when (cardType) {
            VISA -> visa
            MASTER_CARD -> mastercard
            MAESTRO -> maestro
            VISA_ELECTRON -> visaElectron
            AMEX -> amex
            ASDA_MONEY_CARD -> asda
            else -> unknown
        }
    }

    @JvmStatic
    fun getCardType(cardNumber: String?): CreditCardType {
        /*
     * References
     * ==========
     * https://en.wikipedia.org/wiki/Bank_card_number
     * http://en.wikipedia.org/wiki/List_of_Issuer_Identification_Numbers
     *
     * The Bank Identification Number (BIN) are the first six digits of a
     * card number and they allow to know which institution (Visa, etc) and
     * bank have issued the card
     *
     * Regex matching ref: https://confluence.walmart.com/pages/viewpage.action?spaceKey=ASDA&title=Card+Type+Validation
     */
        var pattern: Pattern? = null
        if (cardNumber == null) {
            return unknown
        }

        // More restrictive rules shall be placed first
        val asdaPrefixes = arrayOf("529932", "541647", "557712", "529507")
        pattern = Pattern.compile("(^52993258)")
        var matcher = pattern.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *asdaPrefixes) || matcher.find()) {
            return asda
        }
        val visaElectronPrefixes = arrayOf(
            "417500", "4026", "4405", "4508", "4844", "4913", "4917"
        )
        if (matchesAnyPrefix(cardNumber, *visaElectronPrefixes)) {
            return visaElectron
        }
        val maestroPrefixes = arrayOf(
            "503396", "633300", "5018", "5020", "5038", "5893", "6304", "67"
        )
        pattern = Pattern.compile("^(?:5[0678]\\d\\d|6304|6390|67\\d\\d)\\d{8,15}$")
        val p1 = Pattern.compile("^(5[06-8]|6\\d)\\d{14}(\\d{2,3})?$")
        matcher = pattern.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *maestroPrefixes) ||
            matcher.find() ||
            p1.matcher(cardNumber).find()
        ) {
            return maestro
        }
        val deltaPrefixes = arrayOf("453978", "446200")
        if (matchesAnyPrefix(cardNumber, *deltaPrefixes)) {
            return delta
        }
        val masterCardPrefixes = arrayOf("51", "52", "53", "54", "55")
        pattern = Pattern.compile("(^(5[0-5]|2[2-6]|27[0-1]|2720))")
        matcher = pattern.matcher(cardNumber)
        if (matchesAnyPrefix(cardNumber, *masterCardPrefixes) || matcher.find()) {
            return mastercard
        }
        val amexPrefixes = arrayOf("34", "37")
        if (matchesAnyPrefix(cardNumber, *amexPrefixes)) {
            return amex
        }
        val visaPrefixes = arrayOf("4")
        return if (matchesAnyPrefix(cardNumber, *visaPrefixes)) {
            visa
        } else unknown
    }

    fun matchesAnyPrefix(text: String, vararg prefixes: String?): Boolean {
        for (prefix in prefixes) {
            if (text.startsWith(prefix!!)) {
                return true
            }
        }
        return false
    }

    @JvmStatic
    fun formatFromIsoTime(
        outputTemplate: String,
        isoTemplate: String,
        isoDate: String?
    ): String? {
        val inputDateFormatter: DateFormat = SimpleDateFormat(isoTemplate, Locale.UK)
        val outputFormatter: DateFormat = SimpleDateFormat(outputTemplate, Locale.UK)
        try {
            val date = inputDateFormatter.parse(isoDate)
            return outputFormatter.format(date)
        } catch (e: Exception) {
            Log.e(TAG_REST_UTILS, "Unable to parse date", e)
        }
        return isoDate
    }

    @JvmStatic
    fun parseIso8610Timestamp(ts: String?): Long {
        var result: Long = 0
        if (ts.isNullOrEmpty().not()) {
            try {
                val simpleCalendar = decode(ts)
                if (simpleCalendar != null) {
                    result = simpleCalendar.time
                }
            } catch (e: Exception) {
                Log.e("parseIso8610Timestamp", "bad timestamp: '$ts'")
            }
        }
        return result
    }

    @JvmStatic
    val currentTimeInUK: Calendar
        get() = Calendar.getInstance(UK.TIME_ZONE)

    @JvmStatic
    fun isToday(dateString: String, pattern: String): Boolean {
        try {
            val sdf = SimpleDateFormat(pattern, Locale.UK)
            sdf.timeZone = TimeZone.getTimeZone(UK.UK_TIME_ZONE)
            val date = sdf.parse(dateString)
            return DateUtils.isToday(date.time)
        } catch (e: ParseException) {
            Log.e("RestUtils.isToday", Arrays.toString(e.stackTrace))
        }
        return false
    }

    @JvmStatic
    fun isWithinRange(
        dateString: String,
        dateFormat: String,
        rangeInDays: Int
    ): Boolean {
        try {
            val sdf = SimpleDateFormat(dateFormat, Locale.UK)
            var date = sdf.parse(dateString)
            val calendar = currentTimeInUK
            calendar.time = date
            calendar.add(Calendar.DATE, rangeInDays)
            date = calendar.time
            val today = Date()
            return today.before(date)
        } catch (e: ParseException) {
            Log.e("RestUtils.isWithinRange", Arrays.toString(e.stackTrace))
        }
        return false
    }

    /** Transform ISO 8601 string to Time in Milliseconds  */
    @JvmStatic
    fun getTimeInMillisFromIso(iso8601string: String?): Long {
        if (iso8601string.isNullOrEmpty().not()) {
            try {
                val simpleCalendar = decode(iso8601string) ?: return 0
                return simpleCalendar.time
            } catch (e: Exception) {
                Log.w(TAG_REST_UTILS, "Error parsing iso time = $iso8601string", e)
            }
        }
        return 0
    }

    private var sLookupTable: ArrayMap<String, ArrayList<Int>>? = null

    @JvmField
    @VisibleForTesting
    val sDepartmentNameLengthList = ArrayList<Int>() // how many words does department have
    private val sDepartmentOrderCache = ArrayMap<String, Int>()

    @JvmStatic
    @Synchronized
    fun getDepartmentOrder(context: Context, departmentName: String?): Int {
        if (sLookupTable == null) {
            val inputStream = context.resources.openRawResource(R.raw.department_order)
            try {
                val data = toString(inputStream)
                sLookupTable = ArrayMap()
                var start = 0
                var order = 0
                while (true) {
                    val end = data.indexOf('\n', start)
                    start = if (end > -1) {
                        val rawName = data.substring(start, if (data[end - 1] == '\r') end - 1 else end)
                        updateDepartmentNameList(rawName, order)
                        order++
                        end + 1
                    } else {
                        break
                    }
                }
            } catch (e: IOException) {
                Log.e("getDepartmentOrder", "parse failed", e)
            } finally {
                closeQuietly(inputStream)
            }
        }
        if (departmentName.isNullOrEmpty()) {
            return Int.MAX_VALUE
        }
        val cachedOrder = sDepartmentOrderCache[departmentName]
        if (cachedOrder != null) {
            return cachedOrder
        }
        val order = getBestOrder(departmentName)
        sDepartmentOrderCache[departmentName] = order
        return order
    }

    private fun updateDepartmentNameList(rawName: String, order: Int) {
        val filteredTokens = normalizeDeptName(rawName)
        for (token in filteredTokens) {
            var orders = sLookupTable!![token]
            if (orders == null) {
                orders = ArrayList()
                sLookupTable!![token] = orders
            }
            if (!orders.contains(order)) {
                orders.add(order)
            }
        }
        sDepartmentNameLengthList.add(Math.max(1, filteredTokens.size))
    }

    @JvmStatic
    @VisibleForTesting
    fun getBestOrder(departmentName: String): Int {
        val queryTokens = normalizeDeptName(departmentName)
        val scoresByOrder = IntArray(sLookupTable!!.size)
        Arrays.fill(scoresByOrder, -1)
        for (i in queryTokens.indices) {
            val queryToken = queryTokens[i]
            val orders = sLookupTable!![queryToken]
            if (orders != null) {
                val size = orders.size
                if (size == 1) {
                    return orders[0]
                }
                val score = 100 / size - i
                for (order in orders) {
                    scoresByOrder[order] += score + 10 / sDepartmentNameLengthList[order]
                }
            }
        }
        var maxScore = Int.MIN_VALUE
        var bestOrder = Int.MAX_VALUE
        for (i in scoresByOrder.indices) {
            if (scoresByOrder[i] > maxScore && scoresByOrder[i] >= 0) {
                maxScore = scoresByOrder[i]
                bestOrder = i
            }
        }
        if (bestOrder == Int.MAX_VALUE) {
            bestOrder -= departmentName.hashCode() and 0xFFFFFF
        }
        return bestOrder
    }

    @JvmStatic
    @VisibleForTesting
    fun normalizeDeptName(departmentName: String): ArrayList<String> {
        val filteredTokens = ArrayList<String>()
        if (!TextUtils.isEmpty(departmentName)) {
            val tokens = departmentName.toLowerCase(Locale.ENGLISH).replace(',', ' ').split(" ".toRegex()).toTypedArray()
            for (token in tokens) {
                if (token.length > 0 &&
                    !(token == "&" || token == "and" || token == "the")
                ) {
                    filteredTokens.add(token)
                }
            }
        }
        return filteredTokens
    }

    @JvmStatic
    fun isCurrentTimeIsBeyondSlotCutOffTime(order: WismoOrderResponse.Payload): Boolean {
        val sdf = SimpleDateFormat(
            DateFormats.EEEE_DD_MMMM_YYYY,
            Locale.UK
        )
        sdf.timeZone = TimeZone.getTimeZone(TimeZoneFormat.UK)
        try {
            val slotCutOffCalendar = Calendar.getInstance(TimeZone.getTimeZone(TimeZoneFormat.UK))
            slotCutOffCalendar.time = sdf.parse(order.dateOfDelivery)
            order.slotEndTime?.let {
                slotCutOffCalendar.set(Calendar.HOUR_OF_DAY, Integer.valueOf(it.substring(0, 2)))
                slotCutOffCalendar.set(Calendar.MINUTE, Integer.valueOf(it.substring(3)))
                slotCutOffCalendar.add(Calendar.HOUR_OF_DAY, 1)
                slotCutOffCalendar.add(Calendar.MINUTE, 30)
                if (currentTimeInUK.timeInMillis > slotCutOffCalendar.timeInMillis) {
                    return true
                }
            }
        } catch (e: ParseException) {
            Log.w(TAG, e)
        }
        return false
    }

    private val mAllReferences = ArrayMap<String, ArrayList<WeakReference<Any>>>()

    @JvmField
    @VisibleForTesting
    val sIds = HashMap<String, Long>() // HashMap is better here because of collection size
    private var sId = System.currentTimeMillis()
    @JvmStatic
    fun getId(strId: String?): Long {
        if (strId == null) {
            return 0
        }
        val id = sIds[strId]
        return if (id == null) {
            if (sIds.size > 5000) { // just in case, it's less than 500kb ram
                resetIds()
            }
            val newId = ++sId
            sIds[strId] = newId
            newId
        } else {
            id
        }
    }

    @JvmStatic
    fun resetIds() {
        Log.w(TAG_REST_UTILS, "clean ids cache!")
        sIds.clear()
    }

    @JvmStatic
    fun formatDeliveryTimeSpacing(timeString: String?): String {
        return timeString?.replace("00", "00 ") ?: ""
    }

    // eg. 2021-01-05T00:49:58.000+0000
    @JvmStatic
    @get:MainThread
    val fullDateFormatter: SimpleDateFormat
        get() {
            if (sFullDateFormat == null) {
                sFullDateFormat = SimpleDateFormat(
                    "yyyy-MM-dd'T'HH:mm:ss.SSSZ", Locale.UK
                ) // eg. 2021-01-05T00:49:58.000+0000
                sFullDateFormat!!.timeZone = UK.TIME_ZONE
            }
            return sFullDateFormat!!
        }

    @JvmStatic
    fun removeTrailingZeros(value: String?): String? {
        var value = value
        if (value != null) {
            if (value.endsWith(".00")) {
                value = value.substring(0, value.length - 3)
            } else if (value.endsWith(".0")) {
                value = value.substring(0, value.length - 2)
            }
        }
        return value
    }

    // Util method to check if Location is enabled, not if location permission is granted, those are
    // two different things
    @JvmStatic
    fun isLocationEnabled(context: Context): Boolean {
        val locationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
        return (
            locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) ||
                locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
            )
    }

    @JvmStatic
    fun formatBannerIds(bannerIds: List<String?>): String? {
        val sb = StringBuilder()
        if (bannerIds.isNotEmpty()) {
            if (bannerIds.size == 1) {
                // no comma for first one
                sb.append(bannerIds[0])
            } else {
                for (i in 0 until bannerIds.size - 1) {
                    // need comma
                    sb.append(bannerIds[i]).append(",")
                }
                // no comma for last one
                sb.append(bannerIds[bannerIds.size - 1])
            }
            return sb.toString()
        }
        return null
    }

    @JvmStatic
    @SafeVarargs
    fun formatAnalyticsFilters(vararg selectedFilters: ArrayList<AsdaFilter>?): Array<FilterAnalytics> {
        val selectedFilterAnalytics = ArrayList<FilterAnalytics>()
        for (selectedFilterArray in selectedFilters) {
            if (selectedFilterArray != null) {
                for (item in selectedFilterArray) {
                    if (!TextUtils.isEmpty(item.category) && !TextUtils.isEmpty(item.name)) {
                        selectedFilterAnalytics.add(FilterAnalytics(item.category!!, item.name!!))
                    }
                }
            }
        }
        return selectedFilterAnalytics.toTypedArray()
    }

    @JvmStatic
    fun convertToListItemResponse(item: Item): ListItemsResponse.Item {
        val listItem = ListItemsResponse.Item()
        listItem.id = item.id
        listItem.name = item.name
        listItem.imageURL = item.imageURL
        listItem.extraLargeImageURL = item.extraLargeImageURL
        listItem.quantity = item.quantity
        listItem.pricePerWt = item.pricePerWt
        listItem.pricePerUOM = item.pricePerUOM
        listItem.deptName = item.deptName
        listItem.maxQty = item.maxQty
        listItem.avgWeight = item.avgWeight
        listItem.price = item.price
        listItem.promoId = item.promoId
        listItem.promoType = item.promoType
        listItem.promoDetailFull = item.promoDetailFull
        listItem.weight = item.weight
        listItem.meatStickerDetails = item.meatStickerDetails
        listItem.availability = item.availability
        listItem.futureItemText = item.futureItemText
        listItem.deptId = item.deptId
        listItem.shelfId = item.shelfId
        listItem.cin = item.cin
        try {
            listItem.bundledItemCount = item.bundledItemCount!!.toInt()
        } catch (ex: NumberFormatException) {
            Log.w(TAG, ex)
        }
        listItem.bundleDiscount = item.bundleDiscount
        listItem.promoDetail = item.promoDetail
        listItem.wasPrice = item.wasPrice
        listItem.productAttribute = item.productAttribute
        listItem.isChecked = true
        return listItem
    }

    // Refer  https://developer.android.com/guide/components/activities/parcelables-and-bundles
    @JvmStatic
    fun informAboutTooLargePayload(payloadToBeSaved: Bundle?, tag: String) {
        val parcel = Parcel.obtain()
        parcel.writeValue(payloadToBeSaved)
        val bytes = parcel.marshall()
        parcel.recycle()
        Log.w(tag, "The size of bundle " + bytes.size)
        if (bytes.size > 1024) { //
            Tracker.get()
                .trackEvent(
                    ErrorEvent(
                        Anivia.TOO_LARGE_PAYLOAD,
                        tag + " Potential issue for API data communication " + bytes.size + " check."
                    )
                )
        }
    }

    fun buildBasketXml(): String? {
        return instance.getBasketXml()
    }

    fun getCost(item: ShelfListItem?): String {
        return if (item != null) ItemQuantityController.calcCost(
            item.pricePerUnit, item.avgWeight, item.pricePerWt, item.qty
        ) else ""
    }

    object UK {
        @JvmField
        var TIME_ZONE: TimeZone? = null
        const val UK_TIME_ZONE = "Europe/London"

        init {
            var tz = TimeZone.getTimeZone(UK_TIME_ZONE)
            if (UK_TIME_ZONE != tz.id) {
                tz = TimeZone.getDefault() // TZ not found, use current (hopefully it would be UK's)
            }
            TIME_ZONE = tz
        }
    }

    @IntDef(View.VISIBLE, View.GONE, View.INVISIBLE)
    annotation class Visibility
}
