package com.asda.android.utils.view

import android.content.Context
import android.os.Build
import android.text.TextUtils
import android.util.DisplayMetrics
import android.util.Log
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.ImageButton
import android.widget.TextView
import androidx.annotation.StringRes
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.SimpleItemAnimator
import com.asda.android.R
import com.asda.android.app.main.AsdaApplication
import com.asda.android.restapi.service.data.cms.ProductList
import java.math.BigDecimal

object ViewUtil {
    private const val TAG = "ViewUtil"
    @JvmStatic
    fun getText(id: Int, parent: View): String {
        val textView = parent.findViewById<View>(id) as TextView
        return textView.text.toString()
    }

    @JvmStatic
    fun setText(id: Int, parent: View?, text: CharSequence?) {
        if (parent != null) {
            val textView = parent.findViewById<View>(id) as TextView
            textView.text = text ?: ""
        }
    }

    @JvmStatic
    fun setVisibility(id: Int, parent: View?, show: Boolean) {
        if (parent != null) {
            val view = parent.findViewById<View>(id)
            if (view != null) {
                view.visibility = if (show) View.VISIBLE else View.GONE
            }
        }
    }

    @JvmStatic
    fun setTextHideIfEmpty(
        id: Int,
        parent: View?,
        text: CharSequence?
    ) {
        if (parent != null) {
            val textView = parent.findViewById<View>(id) as TextView
            if (TextUtils.isEmpty(text)) {
                textView.visibility = View.GONE
            } else {
                textView.visibility = View.VISIBLE
                textView.text = text
            }
        }
    }

    @JvmStatic
    fun setText(id: Int, parent: View, textId: Int) {
        val textView = parent.findViewById<View>(id) as TextView
        textView.setText(textId)
    }

    @JvmStatic
    fun setValueWithCurrency(
        id: Int,
        parent: View?,
        text: CharSequence?
    ) {
        setValueWithCurrency(id, parent, text, R.string.shoppinglist_checked_total)
    }

    @JvmStatic
    fun setValueWithCurrency(
        id: Int,
        parent: View?,
        text: CharSequence?,
        @StringRes resourceStringId: Int
    ) {
        if (parent != null) {
            val textView = parent.findViewById<View>(id) as TextView
            if (text == null) {
                textView.text = ""
            } else {
                val value = text.toString()
                if (value.contains("£")) {
                    textView.text = value
                } else if (!TextUtils.isEmpty(value)) {
                    try {
                        textView.text = parent
                            .resources
                            .getString(
                                resourceStringId,
                                BigDecimal(value)
                                    .setScale(2, BigDecimal.ROUND_HALF_UP)
                                    .toPlainString()
                            )
                    } catch (e: NumberFormatException) {
                        textView.text = value
                    }
                }
            }
        }
    }

    @JvmStatic
    fun spToPixels(sp: Float, context: Context): Int {
        return TypedValue.applyDimension(
            TypedValue.COMPLEX_UNIT_SP, sp, context.resources.displayMetrics
        ).toInt()
    }

    @JvmStatic
    fun dpToPixels(dp: Int, displayMetrics: DisplayMetrics): Int {
        return (dp * (displayMetrics.densityDpi / 160.0f)).toInt()
    }

    @JvmStatic
    fun dpToPixels(dp: Int, context: Context?): Int {
        if (context == null) {
            return (dp * (DisplayMetrics.DENSITY_XHIGH / 160.0f)).toInt()
        }
        val displayMetrics = getDisplayMetrics(context)
        return (dp * (displayMetrics.densityDpi / 160.0f)).toInt()
    }

    fun getDisplayMetrics(context: Context): DisplayMetrics {
        return context.resources.displayMetrics
    }

    @JvmStatic
    fun getScreenWidth(context: Context): Int {
        return context.resources.displayMetrics.widthPixels
    }

    fun getColumnCount(context: Context): Int {
        return getScreenWidth(context) / dpToPixels(ProductList.PREFERRED_CARD_WIDTH_DP, context)
    }

    @JvmStatic
    fun getProductTileWidth(context: Context): Int {
        return getScreenWidth(context) / getColumnCount(context)
    }

    @JvmStatic
    fun getDpiString(metrics: DisplayMetrics): String {
        return when (metrics.densityDpi) {
            DisplayMetrics.DENSITY_LOW -> "ldpi"
            DisplayMetrics.DENSITY_MEDIUM -> "mdpi"
            DisplayMetrics.DENSITY_TV -> "tvhdpi"
            DisplayMetrics.DENSITY_HIGH -> "hdpi"
            DisplayMetrics.DENSITY_XHIGH -> "xhdpi"
            DisplayMetrics.DENSITY_260 -> "260dpi (between hdpi and xhdpi)"
            DisplayMetrics.DENSITY_280 -> "280dpi (between hdpi and xhdpi)"
            DisplayMetrics.DENSITY_300 -> "300dpi (between hdpi and xhdpi)"
            DisplayMetrics.DENSITY_340 -> "340dpi (between xhdpi and xxhdpi)"
            DisplayMetrics.DENSITY_360 -> "360dpi (between xhdpi and xxhdpi)"
            DisplayMetrics.DENSITY_400 -> "400dpi (between xhdpi and xxhdpi)"
            DisplayMetrics.DENSITY_420 -> "420dpi (between xhdpi and xxhdpi)"
            DisplayMetrics.DENSITY_XXHIGH -> "xxhdpi"
            DisplayMetrics.DENSITY_560 -> "560dpi (between xxhdpi and xxxhdpi)"
            DisplayMetrics.DENSITY_XXXHIGH -> "xxxhdpi"
            else -> "?"
        }
    }

    @JvmStatic
    fun <T : View?> findViewById(view: View, id: Int): T {
        return view.findViewById<View>(id) as T
    }

    @JvmStatic
    fun <T : View?> inflate(context: Context?, resId: Int): T {
        return LayoutInflater.from(context).inflate(resId, null) as T
    }

    @JvmStatic
    fun <T : View?> inflate(context: Context?, resId: Int, parent: ViewGroup?): T {
        return inflate<T>(context, resId, parent, false)
    }

    @JvmStatic
    fun <T : View?> inflate(context: Context?, resId: Int, parent: ViewGroup?, attachToRoot: Boolean): T {
        return LayoutInflater.from(context).inflate(resId, parent, attachToRoot) as T
    }

    @JvmStatic
    fun hideKeyboard(view: View) {
        val imm = view.context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(view.windowToken, 0)
    }

    @JvmStatic
    fun showKeyboard(view: View) {
        view.requestFocus()
        val imm = view.context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.showSoftInput(view, 0)
    }

    @JvmStatic
    fun getActionBarHeight(context: Context?): Int {
        return if (context == null) {
            Log.e(TAG, "null context passed to getActionBarHeight")
            56 * 3 // should not happen
        } else {
            val typedValue = TypedValue()
            val displayMetrics = context.resources.displayMetrics
            if (context.theme.resolveAttribute(android.R.attr.actionBarSize, typedValue, true)) {
                TypedValue.complexToDimensionPixelSize(typedValue.data, displayMetrics)
            } else {
                Log.e(TAG, "actionBarSize not found")
                dpToPixels(56, displayMetrics) // should not happen as well
            }
        }
    }

    /**
     * ex: to disabe blinking when toggle fav status. It's better to introduce custom animator instead
     * though.
     */
    @JvmStatic
    fun disableChangeAnimations(recyclerView: RecyclerView) {
        val animator = recyclerView.itemAnimator
        if (animator is SimpleItemAnimator) {
            animator.supportsChangeAnimations = false
        }
    }

    @JvmStatic
    fun getToolbarNavigationButton(toolBar: androidx.appcompat.widget.Toolbar?): ImageButton? {
        try {
            toolBar?.let {
                val size: Int = it.childCount
                for (i in 0 until size) {
                    val child: View = it.getChildAt(i)
                    if (child is ImageButton) {
                        val btn = child
                        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                            btn.drawable === it.navigationIcon
                            btn
                        } else {
                            btn
                        }
                    }
                }
            }
        } catch (ex: Exception) {
            return null
        }
        return null
    }

    fun pickImageUrl(extraLargeImageURL: String?, imageURL: String?): String? {
        // image placeholder size is 93dp, imageURL are 93 pixels, extraLargeImageURL are 280 pixels,
        // check which image is better to use:
        return if (imageURL.isNullOrEmpty() ||
            (
                !extraLargeImageURL.isNullOrEmpty() &&
                    dpToPixels(93, AsdaApplication.getInstance()) >= (93 + 280) / 2
                )
        ) extraLargeImageURL else imageURL
    }
}
