package com.asda.android.utils.view

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Rect
import android.os.Build
import android.text.Html
import android.text.Spannable
import android.text.Spanned
import android.text.TextUtils
import android.util.Log
import android.view.*
import android.widget.ListView
import android.widget.ScrollView
import android.widget.TextView
import androidx.core.app.ActivityOptionsCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.analytics.JsonMapper
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.exception.CaughtExceptionEvent
import com.asda.android.app.data.RecoveryAppHelper.RecoveryAppData
import com.asda.android.app.view.CustomTypefaceSpan
import com.asda.android.base.core.constants.BottomNavConstants.TAG_ACCOUNT
import com.asda.android.base.core.constants.BottomNavConstants.TAG_FAVORITES
import com.asda.android.base.core.constants.BottomNavConstants.TAG_HOME
import com.asda.android.base.core.constants.BottomNavConstants.TAG_ORDERS
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SEARCH
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP_OLD
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.restapi.utils.SharedPreferencesUtil.getRecoveryAppUrl
import com.asda.android.utils.Utils
import com.asda.android.utils.view.FontManager.Companion.instance
import com.tooltip.Tooltip
import java.io.IOException
import java.net.URLEncoder

object ViewUtils {
    fun startActivityForResult(
        activity: Activity,
        intent: Intent?,
        requestCode: Int,
        fromView: View?
    ) {
        if (fromView != null) {
            val animOptions = ActivityOptionsCompat.makeScaleUpAnimation(
                fromView, 0, 0, fromView.width, fromView.height
            )
            activity.startActivityForResult(intent, requestCode, animOptions.toBundle())
        } else {
            activity.startActivityForResult(intent, requestCode)
        }
    }

    @JvmStatic
    fun startActivityForResult(
        fragment: Fragment,
        intent: Intent?,
        requestCode: Int,
        fromView: View?
    ) {
        if (fromView != null) {
            val animOptions = ActivityOptionsCompat.makeScaleUpAnimation(
                fromView, 0, 0, fromView.width, fromView.height
            )
            fragment.startActivityForResult(intent, requestCode, animOptions.toBundle())
        } else {
            fragment.startActivityForResult(intent, requestCode)
        }
    }

    @JvmStatic
    fun startActivity(activity: Activity, intent: Intent?, fromView: View?) {
        if (fromView != null) {
            val animOptions = ActivityOptionsCompat.makeScaleUpAnimation(
                fromView, 0, 0, fromView.width, fromView.height
            )
            activity.startActivity(intent, animOptions.toBundle())
        } else {
            activity.startActivity(intent)
        }
    }

    /**
     * Expands touch area for element clickableElem up to topContainer or max possible static parent.
     *
     * @param clickableElem a small clickable element
     * @param topContainer maximum allowed parent to set Touch delegate. May be null.
     */
    @JvmStatic
    fun expandTouchArea(clickableElem: View?, topContainer: ViewGroup) {
        if (clickableElem == null) {
            return
        }
        clickableElem.post(
            object : Runnable {
                override fun run() {
                    val parent = clickableElem.parent
                    if (parent is ViewGroup) {
                        val r = Rect()
                        clickableElem.getHitRect(r)
                        val expandSizePx = ViewUtil.dpToPixels(30, clickableElem.context)
                        r.top = r.top - expandSizePx
                        r.right = r.right + expandSizePx
                        r.left = r.left - expandSizePx
                        r.bottom = r.bottom + expandSizePx
                        addTouchDelegate(parent, r)
                        if (isValidTouchArea(r, parent, 0)) {
                            propagateTouchArea(parent, r)
                        }
                    }
                }

                private fun addTouchDelegate(parent: ViewGroup, r: Rect) {
                    val existingDelegate = parent.touchDelegate
                    if (isValid(existingDelegate)) {
                        parent.touchDelegate = object : AsdaTouchDelegate(r, clickableElem) {
                            override fun onTouchEvent(event: MotionEvent): Boolean {
                                return (
                                    clickableElem.visibility == View.VISIBLE &&
                                        (existingDelegate.onTouchEvent(event) || super.onTouchEvent(event))
                                    )
                            }
                        }
                    } else {
                        parent.touchDelegate = TouchDelegate(r, clickableElem)
                    }
                }

                private fun propagateTouchArea(parent: View, touchArea: Rect) {
                    val grandParent = parent.parent
                    if (!isValidTouchArea(topContainer, parent, grandParent)) {
                        return
                    }
                    val r = Rect()
                    parent.getHitRect(r)
                    r.top += touchArea.top
                    r.left += touchArea.left
                    r.right += touchArea.right - parent.width
                    r.bottom += touchArea.bottom - parent.height
                    addTouchDelegate(grandParent as ViewGroup, r)
                    if (isValidTouchArea(
                            r, grandParent, grandParent.width
                        )
                    ) {
                        propagateTouchArea(grandParent, r)
                    }
                }
            })
    }

    private fun isValid(existingDelegate: TouchDelegate?): Boolean {
        return existingDelegate != null && existingDelegate !is AsdaTouchDelegate
    }

    private fun isValidTouchArea(r: Rect, parent: ViewGroup, width: Int): Boolean {
        return r.top < 0 || r.bottom > parent.height || r.left < 0 || r.right > width
    }

    private fun isValidTouchArea(
        topContainer: ViewGroup,
        parent: View,
        grandParent: ViewParent
    ): Boolean {
        return (
            topContainer !== parent && topContainer !== grandParent && grandParent is ViewGroup &&
                grandParent !is ListView &&
                grandParent !is ScrollView
            )
    }

    fun applyAsdaFont(textView: TextView?) {
        if (textView != null) {
            instance?.getFont("Montserrat-SemiBold.otf")?.let {
                textView.typeface = it
            }
        }
    }

    fun applyAsdaFont(textView: TextView?, start: Int, end: Int, flags: Int) {
        if (textView != null && textView.text is Spannable) {
            instance?.getFont("Montserrat-SemiBold.otf")?.let {
                val spannable = textView.text as Spannable
                spannable.setSpan(
                    CustomTypefaceSpan(it),
                    start,
                    end,
                    flags
                )
            }
        }
    }

    @JvmStatic
    fun showTooltip(
        context: Context?,
        view: View?,
        tooltipText: String?
    ) {
        if (context != null && view != null && !TextUtils.isEmpty(tooltipText)) {
            Tooltip.Builder(view)
                .setGravity(Gravity.BOTTOM)
                .setBackgroundColor(ContextCompat.getColor(context, R.color.black))
                .setTextColor(ContextCompat.getColor(context, R.color.white))
                .setText(tooltipText)
                .setCancelable(true)
                .show()
            view.announceForAccessibility(tooltipText)
        }
    }

    @JvmStatic
    fun openRecoverySurvey(
        context: Context?,
        data: RecoveryAppData,
        screenName: String?
    ) {
        var url: String? = (
            getRecoveryAppUrl(context) +
                "&entity_id=" +
                data.orderId +
                "&session_data="
            )
        try {
            url += URLEncoder.encode(JsonMapper.get().writeValueAsString(data), "UTF-8")
        } catch (e: IOException) {
            Log.e("RestUtils", "Unable to write RecoveryAppData as String")
        }
        com.asda.android.base.core.view.ViewUtil.openWebPage(url, context, screenName)
    }

    @JvmStatic
    fun getTagFromId(itemId: Int, context: Context): String {
        return when (itemId) {
            R.id.navigate_home -> TAG_HOME
            R.id.navigate_shop -> {
                if (AsdaBaseCoreConfig.featureSettingManager.isNewTaxonomyEnabled(context)) {
                    TAG_SHOP
                } else {
                    TAG_SHOP_OLD
                }
            }
            R.id.navigate_favorites -> TAG_FAVORITES
            R.id.navigate_orders -> TAG_ORDERS
            R.id.navigate_account -> TAG_ACCOUNT
            R.id.navigate_search -> TAG_SEARCH
            else -> TAG_HOME
        }
    }

    @Volatile
    private var appVersionCode = -1
    fun getApplicationVersion(activity: Context): Int {
        return if (appVersionCode >= 0) {
            appVersionCode
        } else try {
            val info = activity.packageManager.getPackageInfo(activity.packageName, 0)
            appVersionCode = info.versionCode
            info.versionCode
        } catch (ex: Exception) {
            -1
        }
    }

    @JvmStatic
    @JvmOverloads
    fun handleCaughtException(e: Exception, context: Context?, showDialog: Boolean = true) {
        Tracker.get().trackException(CaughtExceptionEvent(e))
        Log.e("CaughtException", "Caught exception", e)
        if (Utils.isDevBuild && showDialog) {
            DialogHelper.displayErrorDialog(
                "[DEBUG]: Oops! Something strange just happened. Please report this in JIRA and with as much detail as possible write down what happened before you saw this dialog. " +
                    e,
                context,
                null
            )
        }
    }

    @JvmStatic
    fun fromHtml(source: String?): Spanned {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            Html.fromHtml(source, Html.FROM_HTML_MODE_LEGACY)
        } else {
            Html.fromHtml(source)
        }
    }

    private open class AsdaTouchDelegate(bounds: Rect?, delegateView: View?) : TouchDelegate(bounds, delegateView)
}
